/*
**  Module    : ancdef.h
**  Version   : $Revision: 1.0 $
**  Date      : $Date: 2001/03/01$
**  Authors   : Zoltan Zsoka
**
**  Synopsis  : this module collects definitions
**              referred to by more than one module of ANCLES.
**
**  History   : 01-MAR-03 - Creation of the module.
**              01-APR-23 - Insert the features for ASON:
**                          ASON call generators and duplex fibres - ZZ -
*/
# ifndef        ANCDEF_H
#define         ANCDEF_H

/***************************/
/* Compile defined options */
/***************************/

/*
** Define this if you want to use the
** well-known castelli scenarios!! :-)
** #define CASTELLI
*/

/*
** Define this if you want ANCLES to print a lot of debugging informations
** about the load balancing implementation
** #define DEBUG_BALANCING
*/

/***************************
**  Constant Definitions  **
***************************/

# ifndef TRUE
#define  TRUE         (unsigned char)1
# endif

# ifndef FALSE
#define  FALSE        (unsigned char)0
# endif

# ifndef CLOSE_CALL
#define  CLOSE_CALL    1
# endif

# ifndef OPEN_CALL
#define  OPEN_CALL     2
# endif

/* different level of report debug enabled by '-i' switch     */
enum
{
  ND_DEBUG_LEVEL          = 0x01, /* n */
  ND_TRAFFIC_DEBUG_LEVEL  = 0x02, /* t */
  STATISTICAL_DEBUG_LEVEL = 0x04, /* s */
  ROUTING_DEBUG_LEVEL     = 0x08, /* r */
  DEBUG_MEAN_PATH_LENGTH  = 0x10  /* m */
};

#define ALL_DEBUG_LEVEL (ND_DEBUG_LEVEL | ND_TRAFFIC_DEBUG_LEVEL |\
			 STATISTICAL_DEBUG_LEVEL | ROUTING_DEBUG_LEVEL |\
			 DEBUG_MEAN_PATH_LENGTH)

# ifndef DBL_CMP_THS            /* Relative difference between two      */
#define  DBL_CMP_THS    1.0E-10 /* 'double' values to be exceeded to    */
# endif                         /* declare them as 'not equal'.         */

enum
{
  NOT_ANALYSED,
  SIGNIFICANT,
  NOT_SIGNIFICANT
};

# ifndef MAX_SIGNIF_VAR
#define  MAX_SIGNIF_VAR  100 /* Maximum number of significant processes.  */
#endif

# ifndef MAX_NODE
#define  MAX_NODE     32   /* Maximum number of network nodes.          */
# endif

# ifndef MAX_LINK
#define  MAX_LINK     640  /* Maximum number of network links.          */
# endif

# ifndef MAX_PORT
#define  MAX_PORT     32  /* Maximum number of node output ports.       */
# endif

# ifndef MAX_USER
#define  MAX_USER     10  /* Maximum number of users for each node.     */
# endif

# ifndef MAX_NET_USER
#define  MAX_NET_USER   MAX_USER * MAX_NODE     /* Maximum number of    */
# endif                                         /* users in the network.*/

# ifndef MAX_CALL_GEN
#define  MAX_CALL_GEN   MAX_NET_USER * 1        /* Maximum number of    */
# endif                                         /* call generators.     */

# ifndef MAX_ROUTE
#define  MAX_ROUTE     100						/* Maximum number of	*/
# endif											/* predefined routes	*/

# ifndef MAX_PLENGTH
#define  MAX_PLENGTH     1e10				/* Maximum length of path	 */
# endif

# ifndef PRIMARY_LINK_LOAD
#define  PRIMARY_LINK_LOAD MAX_CALL_GEN    /* Used to store the Primary  */
#endif                                     /* link load                  */

# ifndef PRIMARY_ASON_LINK_LOAD
#define  PRIMARY_ASON_LINK_LOAD  0         /* Used to store the Primary  */
#endif                                     /* ason link load  (waves)    */

# ifndef TOTAL_LINK_LOAD                   /* Used to store the Total    */
#define  TOTAL_LINK_LOAD PRIMARY_LINK_LOAD +1
#endif                                     /* link load                  */

# ifndef TOTAL_ASON_LINK_LOAD              /* Used to store the Total    */
#define  TOTAL_ASON_LINK_LOAD PRIMARY_ASON_LINK_LOAD +1
#endif                                     /* ason link load (waves)     */

# ifndef PRECISION
#define  PRECISION      1e-7    /* Precision of the data in the Traffic */
# endif                         /* Relation File: 1e-6 -> 6 digit.      */

# ifndef DEFAULT_SEED           /* Default value for the shared           */
#define  DEFAULT_SEED   101268L /* pseudo-random seed.(HAPPY B'DAY MICHELA*/
# endif

# ifndef NOT_USED_ID
#define  NOT_USED_ID  INT_MAX
# endif

# ifndef VERSION
#define  VERSION     "Gancles - 1.0.10"
# endif

# ifndef MSF_MAX_PATH		/* Maximal number of paths used in the MSF*/
#define MSF_MAX_PATH	16	/* algirithms */
# endif

# ifndef MAX_COST
#define MAX_COST	10000	/* Used in the DLCP functions */
# endif

# ifndef EPSILON
#define EPSILON		1.0	/* Epsilon value in the convergence of	*/
# endif				/* the Ucall close events occouring time*/

# ifndef LEN_IDX_MAX		
#define  LEN_IDX_MAX     50	/* Ucall length histogram array size	*/
# endif

# ifndef MAX_WVOCTETS
#define  MAX_WVOCTETS    128	/* Max number of bytes in the mask for */
# endif							/* wavelength occupation on the links. */

# ifndef MAX_FIBER_NUM
#define  MAX_FIBER_NUM    1024  /* Max number of fibers in a link */
# endif

# ifndef NEW_MEASURE
#define  NEW_MEASURE
# endif

/* Definition to choose the selected routing algorithm         */
enum
{
  IN_ORDER_OF_HOP_NUMBER,
  IN_ORDER_OF_LENGTH,
  BALANCED_IN_ORDER_OF_HOP_NUMBER,
  BALANCED_IN_ORDER_OF_LENGTH,

  /* The number of possible routing path to choose from. Actually 4:
     in order of hop or in order of lenght, with balancing or not. */
  NO_OF_PATHS	
};

enum {
				/* THe kind of load balancing alg. */
  FIRST_NO_OF_HOP,		/* to use: actually 3:             */
                                /* assign first all the paths with */
				/* less hop.                       */

				/* assign a path to each user-user */
  FIRST_NO_OF_EQUIV_PATH,	/* relation.                       */

				/* assign the primary path for all */
  MODIFIED_FIRST_NO_OF_HOP     /* the user-user relation, and then*/
				/* assign the alternate paths by hop*/
};

/* Definitions used to choose the number of secondary paths.    */
enum
{
  FIXED_PATH_NUMBER,
  ALL_PATHS_NUMBER,
  MAX_HOP_PATH_NUMBER,
  MAX_HOP_MORE_PATH_NUMBER,
  MAX_LENGTH_PATH_NUMBER,
  MAX_LENGTH_MORE_PATH_NUMBER,
  ONLY_PRIMARY_PATH
};

/* Definitions identifying the available routing algorithms.            */
enum routing_algorithm
{
  ROUTING_RA1 = 1,	/* Single Path Routing  */
  ROUTING_RA2,		/* Controlled Alternate Routing  */
  ROUTING_RA3,		/* Uncontrolled Alternate Routing  */
  ROUTING_RA4,		/* Minimum Cost Routing */
  ROUTING_RA5,          /* Load-dependent Routing */
  ROUTING_RA6,          /* Shortest-Widest Routing */
  ROUTING_RA7,          /* Minimum-distance Routing */
  ROUTING_RA8,          /* Multimetric Sequential Filtering Routing */
  ROUTING_RA9,          /* Periodical Algorithm Choice (MD/SP) Routing*/
  ROUTING_RA10,         /* MDist Congestion Detection a priori Routing */
  ROUTING_RA11,         /* MDist Congestion Detection a posteori Routing */
  ROUTING_RA12,         /* MCost Congestion Detection a priori Routing */
  ROUTING_RA50  = 50,	/* Only primary path ason routing */
  ROUTING_RA51,		   /* Shortest-lowest ason routing */
  ROUTING_RA52,		   /* Shortest-widest path ason routing */
  ROUTING_RA53,		   /* Alternate shortest path ason routing */
  ROUTING_RA60  = 60,	/* Only primary path ason routing with random WA*/
  ROUTING_RA62	= 62,	/* Shortest-widest path ason routing with random WA*/
  ROUTING_RA63,		   /* Alternate shortest path ason routing with random WA*/
  ROUTING_RA70  = 70,	/* Only primary path-PAIR ason routing (1+1 protection) */
  ROUTING_RA73  = 73,	/* Alternate primary path-PAIR ason routing (1+1 protection) */
  ROUTING_RA74,			/* Only primary path-PAIR ason routing (1+1 protection) with */
					/* equal wavelength on the working and protection channel	 */
  ROUTING_RA75,			/* Alternate primary path-PAIR ason routing (1+1 protection) */
					/* with equal wavelength on the working and protection channel */
  ROUTING_RA76,			/* Only primary path-PAIR ason routing (1+1 protection) */
					/* with best effort protection possibility	*/
  ROUTING_RA77,			/* Alternate primary path-PAIR ason routing (1+1 protection) */
					/* with best effort protection possibility	*/
  ROUTING_RA78,			/* Only primary path-PAIR ason routing (1+1 protection) with */
					/* equal wavelength on the working and protection channel	 */
					/* and with best effort protection possibility	*/
  ROUTING_RA79,			/* Alternate primary path-PAIR ason routing (1+1 protection) */
					/* with equal wavelength on the working and protection channel */
					/* and with best effort protection possibility	*/
  ROUTING_RA80  = 80,	/* Only primary path-PAIR ason routing (n:1 protection) */
  ROUTING_RA83  = 83,	/* Alternate primary path-PAIR ason routing (n:1 protection) */
  ROUTING_RA87  = 87,	/* Alternate primary path-PAIR ason routing (n:1 protection) */
						/* with best effort protection possibility	*/

  ROUTING_RA90 = 90,	/* multiple fieber + routing_RA60 */ 
  ROUTING_RA92 = 92,	/* multiple fieber + routing_RA62 */
  ROUTING_RA93,	/* multiple fieber + routing_RA63 */
  ROUTING_RA120 = 120,	/* Fixed shortest 1+1 pair routing + random wa */
  ROUTING_RA123 = 123,	/* Alternate shortest 1+1 pair routing + random wa */
  ROUTING_RA124,	/* Fixed shortest 1+1 pair with ewl routing + random wa */
  ROUTING_RA125,	/* Alternate shortest 1+1 pair with ewl routing + random wa */
  ROUTING_RA126,	/* Best effort fixed shortest 1+1 pair routing + random wa */
  ROUTING_RA127,	/* Best effort alternate shortest 1+1 pair routing + random wa */
  ROUTING_RA128,	/* Best effort fixed shortest 1+1 pair with ewl routing + random wa*/
  ROUTING_RA129,	/* Best effort alternate shortest 1+1 pair with ewl routing + random wa */
  ROUTING_RA133 = 133,	/* Alternate shortest shared pair routing + random wa */
  ROUTING_RA137 = 137,	/* Best effort alternate shortest 1+1 pair routing + random wa */

  ROUTING_RA200 = 200,	/* Controlled DLCP routing */
  ROUTING_RA201		   /* Uncontrolled DLCP routing */

};

/* Define which RAs are static */
#define IS_STATIC_ROUTING(x) (x <= ROUTING_RA3)

/* Define which RAs are dynamic */
#define IS_DYNAMIC_ROUTING(x) (x >= ROUTING_RA4)

/* Definitions identifying the available CAC algorithms.                */
enum call_adm_ctrl
{
  CALL_ADM_CTRL1 = 1,	/* Equivalent Bandwidth Allocation.  */
  CALL_ADM_CTRL2,	/* Peak rate allocation over real time traffic
			   estimation based on statitiscal parameter */

  CALL_ADM_CTRL3,	/* Peak Rate Allocation over Real Time Traffic
			   Estimation based on aggregated statistical
			   parameters.*/

  CALL_ADM_CTRL4,	/* Mixed Traffic CAC. */

  CALL_ADM_CTRL5	/* ASON Traffic Simple CAC. */
};

/* Constant definitions for the Call Generator types.                   */
/* Arrivals: Poisson.                   */
/* Holding time distribution: negexp.   */

enum cg_type
{
  CG_TYPE1 = 1,			/* CBR calls.  */
				/* Constant Bit Rate distr.: uniform.  */

  CG_TYPE2,			/* On-Off VBR calls.  */
				/* Peak Rate distribution: uniform.  */
                                /* 'On' and 'Off' Time distr.: negexp.  */

  CG_TYPE3,			/* Uniform VBR calls.  */
				/* Bit rate distribution: uniform */

  CG_TYPE4,			/* Video source VBR calls.  */
				/* Lower bit rate distr.: uniform */
                                /* Medium bit rate distr.: uniform */
                                /* Higher bit rate distr.: uniform */

  CG_TYPE5,			/* Class U DBR calls.  */
				/* Maximum bit rate distr.: uniform */

  CG_TYPE6,			/* ASON type calls.  */
				/* Bit rate distr.: discrete (now deterministically 1)*/
                                /* The rate in this case means the */
                                /* number of required wavelegths   */

  CG_TYPE7,			/* On-Off ASON calls.  */
				/* Bit rate distr.: deterministically 1 wave */

  CG_TYPE8,			/* Never closing ASON calls.  */
				/* Bit rate distr.: deterministically 1 */

  CG_TYPE9,			/* Pascal (Z<1) ASON calls.  */
				/* Bit rate distr.: deterministically 1 */

  CG_TYPE10 = 10,		/* Receiver for calls generated by */
				/* CG_TYPE1, CG_TYPE2, CG_TYPE3 and */
				/* CG_TYPE4 call generators.  */
				/* Properly speaking, this is not a */
				/* call generator.  */

  CG_TYPE11,			/* Receiver for calls generated by */
				/* CG_TYPE5 (Class U) call generators.	*/
				/* Properly speaking, this is not a */
				/* call generator.  */

  CG_TYPE12			/* Receiver for calls generated by CG_TYPE6 (ASON) */
				/* CG_TYPE7, CG_TYPE8 and CG_TYPE9 call generators.	*/
				/* Properly speaking, this is not a */
				/* call generator.  */
};

/* Constant definitions for the Call types.  */
enum call_type
{
  CALL_TYPE1 = 1,       /* CBR call.  */
			/* Parameter: Constant Bit Rate.  */


  CALL_TYPE2,		/* On-Off VBR call.  */
			/* Parameters: Peak Rate, Minimum */
			/* Holding Time, Maximum Holding Time.  */

  CALL_TYPE3,		/* Uniform VBR call.  */
			/* Parameters: Minimum Bit Rate, */
			/* Maximum Bit Rate.  */

  CALL_TYPE4,		/* Video source VBR call.  */
			/* Parameters: Lowest Bit Rate, Medium */
			/* Bit Rate, Highest Bit Rate, Lowest, */
			/* Medium and Highest Bit Rate */
			/* Probability.  */

  CALL_TYPE5,		/* Class U DBR call.  */
			/* Parameter: Maximum Bit Rate.  */

  CALL_TYPE6,		/* ASON call.  */
			/* Parameters: Required number of wavelengths */

  CALL_TYPE8 = 8	/* ASON call or virtual IP link call.  */
			/* Parameters: Required number of wavelengths, 
			   realized virtual link(s)*/
};

/* Constant definitions for the Event types.  */
enum
{
  OPENING_CALL_REQUEST, /* Call opening request Event.  */
  CLOSING_CALL,         /* Call closure Event.  */
  LOAD_STATE_CHANGE,    /* Change CG load state */
  OFFICIAL_FBW_UPDATE,	/* Updating official free bw of a link	*/
  COMPUTE_AGGREGATE_COSTS, /* Computes the aggregate costs (DLCP) */
  UPDATE_ALG		/* Updates the algorithms for the sd pairs */
};


/* Definitions identifying the available virt link opener algorithms.*/
enum vl_open
{
  DIRECT_VLINK = 1,	/* Tries to open a direct link between endpoints 
					if the exisisting virtual topology is not enough.*/
  OCH_FIRST,		/* First try to open optical channel.	*/
  EHC_GROOM,		/* Similar to direct_vlink, but with extended check	*/
  EFF_VLINK			/* Tries to open link to connect the endpoints 
					if the exisisting virtual topology is not enough.*/
};

/* Definitions identifying the available virt link closer algorithms.*/
enum vl_close
{
  EMPTY_CLOSE = 1	/* Close virtual link if it is empty for a while. */
};

/* Definitions identifying the virtual topologies to precreate.*/
enum pcv_topology
{
  NONE = 1,			/* No precreated topology */
  PHYSICAL,			/* Topology identical to the physical one */
  SPANTREE,			/* Passing to a minimal spanning tree in optics */
  FULLMESH			/* Full connection in the data-level */
};

# ifndef MAX_THRESHOLD          /* Max number of allowed threshold on   */
#define MAX_THRESHOLD   4       /* links.                               */
# endif

# ifndef SIBAL_THRESHOLD           /* Constant to identify when the Sibal  */
#define SIBAL_THRESHOLD -0.9011971 /* DeSimone threshold is to be used     */
# endif

# ifndef UNDEFINED_BW           /* The mean bandwidth of Type 5 calls 	*/
#define  UNDEFINED_BW   -1.0    /* cannot be defined: the value defined */
# endif                         /* here allows the Type 5 calls to be	*/
				/* identified immediately.		*/

/* Constant Definition allowing the conversion of the elapsed CPU time  */
/* apart from the included version of the header file 'time.h'.         */
# ifndef CLOCKS_PER_SEC
#define  CLOCKS_PER_SEC         CLK_TCK
# endif


/* All the variables preceded by 'DECL' are allocated in the module     */
/* containing the definition of 'MAIN'.                                 */
/* The same variables are simply declared as 'extern' in all the other  */
/* modules.                                                             */
# ifdef  MAIN
#define  DECL
# else
#define  DECL extern
# endif

# endif /* ANCDEF_H */
