 /*
**  Module    : ancstr.h
**  Version   : $Revision: 1.0 $
**  Date      : $Date: 2001/03/01$
**  Authors   : Zoltan Zsoka
**
**  Synopsis  : this module collects definitions of strutctures
**              referred to by more than one module of ANCLES.
**
**  History   : 01-MAR-03 - Creation of the module.
**              01-APR-23 - Insert the features for ASON:
**                          ASON call generators and duplex fibres - ZZ -
*/
# ifndef        ANCSTR_H
#define         ANCSTR_H

#include "ancdef.h"

/***********************
**  Type Definitions  **
***********************/

typedef unsigned char           Boolean;

typedef double                  TimeUnit;

typedef union   alter_unit      Alter;
typedef union   descr_unit      Descr;
typedef union   descr_unit_2    Descr_2;

typedef struct  node_unit       Node;
typedef struct  link_unit       Link;
typedef struct  linklist_unit   LinkList;
typedef struct  path_unit       Path;
typedef struct  pair_unit       Pair;
typedef struct  routing_unit    Routing;
typedef struct  choice_unit     Choice;
typedef struct  user_unit       User;
typedef struct  callgen_unit    CallGen;
typedef struct  call_unit       Call;
typedef struct  call_list_unit  CallList;
typedef struct  vcset_unit      VCset;
typedef struct  photo_attrib    PHattr;
typedef struct  route_unit      Route;

typedef struct  cg_type1_par    CGType1;
typedef struct  cg_type2_par    CGType2;
typedef struct  cg_type3_par    CGType3;
typedef struct  cg_type4_par    CGType4;
typedef struct  cg_type5_par    CGType5;
typedef struct  cg_type6_par    CGType6;
typedef struct  cg_type7_par    CGType7;
typedef struct  cg_type8_par    CGType8;
typedef struct  cg_type9_par    CGType9;
/* No parameters are required to describe call generators referring to	*/
/* CG_TYPE10, CG_TYPE11 and CG_TYPE12.						*/

typedef struct  call_type1_par  CallType1;
typedef struct  call_type2_par  CallType2;
typedef struct  call_type3_par  CallType3;
typedef struct  call_type4_par  CallType4;
typedef struct  call_type5_par  CallType5;
typedef struct  call_type6_par  CallType6;
typedef struct  call_type8_par  CallType8;

typedef struct  stack_unit      Stack;
typedef struct  event_unit      Event;
typedef struct  stat_var_unit   Stat_var;
typedef struct  link_ucall_dat  Link_ucall_stat;

typedef struct  R_info_unit		R_info;
typedef struct	SD_pair_unit	SD_pair;

typedef struct  protch_unit     ProtCh;
typedef struct  intlist_unit	Intlist;
typedef struct  dbllist_unit	Dbllist;


/************************
**  Union Definitions  **
************************/

/*
**  Union     : alter_unit
**  Remarks   : union used to describe the available types of path number
**              choice.
*/
union alter_unit
 {
  int          ival;
  double       dval;
 };


/*
**  Union     : descr_unit
**  Remarks   : union collecting auxiliary information about the event.
*/
union descr_unit
 {
  int       source_cgen_id;   /* Identifier of the source call generator */
                              /* used by an OPENING_CALL_REQUEST Event.  */
  Call      *deleting_call;   /* Pointer to the Call to be closed used   */
                              /* by a CLOSING_CALL Event.                */
  int       link_id; 	      /* Identifier of the link, whose official  */
                              /* free bandwidth will be updated.	 */
			      /* Also used for dynamic observation	 */
 };


/*
**  Union     : descr_unit_2
**  Remarks   : union collecting secondary information about the event.
*/
union descr_unit_2
 {
  int       dest_cgen_id;     /* Identifier of the destination call generator */
                              /* used by an OPENING_CALL_REQUEST Event.  */
 };

/****************************
**  Structure Definitions  **
****************************/

/*
**  Structure : node_unit
**  Remarks   : collection of node attributes.
*/
struct node_unit
 {
  int      id;          /* Node identifier.                             */
  int      no_of_ports; /* Number of available output ports.            */
  int      no_of_users; /* Number of connected users.                   */
  int      connected_user[MAX_USER];
                        /* Array of connected users' identifiers.       */
  int      connected_node[MAX_PORT];
                        /* Array of connected nodes' identifiers.       */
  double   alpha;       /* Weighting coefficient for DLCP algorithm.    */
  double   beta;        /* Weighting coefficient for DLCP algorithm.    */
  double   aggregate_cost; /* Used by DLCP cost function.               */
  Boolean  opaque;		/* Indicates the opaque functionality of the	*/
						/* node if an ASON call has to be routed.		*/
  Boolean  unreachable;	/* Indicates if the node is fallen out  (no 	*/
						/* node-node links are collegated)				*/
  Link 	   *from_user[MAX_USER]; /* Array of users' incoming links.     */
  Link     *to_user[MAX_USER];   /* Array of users' outgoing links.     */
  Link     *inqueue[MAX_PORT];   /* Array of links from other nodes.    */
  Link     *outqueue[MAX_PORT];  /* Array of links to other nodes.      */
 };


/*
**  Structure : link_unit
**  Remarks   : collection of link attributes.
*/
struct link_unit
 {
  int      id;          /* Link identifier.                             */
  int      source;      /* Link source (node or user).                  */
  int      dest;        /* Link destination (node or user).             */
  double   timescale;   /* Timescale of the link.                       */
  double   free_band;   /* Available link bandwidth                     */
  TimeUnit bw_update_p; /* Update period time of the official bandwidth */
  double   link_len;    /* Link physical length (km).                   */
  double   routing_threshold[MAX_THRESHOLD];
                        /* The threshold used during the call admission.*/
  CallList *calls_on;   /* Pointer to the set of active calls currently */
                        /* routed over the link.                        */

  Boolean  duplex_optic;/* If the link is a duplex fibre. By this type  */
                        /* the bandwidth is not taken in consideration. */
  Boolean  substituted;	/* Indicates if the link has to be substituted	*/
						/* by another section (section protection)		*/
  int      max_waves;   /* The number of the wavelengths on this fibre. */
  int      used_waves;  /* The number of the occupied wavelengths.      */
  int      peak_wave;   /* The actually highest occupied wavelength.    */
  unsigned char free_waves[MAX_WVOCTETS];
                        /* The aviable wavelengths on this link stored	*/
                        /* as a bitwise mask array. 1 -> free wave      */
  int	   *free_fibers;/* Array of the number of fibers with free		*/
						/* channel of the indiced wavelenght.			*/
  int	   fiber_num;	/* Number of fibers on this link.				*/
  Boolean  is_node_node;/* if the link is node-node then TRUE			*/
  int      nnid;        /* Identifier in the node-bode linklists.       */
  Call	   *linked_call;/* If IP over WDM is enabled, the IP links may  */
						/* linked to a type8 call.						*/
 };


/*
**  Structure : linklist_unit
**  Remarks   : this structure is used to build the paths as linked lists of
**              'Link' pointers.
*/
struct linklist_unit
 {
  Link     *link_elem;  /* Pointer to the link.                         */
  int	   direction;	/* Direction of the link in the path.			*/
  Boolean  endopsec;	/* The end-node of the link can be the end
							of an opaque section. */
  LinkList *next;       /* Pointer to the next item of the linked list. */
 };


/*
**  Structure : choice_unit
**  Remarks   : 'type' refers to the operating criterion for ordering the
**              paths between each pair of nodes; 'value' completes the
**              definition of the ordering criterion.
*/
struct choice_unit
 {
  int           type;
  Alter         value;
 };


/*
**  Structure : path_unit
**  Remarks   : collection of path attributes.
*/
struct path_unit
 {
  Path     *next;       /* Pointer to the next path, ordered according  */
                        /* to the minimum-hop or minimum-length         */
                        /* criterion.                                   */
  LinkList *path;       /* Pointer to the sequence of links forming     */
                        /* the path.                                    */
  int      no_of_hops;  /* Number of path hops.                         */
  double   length;      /* Total path length.                           */
  int	   no_of_opaque_crs; /* Number of opaque section.				*/

/*  CallList *prot_calls_on;   * Pointer to the set of calls currently	*/
                        /* protected by a channel on this path.			*/

  int      num_of_prot_chs;  /* Number of the open protection channels. */
  ProtCh   *prot_channels;	/* List of the open protection channels.	*/
  int	   *protected_links; /* Number of those calls on the nn-link,	*/
						/* which are protected by this path.(nnid index)*/
  int	   *protected_nodes; /* Number of those calls crossing the node,*/
						/* which are protected by this path. (id index)	*/
 };


/*
**  Structure : pair_unit
**  Remarks   : collection of independent pathpair attributes.
*/
struct pair_unit
 {
  Pair     *next;       /* Pointer to the next pair, ordered according  */
                        /* to the minimum-hop or minimum-length         */
                        /* criterion.                                   */
  Path	   *path1;      /* Pointer to the first path.                   */
  Path	   *path2;      /* Pointer to the second path.                  */
 };


/*
**  Structure : routing_unit
**  Remarks   : collection of routing attributes referring to the CG.
*/
struct routing_unit
 {
  enum routing_algorithm selected_routing;
				 /* Identifier for the selected Routing  */
                                 /* algorithm.                           */
  int       path_order_type;     /* Path ordering criterion.             */
  Choice    selected_range_type;
                                 /* Further information about the        */
                                 /* criterion used to order the paths.   */
  Boolean   use_hop_by_hop;	 /* Select hop by hop strategy. 	 */

  Boolean   use_crankback;	 /* Select crankback mode.		 */

  int       primary_threshold;   /* The threshold index used by the CG   */
                                 /* in the primary path links.           */
  int       alternate_threshold; /* The threshold index used by the CG   */
                                 /* in the alternate path links.         */
 };


/*
**  Structure : user_unit
**  Remarks   : collection of user attributes.
**              Users are referred to as collections of call generators,
**              providing them the physical location into the network.
**
*/
struct user_unit
 {
  int       id;                 /* User identifier.                     */
  int       connected_node;     /* Identifier of the connected node.    */
  CallGen   *generator;			/* Pointer to the generator of the user.*/
  Link      *out_link;          /* Output link to the connected node.   */
  Link      *in_link;           /* Input Link from the connected node.  */
 };


/*
**  Structure : callgen_unit
**  Remarks   : collection of call generator attributes.
*/
struct callgen_unit
 {
  int       id;         /* Call Generator identifier.                   */
  enum      cg_type type;    /* Call Generator type.                    */
  Routing   choosen_routing; /* Call Generator choosen routing struct   */
  void      *info;      /* Pointer to the peculiar information about    */
                        /* the call generator.                          */
  User      *ref_user;  /* Pointer to the owner user.                   */
  Boolean  is_nonstationary; /* Flag for non-stationary generators      */
  double   load_reduction_factor; /* Load reduction in low-load periods */
  TimeUnit normal_load_duration; /* duration of normal load periods     */
  TimeUnit low_load_duration;    /* duration of low load periods        */
  Boolean  is_low;          /* Boolean flag defining CG state           */
 };


/*
**  Structure : call_unit
**  Remarks   : collection of call attributes.
*/
struct call_unit
 {
  enum call_type call_type; /* Call type identifier.                    */
  void          *info;      /* Pointer to the record describing the     */
                            /* call instance.                           */
  int           source_id;  /* Call source identifier.                  */
  int           dest_id;    /* Call destination identifier.             */
  int           curr_path_no;
                            /* The number of the link that carries the  */
                            /* call relative to the ordered list of     */
                            /* avaiable paths.                          */
  TimeUnit      start_time; /* Time of call starting.					*/
  TimeUnit      end_time;   /* Scheduled call ending time.              */
  Event			*my_close_event;
                            /* Pointer to the event which will close	*/
							/* this call.				*/
  Path          *path_ptr;  /* Pointer to the covered path.             */
  Path			*prot_path_ptr;	/* Pointer to the protection path.		*/
  ProtCh		*dedic_prot_ch; /* Protection channel dedicated to call */
  Call          *pred;      /* Pointer to the previous item in the      */
                            /* General Call List.                       */
  Call          *next;      /* Pointer to the next item in the          */
                            /* General Call List.                       */
  Boolean		repl;	    /* TRUE if the call should be replaced at 	*/
  			    /* every link change. (no warning more)	*/
  int           no_ch_link;	/* Checked link number during the routing	*/
 };


/*
**  Structure : call_list_unit
**  Remarks   : this structure is used to list the calls that are
**              currently sharing the link capacity.
**              The calls in the list are ordered according to their
**              scheduled releasing time.
*/
struct call_list_unit
 {
  Call          *call_elem;  /* Pointer to the corresponding record in  */
                             /* the General Call List.                  */
  double        ref_bw;      /* Reference bandwidth value for the call  */
                             /* over the considered link.               */
  int           used_wave;   /* The wavelength used by the call on the  */
                             /* link.                                   */
  CallList      *pred;       /* Pointer to the previous item in the     */
                             /* Local Call List.                        */
  CallList      *next;       /* Pointer to the next item in the Local   */
                             /* Call List.                              */
 };

/*
**  Structure : route_unit
**  Remarks   : collection of route attributes.
*/
struct route_unit
 {
  int      id;          /* Route identifier.                            */
  int      from;		/* Id of the starting node						*/
  int      to;			/* Id of the ending node						*/
  Path     *routepath;  /* Node-node path according to the route def.   */
 };

/*
**  Structure : vcset_unit
**  Remarks   : this structure is used to aggregate connections having the
**              same parameters and routed over the same path when a
**              network state snapshot is taken.
**              An array of lists is used to collect the VC sets: each list
**              in the array refers to a source user.
*/
struct vcset_unit
 {
  Call          *call_elem;  /* Pointer to the call referred to as the  */
                             /* prototype of the VC set.                */
  long          no_vc;       /* Number of VCs in the set.               */
  VCset         *next;       /* Next item in the list.                  */
 };

/*
**  Structure : photo_attrib
**  Remarks   : this structure is used to store information about congestion
**              level and congestion duration when a snapshot is taken. It
**              also contains the filename where such information is downloaded
**
*/
struct photo_attrib
 {
  double    	load_level;           /* congestion level             	    */
  double        congestion_duration;  /* congestion duration                */
  char          filename[50];         /* name of file where snapshot info   */
				      /* is stored			    */
  PHattr        *next;                /* pointer to next photo attributes   */
 				      /* the list			    */
 };

/*
**  Structure : cg_type1_par
**  Remarks   : collection of attributes referring to the call generators of
**              type 'CG_TYPE1'.
**              A 'CG_TYPE1' Call Generator provides CBR calls whose
**              constant bit rates are obtained from a uniform distribution
**              having 'min_bit_rate' and 'max_bit_rate' as lower and upper
**              bounds.
**              If 'min_bit_rate' equals 'max_bit_rate' the Constant Bit
**              Rate is deterministically assigned.
**              Calls arrive in a Poisson fashion, having exponential
**              holding times.
*/
struct cg_type1_par
 {

  double min_bit_rate;          /* Minimum Bit Rate for the CBR calls.  */
  double max_bit_rate;          /* Maximum Bit Rate for the CBR calls.  */
  double arrival_rate;          /* Arrival Process Rate.                */
  double mean_hold_time;        /* Mean Holding Time of a call.         */
 };


/*
**  Structure : call_type1_par
**  Remarks   : this record is used to provide the instantaneous load
**              contribution of a CBR call originated by a 'CG_TYPE1' call
**              generator.
**              The only parameter considered here is the constant bit rate
**              of the call.
**              The 'next' field is used in order to manage the 'CallType1'
**              free list.
*/
struct call_type1_par
 {
  double        cbr_value;
  CallType1     *next;
 };


/*
**  Structure : cg_type2_par
**  Remarks   : collection of attributes referring to the call generators of
**              type 'CG_TYPE2'.
**              'CG_TYPE2' Call Generators provide VBR On-Off calls whose
**              peak rates are obtained from a uniform distribution having
**              'min_peak_rate' and 'max_peak_rate' as lower and upper bounds.
**              If 'min_peak_rate' equals 'max_peak_rate' the Peak Rate
**              is deterministically assigned.
**              Calls arrive in a Poisson fashion, having exponential
**              holding times.
**              The call parameters fixed by the user are mean 'on' time and
**              mean 'off' time ('on' time and 'off' time are both
**              exponentially distributed).
*/
struct cg_type2_par
 {
  double        min_peak_rate;  /* Minimum Call Peak Rate.              */
  double        max_peak_rate;  /* Maximum Call Peak Rate.              */
  double        mean_on_time;   /* Mean 'On' Time of the source.        */
  double        mean_off_time;  /* Mean 'Off' time of the source.       */
  double        arrival_rate;   /* Arrival Process Rate.                */
  double        mean_hold_time; /* Mean Holding Time of a Call.         */
 };


/*
**  Structure : call_type2_par
**  Remarks   : this record is used to provide the instantaneous load
**              contribution of an On-Off VBR call originated by a 'CG_TYPE2'
**              call generator.
**              The 'next' field is used in order to manage the 'CallType2'
**              freelist.
*/
struct call_type2_par
 {
  double        peak_rate;      /* Call Peak Rate.                      */
  double        mean_on_time;   /* Mean 'On' Time.                      */
  double        mean_off_time;  /* Mean 'Off' Time.                     */
  Boolean       on_status;      /* True if the call is in the ON status.*/
  int    	cgen_id;        /* Index of the rows of the P_xx matrix */
  CallType2     *next;          /* Pointer to the next item in the      */
                                /* 'CallType2' freelist.                */
 };


/*
**  Structure : cg_type3_par
**  Remarks   : collection of attributes referring to the call generators of
**              type 'CG_TYPE3'.
**              A 'CG_TYPE3' Call Generator provides VBR calls whose real
**              time bit rates are obtained from a uniform distribution.
**              The limits of the pdf are extracted from two uniform
**              distributions.
**              Calls arrive in a Poisson fashion, having exponential
**              holding times.
*/
struct cg_type3_par
 {
  double min_bit_rate_avg;      /* Average Minimum Bit Rate for the VBR */
                                /* calls.                               */
  double max_bit_rate_avg;      /* Average Maximum Bit Rate for the VBR */
                                /* calls.                               */
  double half_bw_interval;      /* Halfwidth of the uniform distribution*/
                                /* used to decide the bit rate value.   */
  double arrival_rate;          /* Arrival Process Rate.                */
  double mean_hold_time;        /* Mean Holding Time of a call.         */
 };


/*
**  Structure : call_type3_par
**  Remarks   : this record is used to provide the instantaneous load
**              contribution of a VBR call originated by a 'CG_TYPE3' call
**              generator.
**              The parameters considered here are the minimum and the
**              maximum bit rate of the call.
**              The 'next' field is used in order to manage the 'CallType3'
**              free list.
*/
struct call_type3_par
 {
  double        min_vbr_value;
  double        max_vbr_value;
  CallType3     *next;
 };


/*
**  Structure : cg_type4_par
**  Remarks   : collection of attributes referring to the call generators of
**              type 'CG_TYPE4'.
**              A 'CG_TYPE4' Call Generator provides VBR calls whose model
**              is concerned with a wide range of video sources: from
**              videophones to studio television; they have a relatively
**              stable behaviour most of the time and intermittent period
**              with a substantially higher bit rate requirement.
**              The position of each of the three Dirac delta of the pdf
**              is decided through a uniform distribution.
**              If 'half_bw_interval' equals 0 the position of the three
**              Dirac delta of the Bit Rate pdf equals its average value.
**              Calls arrive in a Poisson fashion, having exponential
**              holding times.
*/
struct cg_type4_par
 {
  double arrival_rate;            /* Arrival Process Rate.                */
  double mean_hold_time;          /* Mean Holding Time of a call.         */
  double higher_bit_rate_avg;     /* Average value of the higher bit rate.*/
  double medium_bit_rate_avg;     /* Average value of the medium bit rate.*/
  double lower_bit_rate_avg;      /* Average value of the lower bit rate. */
  double half_bw_interval;        /* Halfwidth of the uniform distribution*/
                                  /* used to decide the value of bit rate.*/
  double higher_bit_rate_prob;    /* Height of the higher bit rate Dirac  */
                                  /* delta.                               */
  double medium_bit_rate_prob;    /* Height of the medium bit rate Dirac  */
                                  /* delta.                               */
  double lower_bit_rate_prob;     /* Height of the lower bit rate Dirac   */
                                  /* delta.                               */
 };


/*
**  Structure : call_type4_par
**  Remarks   : this record is used to provide the instantaneous load
**              contribution of a VBR call originated by a 'CG_TYPE4' call
**              generator.
**              The parameters considered here are the three different
**              bit rates of the call and their probabilities.
**              The 'next' field is used in order to manage the 'CallType4'
**              free list.
*/
struct call_type4_par
 {
  double        higher_bit_rate_value;
  double        medium_bit_rate_value;
  double        lower_bit_rate_value;
  double        higher_bit_rate_prob;
  double        medium_bit_rate_prob;
  double        lower_bit_rate_prob;
  CallType4     *next;
 };


/*
**  Structure : cg_type5_par
**  Remarks   : collection of attributes referring to the call generators of
**              type 'CG_TYPE5'.
**              A 'CG_TYPE5' Call Generator provides DBR calls whose
**              maximum bit rates are obtained from a uniform distribution
**              having 'min_max_rate' and 'max_max_rate' as lower and upper
**              bounds.
**              If 'min_max_rate' equals 'max_max_rate' the Maximum Bit
**              Rate is deterministically assigned.
**              Calls arrive in a Poisson fashion, having exponential
**              holding times.
*/
struct cg_type5_par
 {
  double min_max_rate;          /* Minimum value of the Maximum Bit     */
                                /* Rate for the Class U DBR calls.      */
  double max_max_rate;          /* Maximum value of the Maximum Bit     */
                                /* Rate for the Class U DBR calls.      */
  double arrival_rate;          /* Arrival Process Rate.                */
  double mean_gen_traffic;      /* Mean Generated Traffic.         	*/
  double min_accept_bw_thrs;	/* Threshold for the call interruption.	*/
  double min_acceptable_bw;	/* Min value for the call interruption.	*/
  TimeUnit longest_call;	/* Length of the longest call (histogr).*/
  double vlopen_ubw_thrs;	/* Threshold for the vlink opening.	*/
 };


/*
**  Structure : call_type5_par
**  Remarks   : this record is used to provide the instantaneous load
**              contribution of a Class U DBR call originated by a
**              'CG_TYPE5' call generator.
*/
struct call_type5_par
 {
  double        last_load_time; /* Absolute simulation time when the    */
                                /* call acceptance occurred or when     */
                                /* the statistics about gen_*Uload are  */
                                /* reset.                               */
  double        mbr_value;      /* Maximum Bit Rate requested by        */
                                /* the call.                            */
  double        curr_br;        /* Current Bit Rate of the call.        */
  double        last_updt_time; /* Absolute simulation time when the    */
                                /* last Bit Rate arrangement occurred.  */
  double        br_integ;       /* Bit Rate integral.                   */
  double	tr_left;	/* Amount of the traffic left to 	*/
				/* transfer (at the last_updt_time)	*/
  int		class;		/* Classification by req traffic amount	*/
  Boolean       bott_state;     /* flag to keep track of the bottleneck */
                                /* updating process                     */
  CallType5     *next;          /* Pointer to the next record in the    */
                                /* 'CallType5' free list.               */
 };


/*
**  Structure : cg_type6_par
**  Remarks   : collection of attributes referring to the call generators of
**              type 'CG_TYPE6'.
**              A 'CG_TYPE6' Call Generator provides ASON calls whose
**              constant rate is measured in wavelengths.
**              This type of call can be routed on duplex_optic links too
**              Calls arrive in a Poisson fashion, having
**              exponential holding times.
*/
struct cg_type6_par
 {

  int min_no_of_waves;          /* Min number of needed wavelengths calls.  */
  int max_no_of_waves;          /* Max number of needed wavelengths calls.  */
  double arrival_rate;          /* Arrival Process Rate.                */
  double mean_hold_time;        /* Mean Holding Time of a call.         */
 };


/*
**  Structure : call_type6_par
**  Remarks   : this record is used to provide the instantaneous load
**              contribution of an ASON call originated by a 'CG_TYPE6',
**              'CG_TYPE7' or 'CG_TYPE9' call generator.
**              The parameters considered here are: the number of used
**              wavelengths of the call.
**              The 'next' field is used in order to manage the 'CallType6'
**              free list.
*/
struct call_type6_par
 {
  int           nwaves;
  CallType6     *next;
 };


/*
**  Structure : cg_type7_par
**  Remarks   : collection of attributes referring to the call generators of
**              type 'CG_TYPE7'.
**              A 'CG_TYPE7' Call Generator provides On-Off ASON calls whose
**              constant rate is measured in wavelengths.
**              This type of call can be routed on duplex_optic links too
**              Calls arrive in ON-OFF fashion, having
**              exponential holding times.
*/
struct cg_type7_par
 {

  double saturation;		  /* Saturation of the call.*/
  int no_of_waves;              /* Number of all needed wavelengths calls.  */
  double mean_hold_time;        /* Mean Holding Time of a call.         */
  double arrival_rate;          /* Arrival Process Rate.                */
 };


/*
**  Structure : cg_type8_par
**  Remarks   : collection of attributes referring to the call generators of
**              type 'CG_TYPE8'.
**              A 'CG_TYPE8' Call Generator provides Never closing ASON calls whose
**              constant rate is measured in wavelengths.
**              This type of call can be routed on duplex_optic links too
**              Calls arrive before the start of the simulation, and they never close.
*/
struct cg_type8_par
 {

  int no_of_waves;          /* Number of needed wavelengths calls.  */
 };

/*
**  Structure : call_type8_par
**  Remarks   : this record is used to provide the instantaneous load
**              contribution of an ASON call originated by a 'CG_TYPE8',
**              call generator.
**              The parameters considered here are: the number of used
**              wavelengths of the call, pointer(s) on the link(s) realized
**				by this optical channel.
**              The 'next' field is used in order to manage the 'CallType8'
**              free list.
*/
struct call_type8_par
 {
  int           nwaves;
  Link			*fwd_link; /* direction: equal to that of call */
  Link			*bwd_link; /* direction: opposite to that of call */
  CallType8     *next;
 };

/*
**  Structure : cg_type9_par
**  Remarks   : collection of attributes referring to the call generators of
**              type 'CG_TYPE9'.
**              A 'CG_TYPE9' Call Generator provides ASON calls whose
**              constant rate is measured in wavelengths.
**              This type of call can be routed on duplex_optic links too
**              Calls arrive in a Pascal distribution (Z<1) fashion, having
**              exponential holding times.
*/
struct cg_type9_par
 {

  int min_no_of_waves;          /* Min number of needed wavelengths calls.  */
  int max_no_of_waves;          /* Max number of needed wavelengths calls.  */
  double arrival_rate;          /* Arrival Process Rate.					*/
  double mean_hold_time;        /* Mean Holding Time of a call.				*/
  double arr_rate_increment;    /* Incrementation of arrival rate. (lambda)	*/
  double peakedness;			/* Peakedness of the Pascal distribution	*/
/*  int num_of_calls;				* Number of active calls generated by cg9.	*/
  Dbllist *call_closures;		/* Ordered linked list of the end-times		*/
/*  Boolean sch_at_end;			* Indicator: schedule next arrival at the	*/
								/* end of the next finishing call			*/
 };


/*
**  Structure : link_ucall_dat
**  Remarks   : this record is used to manage the calls originating from
**              call generators of type 5
*/
struct link_ucall_dat
 {
  double        start_time;     /* Set-up time for the statistical 	*/
				/* indices.           			*/
  double        last_updt_time; /* Absolute simulation time when the    */
                                /* last variation of the number of      */
                                /* calls (both Class 1/2 and Class U)	*/
				/* occurred.                      	*/
  double        free_Uband;     /* Free bandwidth for calls of Class U  */
                                /* (Type 5).                            */
  double   	official_free_Uband;
				/* Official available link bandwidth,   */
				/* Official -> the value at the last 	*/
				/* update.				*/
  double        fb_integ;       /* Free bandwidth integral.             */
  double        last_unus_time; /* Absolute simulation time when the    */
                                /* last variation of the unused free    */
                                /* bandwidth occurred.                  */
  double        unused_free_band;
                                /* Total unused free bandwidth.         */
  double        un_fb_integ;    /* Unused free bandwidth integral.      */
  Boolean   	official_cong;  /* Official information: congested?	*/
  int           no_of_Ucall;    /* Number of calls of Class U that are  */
                                /* currently using the link.            */
  double        no_of_Ucall_integ;
				/* Integral of the previous.            */
  double   	official_no_of_Ucall;
				/* Official number of U Class calls	*/
  double        estim_no_of_Ucall;
  				/* Estimated number of calls, calculated*/
				/* from the estim. traffic matrix	*/
  Boolean       is_node_node;   /* True if connecting two nodes         */
  double        acc_PCR_1_2;    /* PCR accumulator for the calls        */
                                /* belonging to Class 1 and 2.          */
  double        acc_PCR_U;      /* PCR accumulator for the calls        */
                                /* belonging to Class U.                */
  double	U_PCR_int;	/* Integral of the Peak Cell Rate of the*/
				/* Class U calls routed over the link.	*/
  double	last_Uupdt_time;/* Absolute simulation time when the 	*/
				/* last variation of the number of 	*/
				/* Class U calls occurred.		*/
  CallList      *U_calls_on;    /* Pointer to the set of calls of       */
                                /* Class U that are currently routed    */
                                /* over the link.                       */
  SD_pair	*sp_on;		/* Linked list of the src-dst pairs, in */
  				/* which SP this link is present	*/
 };


/*
**  Structure : stack_unit
**  Remarks   : this structure is needed by the path detection algorithm.
*/
struct stack_unit
 {
  int      node_id;     /* Node identifier.                             */
  int      port_number; /* Identifier of the currently considered       */
                        /* output port.                                 */
  int	   direction;	/* Direction of the link in the path: 0 means	*/
						/* normal (source->destination)					*/
  Stack    *next;       /* Pointer to the record previously on the top  */
                        /* of the stack.                                */
 };


/*
**  Structure : event_unit
**  Remarks   : collection of event attributes.
*/
struct event_unit
 {
  TimeUnit      ev_time;        /* Occurrence time of the event.        */
  Event         *pred;          /* Pointer to the previous record       */
                                /* in the Event List.                   */
  Event         *next;          /* Pointer to the next record           */
                                /* in the Event List.                   */
  char          ev_type;        /* Type of event.                       */
  Descr         ev_dscrpt;      /* Union collecting the event           */
                                /* description.                         */
  Descr_2       ev_dscrpt_2;    /* Union collecting the secondary       */
                                /* event description.                   */
 };


/*
**  Structure : stat_var_unit
**  Remarks   : collection of statistical variables attributes.
*/
struct stat_var_unit
 {
  int           info;           /* Informative field about the way of   */
                                /* analysing the variable.              */
  int           number;         /* Number used to address the           */
                                /* statistically analysed variable.     */
 };

/*
**  Structure : R_info_unit
**  Remarks   : infos about the routing between callgenerator s-d pairs.
*/
struct R_info_unit
 {
  int		actual_callnum;	/* The instant number of U calls	*/
  Boolean	bn_bw_ready;	/* The estim. bottleneck bw is ready	*/
  double	Bw_SP;		/* Bandwidth of the SP or RSP path	*/
  double	Bw_MD;		/* Bandwidth of the MD path		*/
  double	mu_SP;		/* Mean of the RSP path bottleneck	*/
  double	mu_MD;		/* Mean of the MD path bottleneck	*/
  Boolean	sprou;		/* Using the random shortest algorithm, */
				/* if false -> minimum distance		*/
  Boolean	pac;		/* If the pair uses the PAC routing	*/
  Path		*spath;		/* Pointer to the shortest path		*/
 };

/*
**  Structure : SD_pair_unit
**  Remarks   : a pair of source and destination with pointer to the next
*/
struct SD_pair_unit
 {
  int		src;
  int		dst;
  SD_pair	*next;
 };

/*
**  Structure : intlist_unit
**  Remarks   : linked list element of integer value list
*/
struct intlist_unit
 {
  int		value;
  Intlist	*next;
 };

/*
**  Structure : dbllist_unit
**  Remarks   : linked list element of double value list
*/
struct dbllist_unit
 {
  double	value;
  Dbllist	*next;
 };

/*
**  Structure : protch_unit
**  Remarks   : linked list element of protection channel description
*/
struct protch_unit
 {
  Intlist	*opsec_waves;
  ProtCh	*next;
 };

# endif /* ANCSTR_H */
