/*
**  Module    : ancvar.h
**  Version   : $Revision: 1.0 $
**  Date      : $Date: 2001/03/01$
**  Authors   : Zoltan Zsoka
**
**  Synopsis  : this module collects declarations of variables
**              referred to by more than one module of ANCLES.
**
**  History   : 01-MAR-03 - Creation of the module.
**              01-APR-23 - Insert the features for ASON:
**                          ASON call generators and duplex fibres - ZZ -
*/
# ifndef        ANCVAR_H
#define         ANCVAR_H

#include "ancdef.h"
#include "ancstr.h"


/**********************************
**  Global Variable Definitions  **
**********************************/

/* Network informations. */
DECL  double    capacity;       /* Capacity in Mbit/s of links with     */
                                /* Timescale = 1.                       */
DECL  double    load;           /* Offered relative load.               */
DECL  double    load_conv_fact; /* Factor used to convert the normalized*/
                                /* source user load contributions into  */
                                /* absolute load contributions.         */
DECL  double    wavelength_capacity;    /* Capacity of an ason-wave     */
DECL  double    bw_measure_uncertainty; /* Uncertainty in the measure	*/
				/* of number of Ucalls, in percent	*/

DECL  int		xxx_max_no_of_hops;	/* Global absolute hop nuber	*/
										/* constraint for paths */
DECL  int		xxx_more_no_of_hops;	/* Global relative hop nuber	*/
										/* constraint for paths */
				
/* Variables referring to the available routing algorithms.		*/
DECL  double    hop_weight;
DECL  double    length_weight;
DECL  double    congestion_weight;
DECL  double    ra5_min_efficiency;
DECL  double    cong_thrsh;	/* threshold for congestion on link	*/

/* For the statistics about the choices of algorithm in the PAC routing */
DECL  double	ac_period;	/* period of the algorithm choice	*/
DECL  double    sample_deviation; /* exponential fading of samples	*/
DECL  double    sample_deviation_parameter; /* value as given in the nd	*/
DECL  double	alg_decision_thrsh; /* force the one alg. in the choice	*/
DECL  double	histeresys_thrsh; /* hysteresys in the choice		*/
DECL  R_info    **pair_routing_info; /* useful infos about the generator pair */

/* DECL  long	RSPstate[MAX_CALL_GEN][MAX_CALL_GEN]; Not used.		*/
/* DECL  long	MDstate[MAX_CALL_GEN][MAX_CALL_GEN];  Not used.		*/

/* DLCP routing */
DECL  double    widest_bottleneck_range;
DECL  double	recomputation_slot_duration;
DECL  Boolean   use_hop_by_hop;
DECL  Boolean   use_crankback;

/* Variables referring to the available CAC algorithms.                 */
/* Type 1 CAC.                                                          */
DECL  double    cell_loss_rate; /* Cell Loss Rate used as reference in  */
                                /* the evaluation of the Equivalent     */
                                /* Bandwidth of both incoming calls and */
                                /* available call generators.           */

/* Type 2 and 3 CAC.                                                    */
DECL  int       no_load_samples;/* Number of load samples used to       */
                                /* estimate the traffic currently       */
                                /* carried by the links.                */
DECL  int       meas_wind_size; /* Number of link slots forming an      */
                                /* interval for the estimation of the   */
                                /* carried load.                        */
DECL  double    *load_samples_vect[MAX_LINK];   /* Array of pointers to */
                                /* the load contribution accumulators.  */
DECL  double    backoff_ths[MAX_LINK];          /* Array of thresholds  */
                                /* used to decide whether the CAC       */
                                /* algorithm has to be applied to the   */
                                /* incoming call.                       */
DECL  double    k_star;         /* Coefficient weighting the standard   */
                                /* deviation of the load samples.       */
DECL  double    prot_fact;      /* Protection factor applied to the     */
                                /* link capacity when a CAC of type 2   */
                                /* or 3 is running.                     */
DECL  double    meas_rel_error; /* Relative half-width of the uniform   */
                                /* distribution providing the errors on */
                                /* the call rate measures.              */

/* Type 3 CAC.                                                          */
DECL  double    *link_mean;     /* Pointer to the array collecting the  */
                                /* accumulators for the means of the    */
                                /* load contributions, link by link.    */
DECL  double    *link_variance; /* Pointer to the array collecting the  */
                                /* accumulators for the variances of the*/
                                /* load contributions, link by link.    */

/* Type 4 CAC.                                                          */
DECL Boolean    UCall_present;  /* Flag used to know if Ucall generator */
                                /* are present                          */
DECL Link_ucall_stat  *link_Ucall_stat[MAX_LINK];
                                /* Pointer to the array of records      */
                                /* involved in the management of Ucalls.*/
DECL  double    alpha;          /* Protection factor for Class 1/2 	*/
                                /* calls in CAC 4.                      */
DECL  double    beta;           /* Protection factor for Class U 	*/
                                /* calls in CAC 4. 			*/
				/* It can be greater than one.		*/
DECL  double    Gamma;          /* Modification factor for non-CBR 	*/
                                /* calls in CAC 4. 			*/

/* Variables referring to the creation of network state photographs.    */
DECL  double    rel_load_ths;   /* Relative Load Threshold used to      */
                                /* decide whether a new snapshot of the */
                                /* network state has to be taken.       */
DECL  double    cong_duration;	/* Congestion Duration Time used to     */
                                /* decide whether a new snapshot of the */
                                /* network state has to be taken.       */
				/* It is expressed in seconds.		*/
DECL  int       max_state_photos;
				/* Number of network state 'photos' 	*/
				/* that can be still provided. 		*/
DECL  int    	min_ucall_present;
				/* Number of Ucall present on a link to */
				/* allow ANCLES to take a photo of the  */
				/* net status				*/

DECL Boolean    nonASON_Call_present;  /* Flag used to know if there	*/
								/* any non ASON generatora (CG1-CG5)	*/

/* Variables carrying topological information.                          */
DECL  Path  * ***path_mat;
                                /* Pointers to the paths connecting     */
                                /* each pair of call generators.        */
DECL  Pair  * ***pair_mat;
                                /* Pointers to the independent pairs    */
                                /* of paths between the call generators.*/
DECL  Path  * **virt_nn_mat;
                                /* Inordered pathlist connecting each	*/
                                /* node pair of the logical topology.   */
DECL  Node      *node_vect[MAX_NODE];
                                /* Array of pointers to the nodes of    */
                                /* the network.                         */
DECL  Link      *link_vect[MAX_LINK];
                                /* Array of pointers to the links of    */
                                /* the network.                         */
DECL  int	nnlink_vect[MAX_LINK];
                                /* Array of link_vect indexes of the    */
                                /* node-node links of the network.      */
DECL  User      *user_vect[MAX_NET_USER];
                                /* Array of pointers to the users of    */
                                /* the network.                         */
DECL  CallGen   *callgen_vect[MAX_CALL_GEN];
                                /* Array of pointers to the call        */
                                /* generators of the network.           */
DECL  Route     **route_vect;
                                /* Array of pointers to the predefined	*/
								/* routes								*/

DECL  int       no_of_nodes;    /* Number of network nodes.             */
DECL  int       no_of_links;    /* Number of network links.             */
DECL  int       no_virt_links;  /* Number of virtual IP links.          */
DECL  int       no_of_nnlinks;  /* Number of node-node network links.   */
DECL  int       no_of_users;    /* Number of network users.             */
DECL  int       no_of_cgens;    /* Number of network call generators.   */
DECL  int       no_of_routes;   /* Number of predefined routes.			*/

DECL  int       no_of_type1_cg; /* Number of TYPE1 call  generators.    */
DECL  int       no_of_type2_cg; /* Number of TYPE2 call  generators.    */
DECL  int       no_of_type3_cg; /* Number of TYPE3 call  generators.    */
DECL  int       no_of_type4_cg; /* Number of TYPE4 call  generators.    */
DECL  int       no_of_type5_cg; /* Number of TYPE5 call  generators.    */
DECL  int       no_of_type6_cg; /* Number of TYPE6 call  generators.    */
DECL  int       no_of_type7_cg; /* Number of TYPE7 call  generators.    */
DECL  int       no_of_type8_cg; /* Number of TYPE8 call  generators.    */
DECL  int       no_of_type9_cg; /* Number of TYPE9 call  generators.    */
DECL  int       no_of_type10_cg;/* Number of TYPE10 call generators.    */
DECL  int       no_of_type11_cg;/* Number of TYPE11 call generators.    */
DECL  int       no_of_type12_cg;/* Number of TYPE12 call generators.    */

DECL  int       max_wvoct_all_links;    /* Maximum number of octes of	*/
								/* wavelengths on all the links.		*/
DECL  double	max_link_capa;	/* Maximum of node-node link capacities	*/

DECL  long      cg_seed_vect[MAX_CALL_GEN];
                                /* Array of pseudo-random seeds (one    */
                                /* for each call generator).            */
DECL  long      sh_seed;        /* Shared pseudo-random seed.           */

DECL  int       range_of_equiv_hop;
                                /* The paths that differ one each other */
                                /* by less than this number of hop are  */
                                /* considered equivalent.               */
DECL  double    range_of_equiv_length;
                                /* The paths that differ one each other */
                                /* by less than this length are         */
                                /* considered quivalent.                */
DECL  int       load_balancing_algorithm;
                                /* The choice is between                */
                                /* first_no_of_equiv_path and           */
                                /* first_no_of_hop algorithm            */
DECL  Boolean   selected_non_U_priority;
                                /* If TRUE, balance first the non-U     */
                                /* class, then the class-U paths. Else  */
                                /* do it the same time.                 */
DECL  int       selected_cac;   /* Identifier for the selected Call     */
                                /* Admission Control algorithm.         */

DECL  int       no_of_thresholds;
                                /* The number of thresholds used during */
                                /* the simulation.                      */

/* Variables carrying temporal information.                             */
DECL  clock_t   start_CPU_time; /* CPU time elapsed at the beginning of */
                                /* the main simulation cycle.           */
DECL  clock_t   end_CPU_time;   /* CPU time elapsed at the end of the   */
                                /* simulation run.                      */
DECL  TimeUnit  ancles_time;    /* Absolute simulation time.            */
DECL  TimeUnit  max_time;       /* Maximum simulation time.             */
DECL  TimeUnit  sched_events_start_time;
                                /* Starting time of the steady state    */
                                /* phase for the evaluation of the mean */
                                /* number of events seen in the FES by  */
                                /* an event just scheduled.             */
DECL  TimeUnit  global_start_time[3];
                                /* Starting time of the steady state    */
                                /* simulation for global statistical    */
                                /* variables.                           */
DECL  TimeUnit  *user_start_time;
                                /* Starting time of the steady state    */
                                /* simulation for statistical variables */
                                /* referring to network users.          */
DECL  TimeUnit  *classU_user_start_time;
                                /* Starting time of the steady state    */
                                /* simulation for statistical variables */
                                /* referring to network users.          */
DECL  TimeUnit  *link_start_time;
                                /* Starting time of the steady state    */
                                /* simulation for statistical variables */
                                /* referring to network links.          */
DECL  TimeUnit  *ason_link_start_time;
                                /* Starting time of the steady state    */
                                /* simulation for statistical variables */
                                /* referring to network ason links.     */

/* Variables dealing with the management of the ANCLES lists.           */
DECL  Call      *head_of_call_list;
                /* Pointer to the empty head of the circular list       */
                /* collecting the currently active calls.               */
DECL  Event     *head_of_event_list;
                /* Pointer to the empty head of the Event List.         */
DECL  VCset     **head_of_VC_list;
                /* Pointers to the lists of VC sets associated to the   */
                /* source users.                                        */
DECL  PHattr    **photo_vect;
                /* Pointers to lists of snapshots associated to each    */
 				/* link.						*/

/* Variables referring to the evaluation of performance indices.        */
DECL  int       no_stat_vars;
                        /* Number of observed statistical variables.    */
DECL  int       no_sign_vars;
                        /* Number of significant statistical variables. */
DECL  double    **link_carried_load;
                        /* Load carried by the links: it is evaluated   */
                        /* for each link and as the sum of the average  */
                        /* bandwidths of the calls currently active on  */
                        /* the link [stored in TOTAL_LINK_LOAD position]*/
			               /* Added also the primary load stats.           */
DECL  double    **ason_link_carried_waves;
                        /* Load (waves) carried by the ason links       */
                        /* Only for the total and for the primary path! */
DECL  double    **ason_link_peak; /* Peak wave on the ason links        */
DECL  double    **ason_link_util; /* Utilization of the ason links      */
DECL  int       *no_times_this_path_is_utilized;
                        /* The histogram of the number of times the path  */
                        /* is used to carry the calls, i.e. call accepted */
                        /* in the first path, in the second one, int the  */
                        /* third one etc.                                 */
DECL  int       max_cgen_cgen_no_of_path;
                        /* max number of path from cgen to cgen, i.e.     */
                        /* the dimension of no_times_this_path_is_utilized*/
DECL  double    total_carried_load;
                        /* Load currently carried by the whole network. */
DECL  double    total_carried_waves;
                        /* Currently "carried" waves by the network.    */
DECL  Stat_var  global_stat_vars[3];
                        /* Information regarding the global variables:  */
                        /* 'total carried load', 'total call blocking   */
                        /* probability' and 'used wavelengths'.         */
DECL  Stat_var  link_stat_vars[MAX_LINK];
                        /* Information regarding the variable           */
                        /* 'carried load' for each link.                */
DECL  Stat_var  ason_link_stat_vars[MAX_LINK];
                        /* Information regarding the variable           */
                        /* 'used wavelengths' for each ason link.       */
DECL  Stat_var  user_stat_vars[MAX_NET_USER];
                        /* Information regarding the variable 'call     */
                        /* blocking probability' for each user.         */
DECL  Stat_var  classU_user_stat_vars[MAX_NET_USER];
                        /* Information regarding the variable 'assigned */
                        /* bw' for each class U user.                   */
DECL  char      *names_stat_vars[MAX_STAT_VAR];
                        /* Array collecting the names of the observed   */
                        /* statistical variables.                       */
DECL  Boolean  	global_trans_closed[2];
                        /* Flags notifying the closure of the initial   */
                        /* transient for the global statistical         */
                        /* variables.                                   */
DECL  Boolean  	user_trans_closed[MAX_NET_USER];
                        /* Flags notifying the closure of the initial   */
                        /* transient for the statistical variables      */
                        /* referring to the users.                      */
DECL  Boolean  	classU_user_trans_closed[MAX_NET_USER];
                        /* Flags notifying the closure of the initial   */
                        /* transient for the statistical variables      */
                        /* referring to the class U users.              */
DECL  Boolean  	link_trans_closed[MAX_LINK];
                        /* Flags notifying the closure of the initial   */
                        /* transient for the statistical variables      */
                        /* referring to the links.                      */
DECL  Boolean  	ason_link_trans_closed[MAX_LINK];
                        /* Flags notifying the closure of the initial   */
                        /* transient for the statistical variables      */
                        /* referring to the ason links.                 */

DECL  double    *hst_link_load[MAX_LINK];
                        /* Pointers to the arrays collecting data used  */
                        /* to build the link load histograms.           */
DECL  int       hst_intervals[MAX_LINK];
                        /* Number of intervals used to build the link   */
                        /* load histograms.                             */

DECL  Boolean	Ucall_is_present;/* Flag to check the presence of calls */
						/* belonging to Class U.						*/
DECL  double    *gen_finUload;/* Total carried class U load by finished */
						/*  calls for all user          			*/
DECL  double    *gen_finUtras;/* Total transported class U data by		*/
						/* finished calls for all user				*/
DECL  double    *gen_intUload;/* Total carried class U load by			*/
						/* interrupted calls for all user          		*/
DECL  double    *gen_intUtras; /* Total transported class U data by		*/
						/* interrupted calls for all user		*/
DECL  double    **classed_prop_intUtras; /* Accumulated proportional	*/
						/* information about the transported class U	*/
						/* data by interrupted calls for all user and	*/
						/* amount class		*/
DECL  TimeUnit  *gen_Uload_start_time;
                        /* Set up time for the Uload for each user      */

DECL  long      **req_calls;
                /* Number of requested connections per source user      */
                /* and destination user.                                */
DECL  long      **ref_calls;
                /* Number of refused connections per source user        */
                /* and destination user.                                */
DECL  long      **pbe_calls;
                /* Number of connections with protection - best effort	*/
                /* per source user and destination user.                */
DECL  long      ***classed_fin_calls;
                /* Number of competed connections per source user	*/
                /* and destination user classified by the req amount.   */
DECL  long      **int_calls;
                /* Number of interrupted connections per source user	*/
                /* and destination user.                                */
DECL  long      ***classed_int_calls;
                /* Number of interrupted connections per source user	*/
                /* and destination user classified by the req amount.   */
DECL  double	**fin_calls_dilacc;
				/* accumulator for counting the mean dilatation of the 	*/
				/* completed calls					*/
DECL  double	**usr_finUload;
				/* accumulator of the mean bandwidth for user-user pairs*/


DECL  int	**hopdist;
				/* distance in hop number of the cgens					*/
DECL  long      **path_hops_accu;
                /* Accumulator for the average hop number of paths      */
                /* between the source and destination user.             */
DECL  double    **path_length_accu;
                /* Accumulator for the average length of paths          */
                /* between the source and destination user.             */
DECL  long      **path_crop_accu;
                /* Accumulator for the average number of transit opaque */
                /* nodes in the path used between the users				*/

DECL  long      **prot_path_hops_accu;
                /* Accumulator for the average hop number of protective */
                /* path between the source and destination user.        */
DECL  double    **prot_path_length_accu;
                /* Accumulator for the average length of protective		*/
                /* path between the source and destination user.        */
DECL  long      **prot_path_crop_accu;
                /* Accumulator for the average number of transit opaque */
                /* nodes in the protective path used between the users.	*/

DECL  double    **inon_accumulator;
                /* Accumulator for the accepted call durations			*/

DECL  long      *primary_calls;
                /* Counter for the calls routed over primary paths.     */
DECL  long      *alternate_calls;
                /* Counter for the calls routed over alternate paths.   */
DECL  long      **U_fincall_dur;
                /* Counter for the length of finished calls	 	*/
                /* histogramm per source user.				*/
DECL  long      **U_intcall_dur;
                /* Counter for the length of interrupted calls		*/
                /* histogramm per source user.				*/

DECL  TimeUnit  **prev_load_update_time;
                /* Simulation Time in which the last load updating      */
                /* occurred for the referred link (user and total).     */
DECL  TimeUnit  **prev_ason_link_update_time;
                /* Simulation Time in which the last updating           */
                /* occurred for the referred ason link.                 */
DECL  TimeUnit  *ason_util_zero_time;
                /* Shows, how long was the wave utilization zero.        */
DECL  double    **load_accumulator;
                /* Accumulators used to evaluate the link loads.        */
DECL  double    **ason_wave_accumulator;
                /* Accumulators used to evaluate the ason link loads.   */
                /* Only for the total and for the primary path!         */
DECL  double    **ason_peak_accumulator;
                /* Accumulators used to evaluate the peaks of ason link.*/
DECL  double    **ason_util_accumulator;
                /* Accumulators used to eval. the ason link utilization.*/
DECL  double    **histo_ason_wave_accu;
                /* Accumulators used to calculate a histogram of loads  */
                /* on the ason links.									*/
DECL  double    **histo_ason_peak_accu;
                /* Accumulators used to calculate a histogram of peaks  */
                /* on the ason links.									*/
DECL  double	**free_asonfibers_accu;
                /* Accumulators of free fibers per each wavelength		*/
DECL  double	**minf_asonfibers;
                /* Minimal value of free fibers per each wavelength		*/
DECL  long		**inblock_asonwaves_accu;
                /* Accumulators of link loads at call blocking events   */
				/* per each wavelength									*/
DECL  int		*in_path_union;
                /* Shows the links' occurance in the examined paths		*/
DECL  long		**calls_on_link_wave;
                /* Counter of calls opened and using the given			*/
				/* wavelength on this link								*/
DECL  long		**acc_noch_link_accu;
                /* Accumulator of number of links checked during the	*/
				/* routing of an accepted call */
DECL  long		**ref_noch_link_accu;
                /* Accumulator of number of links checked during the	*/
				/* routing of a refused call */
DECL  double	**accu_vlink_lifetime;
                /* Accumulator of virtual links' lifetime,each userpair	*/
DECL  double	**sqraccu_vlink_lifetime;
                /* Accumulator of virtual links' squared lifetime		*/
DECL  long		**closed_vlinks;
                /* Number of (opened and) closed virtual links			*/
DECL  double	**accu_vlink_timehop;
                /* Number of hops * holding time of virtual links		*/
DECL  long		**rt_sp_changed;
                /* Number of times the shortest path in the routing		*/
				/* table was changed									*/
DECL  double	*last_vlreq_arrival;
                /* Time of the last arrival of vlink req (by src user)	*/
DECL  double	*accu_vlreq_iat;
                /* Accumulator of interarrival time of vlink requests	*/
DECL  double	*sqraccu_vlreq_iat;
                /* Accumulator of squared interarrival time of vl reqs	*/
DECL  long		*calls_in_at_reset;
                /* Number of opened calls at the end of transient		*/

/* ASON routing variables */

DECL  int*		tfree; /* Array of the free wavelengths in ason routing */
DECL  int		u_wave[MAX_NODE+2];	/* Used wavelengths on the links of */
				/* the path	by the last call. */
DECL  Path*		used_prot_path;		/* The protection path				*/
DECL  Boolean	new_prot_chan;		/* New protection channel had to be */
				/* opened												*/
DECL  Boolean	dedicated_protchan; /* The protection channel is		*/
				/* dedicated to the call (used in 1+1 protection)		*/
DECL  Boolean	protection_be;	/* If the protection could not function */
				/* but, as best effort, one path could be opened		*/

DECL  long      real_events_cnt;
                /* Counter for the realized events.                     */
DECL  long      sched_events_cnt;
                /* Counter for the scheduled events.                    */
DECL  long      no_of_FES_operations;
                /* Counter for the number of operations on the FES.     */
DECL  long      sched_events_acc;
                /* Accumulator for the number of scheduled events.      */
DECL  double    mean_sum_of_FES_events;
                /* Accumulator for the mean number of scheduled events. */
DECL  long      no_of_sum;
                /* Number of acquired means over the number of          */
                /* scheduled events.                                    */

/* Variables dealing with the traffic management.                       */
DECL  double    traff_pattern[MAX_NET_USER][MAX_NET_USER];
                        /* Traffic pattern matrix.                      */
DECL  double    source_prob[MAX_NET_USER];
                        /* Call generation probabilities.               */
DECL  double    destination_prob[MAX_NET_USER][MAX_NET_USER];
                        /* The (i,j) element of this matrix is the      */
                        /* probability that user j is the destination   */
                        /* of a call generated at user i.               */
DECL  double    *link_primary_load;
                        /* Array of primary load levels assigned to the */
                        /* links on the basis of the traffic pattern    */
                        /* matrix. The array is indexed by link         */
                        /* identifier.                                  */

/* Miscellanea.                                                         */
DECL FILE     *fp_statistics;   /* File collecting the statistical      */
                                /* reports (.STR).                      */
DECL char     file_radix[80];   /* String containing the radix of       */
                                /* all the output files.                */
DECL char     descr_file[80];   /* String containing the name of the    */
                                /* file .ND                             */
DECL int      init_debug_level; /* Trace the initialization phase       */
                                /* on screen. Different level:          */
                                /* NO_DEBUG_LEVEL -> no report          */
                                /* ND_DEBUG_LEVEL -> only nd report     */
                                /* STATISTICAL_DEBUG_LEVEL ->           */
                                /*       + statistical request report   */
                                /* ROUTING_DEBUG_LEVEL ->               */
                                /*                      + routing report*/
DECL Boolean  first_time;       /* Simulation time of first execution   */
                                /* for function 'reset' in ANCSTAT.C.   */
DECL Boolean  normal;           /* True if the execution is ended       */
                                /* normally, false if terminated by a   */
                                /* SIGTERM, SIGQUIT, SIGINT signal.     */
DECL Boolean  interrupt_happened;  /* If a call was found which has to  */
				/* be interrupted.			*/
DECL Boolean  time_based_ct5;	/* Shows if the duration of the best	*/
				/* effort calls is based on time or on	*/
				/* traffic amount.			*/
DECL Boolean  interrupt_statistics; /* Enables to write interrupt 	*/
				/* statistic files.			*/
DECL double   lhstep_factor;	/* Factor for calculate the steplength	*/
				/* of call-length histogram.		*/
DECL double   gen_load_red_factor; /* If general load reduction after	*/
								  /* the transient phase is used, this	*/
								  /* factor reduces the arrival rates.	*/
DECL int      subslinkid;		/* Id of the link which was set to be	*/
								/* substituable */
DECL Path*    subspaths;		/* List of node-node paths which can	*/
								/* substitute the subslinkid link		*/

DECL double** bino;				/* Binomial coefficients				*/
DECL double* facto;				/* Factorial values						*/

DECL Boolean  theo_analysis;	/* Theoretical analysis enabled			*/
DECL Boolean  list_routes;		/* Listing of routes enabled			*/

DECL FILE     *CG8FILE;			/* File collecting the scheduling times */
                                /* of permanent calls (*.CG8)           */

DECL FILE     *ROUFILE;			/* File collecting the lists of routes	*/
                                /* by from - to node (*.ROU)			*/

DECL double   virtlink_bup;		/* Bandwidth update period of IP links	*/
DECL Boolean  ipowdm_ena;		/* IP over WDM is enabled				*/
DECL int	vl_open_algorithm;	/* Strategy of the virtual link opening */
DECL int	vl_close_algorithm;	/* Strategy of the virtual link closing */
DECL int	precreav_topology;	/* Precreated virtual topology			*/
DECL double	vl_close_hold_time;	/* Used in timing of vl closing	event	*/
DECL int	open_loglink;		/* Indicator, performs opening of vl	*/
DECL Boolean  duplex_llc;		/* Duplex connections are used on the	*/
								/* lower level (phisical topology)		*/
DECL Boolean  cl_predef_vl;		/* Enable to close predefined vlinks	*/
DECL Boolean  vlht_hopdep;		/* Enable to depend vl_cl_time on hopnum*/

/*DECL double   vlopen_ubw_thrs;	* Ucall bandwidth threshold, indicates	*/
								/* opening a new virtual link */
DECL int	vlopen_max_hops;	/* Max allowed hops of route, indicates	*/
								/* opening a new virtual link (if over) */

DECL int	maxvirtpaths;		/* (target)maximal num of virtual paths */
DECL int	maxvirtnnpaths;		/* (target)max no: virt node-node paths */

DECL int no_of_free_linklist;

# endif /* ANCVAR_H */
