/*
**  Module    : bottleneck.c
**  Version   : $Revision: 3.2 $
**  Date      : $Date: 1999/01/11 14:53:50 $
**  Authors   : Marco Mellia, Claudio Casetti
**                                           
**  History   : 31.08.1999 - Revision : adding support of data amount based
**					Ucalls and thier interrupts  -ZZ-
**
30-01-2001 Cleaning... - ZZ -
*/

# ifndef        ANCLES_H
#include        "ancles.h"
# endif      

static double *B_link;
static int    *N_link;
static Boolean  curr_bottleneck_state;
static Call	*int_call_ptr;  /* pointer to the call,which has to
				   be interrupted, if any */
static Call	*repl_call_ptr; /* pointer to the call,which has to
				   be replaced, if any */
double min_repl_time;
int	oldno_virt_links; /* number of virtual links at the last
					  bottleneck calculation */

void init_bott_staticvars()
{
	B_link = NULL;
	N_link = NULL;
	oldno_virt_links = 0;
}

void init_struct_dyn()
{
   register int i;
   
   if (N_link == NULL || oldno_virt_links < no_virt_links)
     {
	   if(oldno_virt_links < no_virt_links)
	   {
		   oldno_virt_links = no_virt_links;
		   free(N_link);
		   free(B_link);
	   }
	   else
		   curr_bottleneck_state = TRUE;

	   N_link = (int *)MMmalloc(sizeof(int) * (no_of_links + no_virt_links),
		   "init_struct_dyn");
	   B_link = (double *)MMmalloc(sizeof(double) * (no_of_links + no_virt_links),
		   "init_struct_dyn");
     }
   
   for (i=0;i<no_of_links + no_virt_links;i++)
     {
	B_link[i]=link_Ucall_stat[i]->free_Uband;

	/* skip user-node and node-user links by setting N_link=0 */
	if (link_Ucall_stat[i]->is_node_node)
  	  N_link[i]=link_Ucall_stat[i]->no_of_Ucall;
	else
	  N_link[i]=0;
     }  
}

void evaluate_UBW(Call *this_call, int action)
{
   int l, bottleneck;
   int link_id;
   double min_bw;
   Boolean  more_calls=TRUE;
   CallList *curr_call;
   LinkList *curr_link;
   CallType5 *temp_call;
   double longest_dur = 0.0;
   
   min_repl_time = DBL_MAX;
   
   init_struct_dyn();
if(N_link[0] > 0)
N_link[0] = 0;
   while (more_calls)
     {
	min_bw = capacity + 1.0;
	/* look for bottleneck link over all links, except for */
	/* user-node and node-user links and for those links   */
	/* carrying no U calls                                 */
        bottleneck = -1;
	for (l=0;l<no_of_links + no_virt_links;l++)
	  {
	     if (N_link[l] == 0)  continue;
	     if (B_link[l]/(double)N_link[l] < min_bw)
	       {		  
		  min_bw = B_link[l]/(double)N_link[l];
if(min_bw < 0.0)
bottleneck=0;
		  bottleneck = l;
	       }
	  }
	/* if no suitable U calls were found in the previous cycle, */
	/* exit the while cycle                                     */
	if (min_bw > capacity)  
	  more_calls = FALSE;
	else	
	  {
	     for(curr_call = link_Ucall_stat[bottleneck]->U_calls_on->next;
	         curr_call != link_Ucall_stat[bottleneck]->U_calls_on;
		 curr_call = curr_call->next)
	       {
	          temp_call = ((CallType5 *)(curr_call->call_elem->info));
	/* If the bw for the call is not determined yet */
		  if (temp_call->bott_state != curr_bottleneck_state)
                  {  /* evaluate the new curr_br for this call */
		     if ( temp_call->mbr_value > min_bw)		    
		       temp_call->curr_br = min_bw;
		     else
		       temp_call->curr_br = temp_call->mbr_value;
		       
	/* If the call bw would be too small and dures long-long time, interrupt it immediatly. */
		     if(temp_call->curr_br < ((CGType5 *)(callgen_vect[curr_call->call_elem->
		          source_id]->info))->min_acceptable_bw && 
			 (temp_call->tr_left/temp_call->curr_br > longest_dur))
		      { 
	/* If we are closing a call, or even opened one (in the same moment)
	   don't allow to choose it as interruptable! */
		       if((curr_call->call_elem != this_call || action != CLOSE_CALL) &&
		           temp_call->last_load_time != ancles_time)
			{
			 interrupt_happened = TRUE;
	/* Interrupt the call, which would dure the longest time */
			 longest_dur = temp_call->tr_left/temp_call->curr_br;
			 int_call_ptr = (Call *)curr_call->call_elem;
			}
		      }	

	/* From the replaceable calls we choose one, which could be replaced */
		     if(curr_call->call_elem->repl)
		      { 
	/* We replace only if this is a closing action, choose one, that is not the same
	   that the even closing one and should be replaced to the earliest timepoint */
		       if(curr_call->call_elem != this_call && action == CLOSE_CALL &&
		           temp_call->tr_left/temp_call->curr_br < min_repl_time)
			{
			 min_repl_time = temp_call->tr_left/temp_call->curr_br;
			 repl_call_ptr = curr_call->call_elem;
			}
		      }	

		     temp_call->bott_state = curr_bottleneck_state;
		     for(curr_link = curr_call->call_elem->path_ptr->path;
		         curr_link != NULL; curr_link = curr_link->next)
		       {
		        link_id = curr_link->link_elem->id;
 		        B_link[link_id] -= temp_call->curr_br;
		        /* skip the user-node and node-user links */
		        if (link_Ucall_stat[link_id]->is_node_node)
			     N_link[link_id]--;
		       }
                  }
	       } /* for calls */
	  } /* if more calls */
     }  /*  while more calls */ 
}
			 
void update_bottleneck_measure(Call * this_call, double mean_bw, 
			       double peak_bw, int action)
{
   register int l;
   Link_ucall_stat *this_stat;
   LinkList  *this_link;
   Link  *link_ptr;
   Call *call_ptr;
   CallType5 *temp_call;
   CallList *call_list_ptr;
   double  old_time;

   /* We're going to freeze the situation on every link */
   /* prior to the computation of the new bottleneck    */

   /* scan all links to updated call stats */
   for (l=0 ; l<no_of_links + no_virt_links; l++)
     {
	this_stat = link_Ucall_stat[l];
	/* update unused free band */
	this_stat->un_fb_integ += (this_stat->unused_free_band * 
				(ancles_time - this_stat->last_unus_time));
	this_stat->last_unus_time = ancles_time;
     }

   if (this_call->call_type == CALL_TYPE5)
     {	
	for(this_link = this_call->path_ptr->path; this_link != NULL;
	    this_link = this_link->next)
	  {
	     this_stat = link_Ucall_stat[this_link->link_elem->id];
	     /* update number of Ucalls */
	     this_stat->no_of_Ucall_integ += (this_stat->no_of_Ucall * 
				   (ancles_time - this_stat->last_Uupdt_time));
	     this_stat->U_PCR_int += (this_stat->acc_PCR_U *
			           (ancles_time - this_stat->last_Uupdt_time));
	     this_stat->last_Uupdt_time = ancles_time;
	     if(action == OPEN_CALL)
             {
                this_stat->acc_PCR_U += ((CallType5*)(this_call->info))->mbr_value;
                this_stat->no_of_Ucall ++;
             }
             else
             {
                this_stat->acc_PCR_U -= ((CallType5*)(this_call->info))->mbr_value;
                this_stat->no_of_Ucall --;
             }
	  }

   /* statistics for the pac method */
	if(action == CLOSE_CALL)
	 pair_routing_info[this_call->source_id][this_call->dest_id].actual_callnum --;
	if(action == OPEN_CALL)
	 pair_routing_info[this_call->source_id][this_call->dest_id].actual_callnum ++;
     }
   else
     {
	for(this_link = this_call->path_ptr->path; this_link != NULL;
	    this_link = this_link->next)
	  {
	     this_stat = link_Ucall_stat[this_link->link_elem->id];
	     /* update free bandwidth for class U */
	     this_stat->fb_integ +=(this_stat->free_Uband * 
				    (ancles_time - this_stat->last_updt_time));
	     this_stat->last_updt_time = ancles_time;
             if(action == OPEN_CALL)
             {
	        this_stat->free_Uband -= mean_bw;
	        this_stat->acc_PCR_1_2 += eval_modified_bw(mean_bw, peak_bw);
             }
             else
             {
	        this_stat->free_Uband += mean_bw;
	        this_stat->acc_PCR_1_2 -= eval_modified_bw(mean_bw, peak_bw);
             }
	  }
     }
   
   /* We're going to freeze the situation of every call */
   /* prior to the computation of the new bandwidths    */
   for(call_ptr = head_of_call_list->next; call_ptr != head_of_call_list;
       call_ptr = call_ptr->next)
     {	
        if (call_ptr->call_type == CALL_TYPE5)	
	  {
	     temp_call = (CallType5 *)(call_ptr->info);
     /* Incrementing the transfered and decrementing the untransfered data
        amount, if interrupt happened -> no changes (it was immediatly) */
	     temp_call->br_integ += temp_call->curr_br *
	                              (ancles_time - temp_call->last_updt_time);
	     temp_call->tr_left -= temp_call->curr_br *
	                              (ancles_time - temp_call->last_updt_time);
	     temp_call->last_updt_time = ancles_time;

	     /* set the flag such that the new curr_br is not yet set */
	     /* for this call                                         */ 
	     temp_call->bott_state = curr_bottleneck_state;
	  }
     }

   /* compute the new bottleneck bit rates */
   interrupt_happened=FALSE;
   repl_call_ptr = NULL;
   curr_bottleneck_state = !curr_bottleneck_state;
   evaluate_UBW(this_call, action);
   
   if(interrupt_happened)
    {
     old_time = int_call_ptr->end_time;
     int_call_ptr->end_time = ancles_time;

   /* Replacing closer event */
     unrel_dequeue_event(int_call_ptr->my_close_event, old_time);
     (int_call_ptr->my_close_event)->ev_time = int_call_ptr->end_time;		
     insert_event(int_call_ptr->my_close_event);

   /* The call in the call-list and Ucall-list-on-link should be replaced
      as well, if there was a change in the curr. bw. */
     for(this_link = int_call_ptr->path_ptr->path; this_link != NULL;
         this_link = this_link->next)
      {
       link_ptr = this_link->link_elem;
   /* Find the right element in the Ucall-list of the link (based on the old
      closing time */
       call_list_ptr = search_Ucall_list_on_link(link_ptr, int_call_ptr, old_time);
       unrel_dequeue_Ucall_list_from_link(call_list_ptr, link_ptr, old_time);
       insert_Ucall_list_on_link(call_list_ptr, link_ptr);
      }
   /* Removing the call from the call-list and put it in its new place */
     unrel_dequeue_call(int_call_ptr, old_time);
     insert_call(int_call_ptr);
    }
   else
    {
   /* If necessary, the call closer event of the U call has to be replaced 
     this happens only for the event which will be replaced to the smallest
     occouring time */
     if(repl_call_ptr != NULL)
      {
       old_time = repl_call_ptr->end_time;
       if(!equ_double(ancles_time + min_repl_time, old_time, DBL_CMP_THS))
	{
	  repl_call_ptr->end_time = ancles_time + min_repl_time;

   /* replacing event */
	  unrel_dequeue_event(repl_call_ptr->my_close_event, old_time);
	  (repl_call_ptr->my_close_event)->ev_time = repl_call_ptr->end_time;		
	  insert_event(repl_call_ptr->my_close_event);

   /* The call in the call-list and Ucall-list-on-link should be replaced
      as well, if there was a change in the curr. bw. */
	  for(this_link = repl_call_ptr->path_ptr->path; this_link != NULL;
	      this_link = this_link->next)
	   {
	    link_ptr = this_link->link_elem;
  /* Find the right element in the Ucall-list of the link  */
	    call_list_ptr = 
	       search_Ucall_list_on_link(link_ptr, repl_call_ptr, old_time);
	    unrel_dequeue_Ucall_list_from_link(call_list_ptr, link_ptr, old_time);
            insert_Ucall_list_on_link(call_list_ptr, link_ptr);
	   }
  /* Removing the call from the call-list and put inserting on its new place */
	  unrel_dequeue_call(repl_call_ptr, old_time);
          insert_call(repl_call_ptr);
	 }
      }
    } /* if(! interrupt_happened) */

   /* update unused_free_band over all links after updating the */
   /* bottleneck rates                                          */

   for(l = 0; l < no_of_links + no_virt_links; l++)
     {
      this_stat = link_Ucall_stat[l];
      /* update unused free band */
      this_stat->unused_free_band = B_link[l];
     }
}
