/*
**  Module   : nd_ancles.c
**  Version   : $Revision: 3.3 $
**  Date      : $Date: 1999/01/11 14:53:50 $
**  Authors   : Andrea Francini, Marco Mellia, Giorgio Politano
**              Gabriele Favalessa, Luca Medico,
**		Filippo Di Natale, Zoltan Zsoka
**
**  Synopsis : This module contains the functions needed to allocate
**             and initialize the actual ANCLES data structures.
**
**  Functions: Link     *link_by_id(int link_id);
**             User     *user_by_id(int user_id);
**             void     initialize_ANCLES_structures(void);
**             void     init_real_links(void);
**             void     init_real_users(void);
**             void     init_real_call_gens(void);
**             void     init_real_nodes(void);
**             void     init_real_routes(void);
**             void     init_variables(void);
**             void     init_variables_stat(void);
**
**			   void		assign_nnids(void);
**
01-02-2001 Cleaning... -ZZ-
**              01-APR-23 - Insert the features for ASON:
**                          ASON call generators and duplex fibres - ZZ -
*/

#include "nd.h"

static void  assign_nnids(void);

extern Parse *global_node_list,
             *global_link_list,
             *global_user_list,
             *global_cgen_list,
             *global_route_list;

extern int pnode_num,
           plink_num,
           puser_num,
           pcgen_num,
           proute_num,
           psta_vars,
		   psig_vars;


/*
**  Function  : Link *link_by_id(int link_id)
**  Return    : the pointer to the required link.
**  Remarks   : the link vector is scanned in order to locate the link
**              identified by 'link_id'.
*/
Link *link_by_id(int link_id)
{
 register int i;
 for(i = 0; i < no_of_links; i++)
        if(link_vect[i]->id == link_id)
                return link_vect[i];
 return NULL;
}


/*
**  Function  : User *user_by_id(int user_id)
**  Return    : the pointer to the required user.
**  Remarks   : the user vector is scanned in order to locate the user
**              identified by 'user_id'.
*/
User *user_by_id(int user_id)
{
 register int i;
 for(i = 0; i < no_of_users; i++)
        if(user_vect[i]->id == user_id)
                return user_vect[i];
 return NULL;
}


/*
**  Function  : void initialize_ANCLES_structures(void)
**  Return    : none.
**  Remarks   : first the allocated ND structures are mapped onto the ANCLES
**              structures, then the internal structures are allocated and
**              initialized.
*/
void initialize_ANCLES_structures(void)
{
 int i;
 
 if(init_debug_level & ND_DEBUG_LEVEL)
    printf("\n --- ANCLES STRUCTURE INITIALIZATION ---\n\n");

 for(i=0; i < MAX_LINK; i++)
    link_vect[i] = NULL;
 init_real_links();
 if(init_debug_level & ND_DEBUG_LEVEL)
    printf("ANCLES links           OK\n");

 init_real_users();
 if(init_debug_level & ND_DEBUG_LEVEL)
     printf("ANCLES users           OK\n");

 assign_nnids();

 init_real_nodes();
 if(init_debug_level & ND_DEBUG_LEVEL)
     printf("ANCLES nodes           OK\n");

 init_real_call_gens();
 if(init_debug_level & ND_DEBUG_LEVEL)
     printf("ANCLES call generators OK\n");

 init_real_routes();
 if(init_debug_level & ND_DEBUG_LEVEL)
     printf("ANCLES routes OK\n");
}


/*
**  Function : void init_real_links(void)
**  Return   : none.
**  Remarks  : the link structures are allocated and initialized by using
**             the data available in the Parsed link list.
*/
void init_real_links(void)
{
 register int i, j, k;
 Link *p_link;
 Parse *head;
 PLink *lp;
 struct nd_vector* ths_vect;
 unsigned char set_mask;
 int fib_cap;

 Parse *debhead;
 PLink *deblp;


 max_wvoct_all_links = 0;

/* The allocation of all the links starts here; users and nodes will be */
/* allocated later.                                                     */

 head = global_link_list;
 for(i = 0; i < plink_num; i++, head = head->next)
   {

debhead = global_link_list;
for(j = 0; j < plink_num; j++, debhead = debhead->next)
{
 deblp = debhead->elem.link;
 if(deblp->destination == NOT_USED_ID || deblp->source == NOT_USED_ID)
	printf("anyad");
}
	 lp = head->elem.link;
    if(lp->id > plink_num - 1)
       {
        printf("Error:  The network description contains invalid links.\n");
        exit(1);
       }

    p_link = (Link *)malloc(sizeof(Link));
    if(p_link == NULL)
       {
        printf("Error:  Memory allocation error. ");
        printf("Not enough space for the data structures.\n");
        printf("        Reduce the network complexity.\n");
        exit(1);
       }

    p_link->id = lp->id;
    if(lp->destination == NOT_USED_ID || lp->source == NOT_USED_ID)
        print_error(head, "link seems not to be completely connected to the network");

    p_link->dest = lp->destination;
    p_link->source = lp->source;

    p_link->nnid = -1;
    p_link->is_node_node = TRUE;

    p_link->link_len = real_instance_value(head->instance_list, "link_length");

    p_link->bw_update_p = real_instance_value(head->instance_list, "bw_update_period");

    p_link->duplex_optic = integer_instance_value(head->instance_list, "duplex_optic");

    p_link->substituted = integer_instance_value(head->instance_list, "link_is_substituted");

    p_link->timescale = real_instance_value(head->instance_list, "link_timescale");
    p_link->free_band = capacity / p_link->timescale;

    p_link->linked_call = NULL;

/* Set the number of fibers and aviable wavelengths and according to it
   set the mask of free wavelengths */

	/* Wavelength capacity of one fiber of this link */
	fib_cap = integer_instance_value(head->instance_list, "fiber_capacity");

	if(fib_cap)
	{
		/* fiber capacity is defined */
		p_link->fiber_num = (int)ceil(floor(
			p_link->free_band / wavelength_capacity) /
			fib_cap);
		if(p_link->fiber_num > MAX_FIBER_NUM)
			p_link->fiber_num = MAX_FIBER_NUM;
	
		p_link->max_waves = fib_cap;
	}
	else
	{
		/* not defined -> one fiber */
		p_link->fiber_num = 1;
		p_link->max_waves = (int)floor(p_link->free_band / 
			wavelength_capacity);
	}
	
    p_link->used_waves = 0;
    p_link->peak_wave = 0;
    if(p_link->max_waves > 8 * MAX_WVOCTETS)
        p_link->max_waves = 8 * MAX_WVOCTETS;

	/* set the maximal used octet number */
	if(8*max_wvoct_all_links < p_link->max_waves)
		max_wvoct_all_links = (p_link->max_waves + 7) / 8;

    for(j = 0; j < p_link->max_waves/8; j++)
        p_link->free_waves[j] = 0xFF; /* free waves */
    for(j = p_link->max_waves/8; j < MAX_WVOCTETS; j++)
        p_link->free_waves[j] = 0x00; /* not aviable waves */
    for(j=0; j < p_link->max_waves%8; j++)
	{
        set_mask = 0x80;
		for(k = 0; k < j; k++)
			set_mask >>= 1;
		p_link->free_waves[p_link->max_waves/8] |= set_mask;
       /* free waves in the incomplete octet*/
	}

/* Set the number of available fibers on the wavelengths */
	p_link->free_fibers = (int *)MMmalloc(
		p_link->max_waves * sizeof(int), "init_real_links");
    for(j = 0; j < p_link->max_waves; j++)
        p_link->free_fibers[j] = p_link->fiber_num; /* free fibers */

    ths_vect = real_vector_instance_value(head->instance_list, "routing_threshold[]");

    for (j = 0; j < ths_vect->len; j++)
       p_link->routing_threshold[j] = ths_vect->element[j].real;

    hst_intervals[lp->id] = integer_instance_value(head->instance_list, "link_histo_steps");
    if (hst_intervals[lp->id] > 0)
      {
	hst_link_load[lp->id] =
	  (double *)malloc(sizeof(double) * (hst_intervals[lp->id] + 1));
	for(j = 0; j <= hst_intervals[lp->id]; j++)
	  hst_link_load[lp->id][j] = 0.0;
      }
    else
      hst_link_load[lp->id] = NULL;

    link_stat_vars[lp->id].info =
      integer_instance_value(head->instance_list, "link_carried_load");
    ason_link_stat_vars[lp->id].info =
      integer_instance_value(head->instance_list, "link_used_wavelengths");

    link_vect[i] = p_link;

   }
}


/*
**  Function  : void init_real_users(void)
**  Return    : none.
**  Remarks   : the user vector is allocated and initialized by using
**              the data available in the Parsed user list.
*/
void init_real_users(void)
{
 register int i;
 Parse    *head;
 PUser    *up;
 User     *user_elem;

 head = global_user_list;
 for(i = 0; i < puser_num; i++, head = head->next)
   {
    up = head->elem.user;
    user_elem = (User *)malloc(sizeof(User));
    if(user_elem == NULL)
       {
        printf("Error:  Memory allocation error. ");
        printf("Not enough space for the data structures.\n");
        printf("        Reduce the network complexity.\n");
        exit(1);
       }
    user_elem->id = up->id;
    user_elem->connected_node = up->connected_node;
    user_elem->out_link = link_by_id(up->link_id);
    user_elem->in_link = link_by_id(up->link_from_node);
    
	link_vect[up->link_id]->is_node_node = FALSE;
	link_vect[up->link_from_node]->is_node_node = FALSE;
	
	user_stat_vars[up->id].info =
      integer_instance_value(head->instance_list, "user_blocking_probability");
    classU_user_stat_vars[up->id].info =
      integer_instance_value(head->instance_list, "classU_user_assigned_bw");

    user_vect[i] = user_elem;
   }
}

/*
**  Function : void set_nnids(void)
**  Return   : none.
**  Remarks  : assign nnid to the node - node links and allocates the
**				tfree array
*/
void assign_nnids(void)
{
 register int i;
 int maxnn_waves = 0;

 no_of_nnlinks = 0;
 max_link_capa = 0.0;

 for(i = 0; i < plink_num; i++)
 {
	 if(link_vect[i]->is_node_node)
	 {
		 link_vect[i]->nnid = no_of_nnlinks;
		 nnlink_vect[no_of_nnlinks++] = i;

		 /* set the maximal used link capacity */
		 if(link_vect[i]->free_band > max_link_capa)
			 max_link_capa = link_vect[i]->free_band;

		 /* maximal number of wave channels */
		 if(link_vect[i]->max_waves * link_vect[i]->fiber_num > maxnn_waves)
			 maxnn_waves = link_vect[i]->max_waves * link_vect[i]->fiber_num;
	 }
 }
 tfree = (int *)MMmalloc(maxnn_waves * sizeof(int), "assign_nnids");
}
/*
**  Function  : void init_real_call_gens(void)
**  Return    : none.
**  Remarks   : the call generator vector is allocated and initialized
**              by using the data available in the Parsed call generator list.
*/
void init_real_call_gens(void)
{
 int      i;
 Parse    *head;
 PCGen    *cgenp;
 CallGen  *call_gen;

 head = global_cgen_list;

 for(i = 0; i < pcgen_num; i++, head = head->next)
   {
     long selected_range_type_type;
     long selected_range_type_integer = 0;
     double selected_range_type_real = 0.0;

    cgenp = head->elem.gener;
    call_gen = (CallGen *)malloc(sizeof(CallGen));
    if(call_gen == NULL)
       {
        printf("Error:  Memory allocation error. ");
        printf("Not enough space for the data structures.\n");
        printf("        Reduce the network complexity.\n");
        exit(1);
       }

    callgen_vect[i] = call_gen;

    call_gen->id = cgenp->id;
    call_gen->type = cgenp->type;
    call_gen->ref_user = user_by_id(cgenp->refer_user);

	call_gen->ref_user->generator = call_gen;

    if (set_cgen_context(call_gen->type) & NDC_GENERATOR_MASK)
      {
	call_gen->choosen_routing.selected_routing =
	  integer_instance_value(head->instance_list, "selected_routing");

	call_gen->choosen_routing.path_order_type =
	  integer_instance_value(head->instance_list, "path_order_type");

        call_gen->choosen_routing.use_hop_by_hop =
          integer_instance_value(head->instance_list, "use_hop_by_hop");

        call_gen->choosen_routing.use_crankback =
          integer_instance_value(head->instance_list, "use_crankback");

        call_gen->is_nonstationary =
          integer_instance_value(head->instance_list, "is_nonstationary");

        call_gen->load_reduction_factor =
          real_instance_value(head->instance_list, "load_reduction_factor");

        call_gen->normal_load_duration =
          real_instance_value(head->instance_list, "normal_load_duration");

        call_gen->low_load_duration =
          real_instance_value(head->instance_list, "low_load_duration");

	/*
	** Path number choice
	*/
	selected_range_type_type =
	  integer_instance_value(head->instance_list, "selected_range_type");

	if (selected_range_type_type == FIXED_PATH_NUMBER)
	  {
	    selected_range_type_integer =
	      integer_instance_value(head->instance_list, "path_number_range");

	    if (selected_range_type_integer == 1)
	      selected_range_type_type = ONLY_PRIMARY_PATH;
	    else if (selected_range_type_integer == INT_MAX)
	      selected_range_type_type = ALL_PATHS_NUMBER;
	  }
	else
	  {
	    selected_range_type_integer =
	      integer_instance_value(head->instance_list, "path_number_hop_range");

	    selected_range_type_real =
	      real_instance_value(head->instance_list, "path_number_length_range");
	  }

	call_gen->choosen_routing.selected_range_type.type =
	  selected_range_type_type;
	call_gen->choosen_routing.selected_range_type.value.dval =
	  selected_range_type_real;
	call_gen->choosen_routing.selected_range_type.value.ival =
	  selected_range_type_integer;

	call_gen->choosen_routing.primary_threshold =
	  integer_instance_value(head->instance_list, "primary_threshold");
	call_gen->choosen_routing.alternate_threshold =
	  integer_instance_value(head->instance_list, "alternate_threshold");
      }


/* The data structures concerning the parameters are allocated here.    */
    switch(call_gen->type)
      {
       case CG_TYPE1 :
	 {
	   CGType1* info;

	   info = MMmalloc(sizeof(*info), "init_real_call_gens");
	   call_gen->info = info;

	   ++no_of_type1_cg;

	   info->min_bit_rate =
	     real_instance_value(head->instance_list, "cg1_min_bit_rate");
	   info->max_bit_rate =
	     real_instance_value(head->instance_list, "cg1_max_bit_rate");
	   info->mean_hold_time =
	     real_instance_value(head->instance_list, "cg1_mean_hold_time");
/* The last information field ('arrival_rate') is assigned  after       */
/* reading the .TRF file, which contains the value of the fraction of   */
/* total traffic load generated by that user.                           */
	 }
         break;

       case CG_TYPE2 :
	 {
	   CGType2* info;

	   info = MMmalloc(sizeof(*info), "init_real_call_gens");
	   call_gen->info = info;

	   ++no_of_type2_cg;

	   info->min_peak_rate =
	     real_instance_value(head->instance_list, "cg2_min_peak_rate");
	   info->max_peak_rate =
	     real_instance_value(head->instance_list, "cg2_max_peak_rate");
	   info->mean_on_time =
	     real_instance_value(head->instance_list, "cg2_mean_on_time");
	   info->mean_off_time =
	     real_instance_value(head->instance_list, "cg2_mean_off_time");
	   info->mean_hold_time =
	     real_instance_value(head->instance_list, "cg2_mean_hold_time");
/* The last information field ('arrival_rate') is assigned  after       */
/* reading the .TRF file, which contains the value of the fraction of   */
/* total traffic load generated by that user.                           */
	 }
         break;

       case CG_TYPE3 :
	 {
	   CGType3* info;

	   info = MMmalloc(sizeof(*info), "init_real_call_gens");
	   call_gen->info = info;

	   ++no_of_type3_cg;

	   info->min_bit_rate_avg =
	     real_instance_value(head->instance_list, "cg3_min_bit_rate_avg");
	   info->max_bit_rate_avg =
	     real_instance_value(head->instance_list, "cg3_max_bit_rate_avg");
	   info->half_bw_interval =
	     real_instance_value(head->instance_list, "cg3_half_bw_interval");
	   info->mean_hold_time =
	     real_instance_value(head->instance_list, "cg3_mean_hold_time");
/* The last information field ('arrival_rate') is assigned  after       */
/* reading the .TRF file, which contains the value of the fraction of   */
/* total traffic load generated by that user.                           */
	 }
         break;

       case CG_TYPE4 :
	 {
	   CGType4* info;

	   info = MMmalloc(sizeof(*info), "init_real_call_gens");
	   call_gen->info = info;

	   ++no_of_type4_cg;

	   info->lower_bit_rate_avg =
	     real_instance_value(head->instance_list, "cg4_lower_bit_rate_avg");
	   info->medium_bit_rate_avg =
	     real_instance_value(head->instance_list, "cg4_medium_bit_rate_avg");
	   info->higher_bit_rate_avg =
	     real_instance_value(head->instance_list, "cg4_higher_bit_rate_avg");
	   info->lower_bit_rate_prob =
	     real_instance_value(head->instance_list, "cg4_lower_bit_rate_prob");
	   info->medium_bit_rate_prob =
	     real_instance_value(head->instance_list, "cg4_medium_bit_rate_prob");
	   info->higher_bit_rate_prob =
	     real_instance_value(head->instance_list, "cg4_higher_bit_rate_prob");
	   info->half_bw_interval =
	     real_instance_value(head->instance_list, "cg4_half_bw_interval");
	   info->mean_hold_time =
	     real_instance_value(head->instance_list, "cg4_mean_hold_time");
/* The last information field ('arrival_rate') is assigned  after       */
/* reading the .TRF file, which contains the value of the fraction of   */
/* total traffic load generated by that user.                           */
	 }
         break;

       case CG_TYPE5 :
	 {
	   CGType5* info;

	   info = MMmalloc(sizeof(*info), "init_real_call_gens");
	   call_gen->info = info;

	   ++no_of_type5_cg;

	   info->min_max_rate =
	     real_instance_value(head->instance_list, "cg5_min_max_rate");
	   info->max_max_rate =
	     real_instance_value(head->instance_list, "cg5_max_max_rate");
	   info->mean_gen_traffic =
	     real_instance_value(head->instance_list, "cg5_mean_gen_traffic");
	   info->min_accept_bw_thrs =
	     real_instance_value(head->instance_list, "min_accept_bw_thrs");

/* threshold parameter used in grooming algorithms is set */
	   info->vlopen_ubw_thrs =
	     real_instance_value(head->instance_list, "min_vl_open_thrs");

/* The last information field ('arrival_rate') is assigned  after       */
/* reading the .TRF file, which contains the value of the fraction of   */
/* total traffic load generated by that user.                           */
	 }
         break;

       case CG_TYPE6 :
	 {
	   CGType6* info;

	   info = MMmalloc(sizeof(*info), "init_real_call_gens");
	   call_gen->info = info;

	   ++no_of_type6_cg;

	   info->min_no_of_waves =
	     integer_instance_value(head->instance_list, "cg6_min_req_waves");
	   info->max_no_of_waves =
	     integer_instance_value(head->instance_list, "cg6_max_req_waves");
	   info->mean_hold_time =
	     real_instance_value(head->instance_list, "cg6_mean_hold_time");
/* The last information field ('arrival_rate') is assigned  after       */
/* reading the .TRF file, which contains the value of the fraction of   */
/* total traffic load generated by that user.                           */
	 }
         break;

       case CG_TYPE7 :
	 {
	   CGType7* info;

	   info = MMmalloc(sizeof(*info), "init_real_call_gens");
	   call_gen->info = info;

	   ++no_of_type7_cg;

	   info->saturation =
	       real_instance_value(head->instance_list, "cg7_saturation");
	   info->no_of_waves =
	     integer_instance_value(head->instance_list, "cg7_req_waves");
	   info->mean_hold_time =
	     real_instance_value(head->instance_list, "cg7_mean_hold_time");

	   if((info->saturation == 0.0) == (info->no_of_waves == 0))
	       {
	           fprintf(stderr, "\nExactly one of the saturation and the ");
		   fprintf(stderr, "no_of_waves parameters has to be 0!\n");
		   exit(1);
	       }
/* The last information field ('arrival_rate') is assigned  after       */
/* reading the .TRF file, which contains the value of the fraction of   */
/* total traffic load generated by that user.                           */
	 }
         break;

       case CG_TYPE8 :
	 {
	   CGType8* info;

	   info = MMmalloc(sizeof(*info), "init_real_call_gens");
	   call_gen->info = info;

	   ++no_of_type8_cg;

	   info->no_of_waves =
	     integer_instance_value(head->instance_list, "cg8_req_waves");
       }
	   break;

       case CG_TYPE9 :
	 {
	   CGType9* info;

	   info = MMmalloc(sizeof(*info), "init_real_call_gens");
	   call_gen->info = info;

	   ++no_of_type9_cg;

	   info->min_no_of_waves =
	     integer_instance_value(head->instance_list, "cg9_min_req_waves");
	   info->max_no_of_waves =
	     integer_instance_value(head->instance_list, "cg9_max_req_waves");
	   info->mean_hold_time =
	     real_instance_value(head->instance_list, "cg9_mean_hold_time");
	   info->peakedness =
	     real_instance_value(head->instance_list, "cg9_peakedness");

/* The incremention of arrival rate is calculated using the	peakedness	*/
	   info->arr_rate_increment = ( info->peakedness - 1.0 ) /
		   ( info->peakedness * info->mean_hold_time) ;

/* The information fields ('arrival_rate') is assigned  after       */
/* reading the .TRF file, which contains the value of the fraction of   */
/* total traffic load generated by that user.                           */
	 }
         break;

       case CG_TYPE10 :
         ++no_of_type10_cg;
	 call_gen->info = NULL;
/* The SINK call generator has not parameters to be set.		*/
         break;

       case CG_TYPE11 :
         ++no_of_type11_cg;
	 call_gen->info = NULL;
/* The TCP_SINK call generator has not parameters to be set.		*/
         break;

       case CG_TYPE12 :
         ++no_of_type12_cg;
	 call_gen->info = NULL;
/* The ASON_SINK call generator has not parameters to be set.		*/
         break;

/* When adding a new Call Generator Type, insert here a 'case CG_TYPEx:'*/
/* where 'x' is the new Call Generator Type.                            */
      }
   }
}


/*
**  Function  : void init_real_nodes(void)
**  Return    : none.
**  Remarks   : the node vector is allocated and initialized by using
**              the data available in the Parsed node list.
*/
void init_real_nodes(void)
{
 register int i,j;
 Parse *head;
 PNode *np;
 Node  *node_elem;

 head = global_node_list;
 for(i = 0; i < pnode_num; i++, head = head->next)
   {
    np = head->elem.node;
    node_elem = (Node *)malloc(sizeof(Node));
    if(node_elem == NULL)
       {
        printf("Error:  Memory allocation error. ");
        printf("Not enough space for the data structures.\n");
        printf("        Reduce the network complexity.\n");
        exit(1);
       }
    node_elem->id = np->id;
    node_elem->no_of_ports = np->port_no;
    node_elem->no_of_users = np->user_no;
    node_elem->alpha = real_instance_value(head->instance_list,"alpha_node");
    node_elem->beta = real_instance_value(head->instance_list,"beta_node");
    node_elem->opaque =
		integer_instance_value(head->instance_list,"node_is_opaque");
    node_elem->unreachable =
		integer_instance_value(head->instance_list,"node_is_unreachable");

    for(j = 0; j <np->user_no; j++)
       {
        node_elem->connected_user[j] = np->connected_user[j];
        node_elem->from_user[j] = link_by_id(np->from_user[j]);
        node_elem->to_user[j] = link_by_id(np->to_user[j]);
       }
	for(j = 0; j < np->port_no; j++)
       {
        node_elem->connected_node[j] = np->connected_node[j];
        node_elem->outqueue[j] = link_by_id(np->outlink[j]);
        node_elem->inqueue[j] = link_by_id(np->inlink[j]);
	}
	node_vect[np->id] = node_elem;
   }
}


/*
**  Function : void init_real_routes(void)
**  Return   : none.
**  Remarks  : the route structures are allocated and initialized by using
**             the data available in the Parsed route list.
*/
void init_real_routes(void)
{
 register int i, j;
 Route *p_route;
 Parse *head;
 PRoute *rp;
 LinkList *last;
 int dir, last_node;
 int ordered_cont_link[MAX_LINK];
 Boolean touched_nodes[MAX_NODE];

/* The allocation of all the routes starts here the links and nodes are */
/* already available.                                                   */

 route_vect = (Route **)malloc(proute_num * sizeof(Route *));
 if(route_vect == NULL)
 {
	 printf("Error:  Memory allocation error. ");
	 printf("Not enough space for the data structures.\n");
	 printf("        Reduce the network complexity.\n");
	 exit(1);
 }

 head = global_route_list;
 for(i = 0; i < proute_num; i++, head = head->next)
 {
    rp = head->elem.route;

    p_route = (Route *)malloc(sizeof(Route));
    if(p_route == NULL)
       {
        printf("Error:  Memory allocation error. ");
        printf("Not enough space for the data structures.\n");
        printf("        Reduce the network complexity.\n");
        exit(1);
       }

    if(rp->from_node == NOT_USED_ID || rp->to_node == NOT_USED_ID)
        print_error(head, "the route seems not to be completely defined");

    p_route->id = rp->id;
    p_route->from = rp->from_node;
    p_route->to = rp->to_node;

/* Build the path of the route :) */
	p_route->routepath = path_alloc();
	p_route->routepath->length = 0.0;
	p_route->routepath->no_of_hops = 0;
	if(node_vect[p_route->from]->opaque)
		p_route->routepath->no_of_opaque_crs = 1;
	else
		p_route->routepath->no_of_opaque_crs = 0;
	
	p_route->routepath->num_of_prot_chs = 0;
	p_route->routepath->prot_channels = NULL;
	for(j = 0; j<no_of_nnlinks; j++)
		p_route->routepath->protected_links[j] = 0;
	for(j = 0; j<no_of_nodes; j++)
		p_route->routepath->protected_nodes[j] = 0;

	p_route->routepath->path = NULL;
	
	/* The links are in reverse order because of parsing */
	for(j = 0; j < rp->link_no; j++)
		ordered_cont_link[rp->link_no - j - 1] =
		rp->contained_link[j];


	if(rp->link_no)
	{		
		for(j = 0; j<no_of_nodes; j++)
			touched_nodes[j] = FALSE;
		
		last_node = p_route->from;
		last = p_route->routepath->path = linklist_alloc();

/* check and insert each link of the route */
		for(j = 0; j < rp->link_no; j++)
		{
			last->link_elem = link_vect[ordered_cont_link[j]];
			if(j < rp->link_no - 1)
				last->next = linklist_alloc();
			else
				last->next = NULL;

			if(touched_nodes[last_node])
			{
				printf("Error:  Loop in predefined route %s ",
					route_name(rp->id));
				printf(".\n");
				exit(1);
			}
			touched_nodes[last_node] = TRUE;


/* check node coincidence */
			if(last_node == last->link_elem->source)
			{
				dir = 0;
				last_node = last->link_elem->dest;
			}
			else
			{
				if(last_node == last->link_elem->dest)
				{
					dir = 1;
					last_node = last->link_elem->source;
				}
/* link does not connected to the previous one */
				else
				{
					printf("Error:  Predefined route %s is ",
						route_name(rp->id));
					printf("not continous.\n");
					exit(1);
				}
			}
			
/* if simplex link is used the direction can be only 0 */
			if(!last->link_elem->duplex_optic && dir)
			{
				printf("Error:  Incorrect link in predefined route %s ",
					route_name(rp->id));
				printf("\n");
				exit(1);
			}
/* determine direction */
			last->direction = dir;

			p_route->routepath->length += last->link_elem->link_len;
			p_route->routepath->no_of_hops++;

			if(node_vect[last_node]->opaque)
			{
				last->endopsec = TRUE;
				p_route->routepath->no_of_opaque_crs++;
			}
			else
				last->endopsec = FALSE;
			
			last = last->next;
		}

		if(touched_nodes[last_node])
		{
			printf("Error:  Loop in predefined route %s ",
				route_name(rp->id));
			printf(".\n");
			exit(1);
		}

		/* route does not finish at the and point */
		if(last_node != p_route->to)
		{
			printf("Error:  Predefined route %s is ",
				route_name(rp->id));
			printf("not continous.\n");
			exit(1);
		}

	}
	
    route_vect[i] = p_route;
	
 }
}

static int count_statistical_variables(int    * sign_vars_ptr)
{
  Parse* head;
  long value;
  long stat_vars = 0, sign_vars = 0;

  value = integer_instance_value(global_instance_list, "total_carried_load");
  if (value != NOT_ANALYSED)
    {
      ++stat_vars;

      if (value == SIGNIFICANT)
	++sign_vars;
    }

  value = integer_instance_value(global_instance_list, "total_blocking_probability");
  if (value != NOT_ANALYSED)
    {
      ++stat_vars;

      if (value == SIGNIFICANT)
	++sign_vars;
    }

  value = integer_instance_value(global_instance_list, "used_wavelengths");
  if (value != NOT_ANALYSED)
    {
      ++stat_vars;

      if (value == SIGNIFICANT)
	++sign_vars;
    }

  for (head = global_link_list; head; head = head->next)
    {
      value = integer_instance_value(head->instance_list, "link_carried_load");
      if (value != NOT_ANALYSED)
	{
	  ++stat_vars;

	  if (value == SIGNIFICANT)
	    ++sign_vars;
	}
     }

  for (head = global_link_list; head; head = head->next)
    {
      value = integer_instance_value(head->instance_list, "link_used_wavelengths");
      if (value != NOT_ANALYSED)
	{
	  ++stat_vars;

	  if (value == SIGNIFICANT)
	    ++sign_vars;
	}
     }

  for (head = global_user_list; head; head = head->next)
    {
      value = integer_instance_value(head->instance_list, "user_blocking_probability");
      if (value != NOT_ANALYSED)
	{
	  ++stat_vars;

	  if (value == SIGNIFICANT)
	    ++sign_vars;
	}

      value = integer_instance_value(head->instance_list, "classU_user_assigned_bw");
      if (value != NOT_ANALYSED)
	{
	  ++stat_vars;

	  if (value == SIGNIFICANT)
	    ++sign_vars;
	}
    }

  *sign_vars_ptr = sign_vars;

  return stat_vars;
}

/*
**  Function  : void init_variables(void)
**  Return    : none.
**  Remarks   : the ANCLES main global variables are initialized here.
*/
void init_variables(void)
{
 capacity = real_instance_value(global_instance_list, "capacity");
 wavelength_capacity =
  real_instance_value(global_instance_list, "wavelength_capacity");
 bw_measure_uncertainty =
  real_instance_value(global_instance_list, "bw_measure_uncertainty");
 cell_loss_rate = real_instance_value(global_instance_list, "cell_loss_rate");
 no_load_samples = integer_instance_value(global_instance_list, "no_load_samples");
 meas_wind_size = integer_instance_value(global_instance_list, "meas_wind_size");
 k_star = real_instance_value(global_instance_list, "k_star");
 prot_fact = real_instance_value(global_instance_list, "prot_fact");
 meas_rel_error = real_instance_value(global_instance_list, "meas_rel_error");
 rel_load_ths = real_instance_value(global_instance_list, "rel_load_ths");
 cong_duration = real_instance_value(global_instance_list, "cong_duration");
 max_state_photos = integer_instance_value(global_instance_list, "max_state_photos");
 min_ucall_present = integer_instance_value(global_instance_list, "min_ucall_present");
 alpha = real_instance_value(global_instance_list, "alpha");
 beta = real_instance_value(global_instance_list, "beta");
 Gamma = real_instance_value(global_instance_list, "gamma");
 hop_weight = real_instance_value(global_instance_list, "hop_weight");
 length_weight = real_instance_value(global_instance_list, "length_weight");
 congestion_weight = real_instance_value(global_instance_list, "congestion_weight");
 ra5_min_efficiency = real_instance_value(global_instance_list, "overload_threshold");
 ac_period = real_instance_value(global_instance_list, "algorithm_choice_period");
 sample_deviation_parameter =
   real_instance_value(global_instance_list, "pac_sample_deviation");
 alg_decision_thrsh = real_instance_value(global_instance_list, "alg_decision_threshold");
 histeresys_thrsh = real_instance_value(global_instance_list, "histeresys_threshold");
 cong_thrsh = real_instance_value(global_instance_list, "congestion_threshold");
 recomputation_slot_duration = real_instance_value(global_instance_list,
    					  "recomputation_slot_duration");	
 widest_bottleneck_range =
   real_instance_value(global_instance_list, "widest_bottleneck_range");
 no_of_thresholds = integer_instance_value(global_instance_list, "no_of_threshold");

 load_balancing_algorithm =
   integer_instance_value(global_instance_list, "load_balancing_algorithm");
 selected_non_U_priority =
   integer_instance_value(global_instance_list, "non_u_priority");
 range_of_equiv_hop =
   integer_instance_value(global_instance_list, "range_of_equiv_hop");
 range_of_equiv_length =
   real_instance_value(global_instance_list, "range_of_equiv_length");
 selected_cac = integer_instance_value(global_instance_list, "selected_cac");
 global_stat_vars[0].info =
   integer_instance_value(global_instance_list, "total_carried_load");
 global_stat_vars[1].info =
   integer_instance_value(global_instance_list, "total_blocking_probability");
 global_stat_vars[2].info =
   integer_instance_value(global_instance_list, "used_wavelengths");
/* If the duration of U type calls is fixed, time based calls */
 time_based_ct5 =
  integer_instance_value(global_instance_list, "time_based_ct5");

 gen_load_red_factor =
  real_instance_value(global_instance_list, "general_load_red_factor");

 xxx_max_no_of_hops =
  integer_instance_value(global_instance_list, "xxx_max_no_of_hops");
 xxx_more_no_of_hops =
  integer_instance_value(global_instance_list, "xxx_more_no_of_hops");
/*******************************************************/
/* Interrupt analization variables */
 interrupt_statistics =
   integer_instance_value(global_instance_list, "interrupt_statistics");
 lhstep_factor = real_instance_value(global_instance_list, "lhstep_factor");

/*******************************************************/
/* Theoretical analysis */
 theo_analysis =
   integer_instance_value(global_instance_list, "theo_analysis");

/* Listing routes */
 list_routes =
   integer_instance_value(global_instance_list, "list_routes");

/* handle virtual IP links */
 ipowdm_ena =
  integer_instance_value(global_instance_list, "enable_IPoWDM");
 virtlink_bup =
  real_instance_value(global_instance_list, "virtlink_bw_update_period");
 duplex_llc =
  integer_instance_value(global_instance_list, "duplex_lowerlevel_conn");

 vl_open_algorithm =
   integer_instance_value(global_instance_list, "vl_open_algorithm");
 vl_close_algorithm =
   integer_instance_value(global_instance_list, "vl_close_algorithm");
 precreav_topology =
   integer_instance_value(global_instance_list, "precreate_virtual_topology");
 vl_close_hold_time =
   real_instance_value(global_instance_list, "vl_close_hold_time");

/* vlopen_ubw_thrs = 
   real_instance_value(global_instance_list, "min_vl_open_thrs");*/
 vlopen_max_hops = 
   integer_instance_value(global_instance_list, "max_hop_vl_open");
 cl_predef_vl =
  integer_instance_value(global_instance_list, "ena_close_predef_virtlinks");
 vlht_hopdep =
  integer_instance_value(global_instance_list, "ena_hopdep_vl_hold_time");
 
 maxvirtpaths =
  integer_instance_value(global_instance_list, "max_no_of_virt_routes");
 maxvirtnnpaths =
  integer_instance_value(global_instance_list, "max_no_of_virt_nn_routes");

 no_of_nodes = pnode_num;
 no_of_users = puser_num;
 no_of_links = plink_num;
 no_virt_links = 0; /* number of the virtual, IP links */
 no_of_cgens = pcgen_num;
 no_of_routes = proute_num;

 no_stat_vars = count_statistical_variables(&no_sign_vars);
}


/*
**  Function : void init_variables_stat(void)
**  Return   : none.
**  Remarks  : the ANCLES statistical variables are initialized here.
*/
void init_variables_stat(void)
{
 double confid_lev;
 int index;

 set_ref_accuracy(real_instance_value(global_instance_list, "accuracy"));
 confid_lev = real_instance_value(global_instance_list, "confid_level");
 if(fabs(confid_lev - 0.8) < PRECISION) index = 0;
 else if(fabs(confid_lev - 0.9) < PRECISION) index = 1;
 else if(fabs(confid_lev - 0.95) < PRECISION) index = 2;
 else if(fabs(confid_lev - 0.98) < PRECISION) index = 3;
 else if(fabs(confid_lev - 0.99) < PRECISION) index = 4;
 else if(fabs(confid_lev - 0.999) < PRECISION) index = 5;
 else
     {
      index = 2;
      printf("\nWarning:  value mysmatch on Confidence Level selection\n");
      printf("          The value in input is not in the allowed set. \n");
      printf("          The Confidence Level has been reset to the default\n");
      printf("          value of 0.95 \n");
     }
 set_confid_lev(index);
 set_max_coll_obs(integer_instance_value(global_instance_list, "num_observation"));
 set_min_transient_length(integer_instance_value(global_instance_list, "min_trans_len"));
 set_tr_bat_size(integer_instance_value(global_instance_list, "trans_batch_size"));
 set_tr_seq_len(integer_instance_value(global_instance_list, "trans_seq_len"));
 set_tr_accuracy(real_instance_value(global_instance_list, "trans_accuracy"));
 set_st_bat_size(integer_instance_value(global_instance_list, "stat_batch_size"));
}
