/*
**  Module    : nd_init.c
**  Version   : $Revision: 3.1 $
**  Date      : $Date: 1998/07/28 09:51:21 $
**  Authors   : Andrea Francini, Marco Mellia, Giorgio Politano
**              Gabriele Favalessa, Luca Medico
**
**  Synopsis  : this module contains the functions needed to initialize
**              the ND-ANCLES data structure.
**
**	History   : Support of routes - 2002.04.24. Zoltan Zsoka
**  
**  Functions : void  initialize_parameters(void)
**              Parse *init_parsed_node(int)
**              Parse *init_parsed_link(int)
**              Parse *init_parsed_user(int)
**              Parse *init_parsed_cgen(int)
**              Parse *init_parsed_route(int)
*/
 
#include "nd.h"
 
/*
**  Function  : void initialize_parameters(void)
**  Return    : none.
**  Remarks   : the symbol tables and the parameters are initialized
**              and assigned their predefined default values.
*/
void initialize_parameters(void)
{
 register int i;
 
/* Initializing the Symbol Tables.      */
 for(i = 0; i < MAX_NODE; i++)
   {
    node_symbol_table[i].name = NULL;
    node_symbol_table[i].ident = NOT_USED_ID;
   }
 for(i = 0; i < MAX_NET_USER; i++)
   {
    user_symbol_table[i].name = NULL;
    user_symbol_table[i].ident = NOT_USED_ID;
   }
 for(i = 0; i < MAX_LINK; i++)
   {
    link_symbol_table[i].name = NULL;
    link_symbol_table[i].ident = NOT_USED_ID;
   }
 for(i = 0; i < MAX_CALL_GEN; i++)
   {
    callgen_symbol_table[i].name = NULL;
    callgen_symbol_table[i].ident = NOT_USED_ID;
   }
 for(i = 0; i < MAX_ROUTE; i++)
   {
    route_symbol_table[i].name = NULL;
    route_symbol_table[i].ident = NOT_USED_ID;
   }

 init_constants();
 init_parameters();
}

/*
**  Function  : Parse *init_parsed_node(int ident)
**  Return    : the pointer to the newly allocated Parse structure.
**  Remarks   : the new node structure is allocated and initialized with the
**              declared default values.
*/
Parse *init_parsed_node(int ident)
{
 Parse *ptr;

 ptr = MMmalloc(sizeof(*ptr), "init_parsed_node");
 ptr->next = NULL;
 ptr->type = T_NODE;
 ptr->id = ident;
 ptr->decl_line = linenum;
 ptr->instance_list = create_default_instances(NDC_NODE);

 ptr->elem.node = MMmalloc(sizeof(PNode), "init_parsed_node");
 ptr->elem.node->id = ident;
 ptr->elem.node->user_no = 0;
 ptr->elem.node->port_no = 0;

 return ptr;
}
 
 
/*
**  Function  : Parse *init_parsed_link(int ident)
**  Return    : the pointer to the newly allocated Parse structure.
**  Remarks   : the new link structure is allocated and initialized with the
**              declared default values.
*/
Parse *init_parsed_link(int ident)
{
 Parse *ptr;

 ptr = MMmalloc(sizeof(*ptr), "init_parsed_link");
 ptr->next = NULL;
 ptr->type = T_LINK;
 ptr->id = ident;
 ptr->decl_line = linenum;
 ptr->instance_list = create_default_instances(NDC_LINK);

 ptr->elem.link = MMmalloc(sizeof(PLink), "init_parsed_link");
 ptr->elem.link->id = ident;
 ptr->elem.link->source = NOT_USED_ID;
 ptr->elem.link->destination = NOT_USED_ID;
 ptr->elem.link->assigned = FALSE;

 return ptr;
}
 
 
/*
**  Function  : Parse *init_parsed_user(int ident)
**  Return    : the pointer to the newly allocated Parse structure.
**  Remarks   : the new user structure is allocated and initialized with the
**              declared default values.
*/
Parse *init_parsed_user(int ident)
{
 Parse *ptr;
 
 ptr = MMmalloc(sizeof(*ptr), "init_parsed_user");
 ptr->next = NULL;
 ptr->type = T_USER;
 ptr->id = ident;
 ptr->decl_line = linenum;
 ptr->instance_list = create_default_instances(NDC_USER);

 ptr->elem.user = MMmalloc(sizeof(PUser), "init_parsed_user");
 ptr->elem.user->id = ident;
 ptr->elem.user->link_id = 0;

 return ptr;
}
 
 
/* The instances list creation is postponed until the call generator
   type is known. This is the only 'init_parsed_*' function that does
   not initialize the instances list. */
Parse *init_parsed_cgen(int ident)
{
 Parse *ptr;
 
 ptr = MMmalloc(sizeof(*ptr), "init_parsed_user");
 ptr->next = NULL;
 ptr->type = T_CGEN;
 ptr->id = ident;
 ptr->decl_line = linenum;
 ptr->instance_list = NULL;

 ptr->elem.gener = MMmalloc(sizeof(PCGen), "init_parsed_cgen");
 ptr->elem.gener->id = ident;
 
 return ptr;
}

/*
**  Function  : Parse *init_parsed_route(int ident)
**  Return    : the pointer to the newly allocated Parse structure.
**  Remarks   : the new route structure is allocated and initialized with the
**              declared default values.
*/
Parse *init_parsed_route(int ident)
{
 Parse *ptr;

 ptr = MMmalloc(sizeof(*ptr), "init_parsed_route");
 ptr->next = NULL;
 ptr->type = T_ROUTE;
 ptr->id = ident;
 ptr->decl_line = linenum;
 ptr->instance_list = create_default_instances(NDC_ROUTE);

 ptr->elem.route = MMmalloc(sizeof(PRoute), "init_parsed_route");
 ptr->elem.route->id = ident;
 ptr->elem.route->from_node = NOT_USED_ID;
 ptr->elem.route->to_node = NOT_USED_ID;
 ptr->elem.route->link_no = 0;

 return ptr;
}
 
