/*
**  Module    : nd_lex.l
**  Version   : $Revision: 3.1 $
**  Date      : $Date: 1998/07/28 09:51:21 $
**  Authors   : Andrea Francini, Marco Mellia, Giorgio Politano
**              Gabriele Favalessa, Luca Medico
**
**  Synopsis  : this module contains the lex (flex) definitions of the ND
**              language. It should be compiled with
**                  flex -i nd_lex.l
**              to produce the lexical analyzer needed by nd_yacc.c
**              We need flex to support the case insensitivity and
**              the C-like comments.
**              flex is a public domain software that can be easily
**              retrieved in Public FTP sites for almost every architecture.
**              nd_lex.l depends on the header file produced by nd_yacc.y
**
**				Support of explicite routes
**							- 2002.04.23. Zoltan Zsoka -
**
*/

%option noyywrap

%{
#include "nd.h"
#include "nd_yacc.tab.h"
 
int linenum = 1;
%}
 
DIGIT [0-9]
%x comment
%%
"/*"          BEGIN(comment);
<comment>[^*\n]*
<comment>[^*\n]*\n  ++linenum;
<comment>"*"+[^*/\n]*
<comment>"*"+[^*/\n]*\n ++linenum;
<comment>"*"+"/"    BEGIN(INITIAL);
default        return DEFAULT_TOKEN;
demand         return DEMAND_TOKEN;
parameter      return PARAMETER_TOKEN;
node           return NODE_TOKEN;
user           return USER_TOKEN;
link           return LINK_TOKEN;
call_generator return CALL_GENERATOR_TOKEN;
route		   return ROUTE_TOKEN;
referring_to   return REFERRING_TO_TOKEN;
connected_to   return CONNECTED_TO_TOKEN;
used_by        return USED_BY_TOKEN;
type           return TYPE_TOKEN;
thru           return THRU_TOKEN;
with           return WITH_TOKEN;
endwith        return ENDWITH_TOKEN;
linklist	   return LINKLIST_TOKEN;
from_node	   return FROM_NODE_TOKEN;
to_node        return TO_NODE_TOKEN;
"="            return ASSIGN_TOKEN;
";"            return ENDCLAUSE_TOKEN;
","            return SEPARATOR_TOKEN;
"{"            return BRACE_LEFT_TOKEN;
"}"            return BRACE_RIGHT_TOKEN;
"-"            return MINUS_TOKEN;
 
[a-z][a-z0-9_]*(\[\])? { 
                         yylval.string = strdup(yytext);
                         return STRING_TOKEN;
                       }

{DIGIT}+               { 
                         yylval.constant.value.integer = atoi(yytext); 
	                 yylval.constant.name = "integer number";
                         yylval.constant.type = NDT_INTEGER;

                         return NUMBER_TOKEN; 
                       }

{DIGIT}+"."{DIGIT}*    { 
                         yylval.constant.value.real = atof(yytext); 
	                 yylval.constant.name = "real number";
	                 yylval.constant.type = NDT_REAL;

                         return NUMBER_TOKEN; 
                       }

[ \t]*                 { /* eat whitespace */ }
\n                     linenum ++;
.                      printf("Undefined: %s\n", yytext);
%%
