/*
**  Module    : signal.c
**  Version   : $Revision: 3.1 $
**  Date      : $Date: 1998/07/28 09:51:21 $
**  Authors   : Marco Mellia, Gabriele Favalessa, Luca Medico
**
**  Synopsis  : This modules contain all the requested function to intercep
**              the SIGHUP signal. It prints out the status of the simulation. 
**
**  Functions : void    print_actual_stat(int )
**              void    print_actual_stat_now(void)
**
**  History   : 01-MAY-1997 - Creation of the module.   -MM-
**              08-FEB-1997 - Revision: now use the generate_XXX_report function
*/
 
#include "ancles.h"

#ifdef unix
/* prototipes declaration */
void print_actual_stat_now(void);

void print_all_actual_stat(int signalNumber)
{
#ifdef SYSV_SIGNALS
   /* put back the signal interceptor */
   signal (SIGUSR1, print_all_actual_stat);
#endif
   /* call the real function that print out the proceding of the simulation */
   if (ancles_time == 0.0)
      return;
   print_actual_stat_now();
}

void print_actual_stat(int signalNumber)
{
   FILE * out_fp = stdout;
   int i;

#ifdef SYSV_SIGNALS
   /* put back the signal interceptor */
   signal (SIGUSR2, print_actual_stat);
#endif
   
   if (ancles_time == 0.0)
      return;
   
   /* printing the ancles_time */
   printf("\n\t*******************************************************");
   printf("*************");
   printf("\n\t* Received a SIGUSR2 signal: printing the status of the ");
   printf("simulation *");
   printf("\n\t*******************************************************");
   printf("*************");
   printf("\n\n\tancles time = %f\n",ancles_time);
   
   /* reporting the current statistics for the total blocking probability */
   if (global_stat_vars[1].info != NOT_ANALYSED)
     {
       report_statistics(out_fp, global_stat_vars[1].number, TRUE);
       if(transient_is_over(global_stat_vars[1].number))
	 report_statistics(out_fp, global_stat_vars[1].number, FALSE);
     }

   /* reporting the current statistics fot the total carried load */
   if (global_stat_vars[0].info != NOT_ANALYSED)
     {
       report_statistics(out_fp, global_stat_vars[0].number, TRUE);
       if(transient_is_over(global_stat_vars[0].number))
	 report_statistics(out_fp, global_stat_vars[0].number, FALSE);
     }

   for(i = 0; i < no_of_cgens; i++)
     {
       Stat_var* var;
       
       var = &classU_user_stat_vars[callgen_vect[i]->ref_user->id];

       if (var->info != NOT_ANALYSED)
	 {
	   report_statistics(out_fp, var->number, TRUE);

	   if (transient_is_over(var->number))
	     report_statistics(out_fp, var->number, FALSE);
	 }
     }
}

void write_stat_and_exit(int signal_number)
{
   if (ancles_time == 0.0)
      return;
   printf("\n\n\n WARNING: THE SIMULATION WAS STOPPED BY A SIGINT SIGNAL");
   printf("\n\n          all the output file are written using the current");
   printf("\n          acheived precision\n\n");
   normal = SIGINT;
   stop_simulating();
}

void print_actual_stat_now(void)
{
   FILE *      out_fp;
   printf("\n\t*******************************************************");
   printf("*************");
   printf("\n\t* Received a SIGUSR1 signal: printing the status of the ");
   printf("simulation *");
   printf("\n\t*******************************************************");
   printf("*************");

 out_fp = stdout; 
/* CBR STAT.    */
 generate_CBR_report(out_fp);

/* CL STAT.      */
 generate_CL_report(out_fp);

 if( UCall_present )
   {

/* UCL STAT.    */
    generate_UCL_report(out_fp);

/* UBR STAT.    */
    normal = SIGUSR2;
    generate_UBR_report(out_fp);
    normal = TRUE;

   }
}

#endif
