/*
**  Module    : theo_complex.c
**  Version   : $1.1$
**  Date      : $Date: 2001/01/31$
**  Authors   : Tamas Zombori, Zoltan Zsoka
**
**  Synopsis  : this module contains the functions used for 
**				the theoretical calculation of the blocking
**				probability. "complex" means that the link loads
**				are handled per link (not average)
**
**  Functions:	double	theo_complex(int, double *);
**
**  Revisions:	New function(s) with a more complex calculation
**				- ZSZ 2001.02.01 -
**				Multifiber analysis: complex, Li-Somani, BPP
**				- ZSZ 2003.03.31 -
*/

#include "ancles.h"

static double prxr(Path*, LinkList*, int, int, int);
static double gri(Path*, int);
static double cor_prxr_multi(Path*, LinkList*, int, int);
static double cor_pujrxr_multi(Path*, LinkList*, int, int);
static double cor_gri_multi(Path*, int);

static double powering(double, int);
static double gamM0(int, int);
static double gamM1(int, int);
static double getjim(int, int, int, int);

static double U_lisomani(int, int, int, int, int);
static double S_lisomani(int, int, int, int);
static double RWTLC(int, int, int, int);
static double P_1_lisomani(int, int, int, int);
static double P_2_lisomani(int, int, int, int, int);
static double P_3_lisomani(int, int, int, int, int);
static double P_4_lisomani(int, int, int, int, 
					int, int, int);
static double P_5_lisomani(int, int, int, int);
static double pi_lisomani(int, int, int, int, int);

double m[MAX_LINK];		// tomb a linkeknkenti terheltseg ertekekhez
int n[MAX_LINK];		// tomb a linkeknkenti Markov lanc meret ertekekhez
double lambda[MAX_LINK];	// tomb a linkeknkenti lambda ertekekhez
double poilambda[MAX_LINK];	// tomb a linkeknkenti lambda ertekekhez
							// Poisson forgalom eseteben
double p0[MAX_LINK];		// tomb a linkeknkenti p0 ertekekhez
double poip0[MAX_LINK];		// tomb a linkeknkenti p0 ertekekhez Poisson

double **betaji;		/* ketdimenzios tomb a linkenkent ertelmezett
							   beta ertekrol: adott i meretu hullamhossz halmaz
								szabad a j linken */
double **etaji;		/* ketdimenzios tomb a linkenkent ertelmezett
							   eta ertekrol: az i-dik hh. szabad, felteve, hogy
								a tobbi i-1 szabad */
double ***jim;		/* set i is free on link j given m free channels */
double lambdaj[MAX_LINK];	/* mean arrival rate and on the link */
double **lambdajk;	/* and its correlated version (j yes - k no !!!) */

double **gM0;	/* gammaM0 values */
double **gM1;	/* gammaM1 values */

double **pcl1;	/* on link j, probability of k occupied fibers (on a given wvl) */
double ****pcl2;	/* from link j to link jsuc, probability of i non continuing 
					and k cont calls (on a given wvl of j) */
double ***pn;	/* from link j to link jsuc, probability of Mjsuc - i non continuing 
					calls on jsuc given that i cont from j (on a given wvl) */

int F;
int W;
double **f_lisomani;	/* the values of the f function */
double ***g_lisomani;	/* the values of the g function */
/*double *lambdaej;		* entering traffic on link j */
/*double *lambdalj;		* leaving traffic on j */
double **lambdacjk;		/* common traffic on j and k */
double **pidenom;		/* normalizing factor of pi_lisomani */

double *****pi_lis; /* x_l, x_c, x_e, j, jprev */
double ****P_5_lis; /* Ncap_f_2, Xcap_f_1, zp_X_f1, y_X_f1 */
/* double *******P_4_lis; * y_X_b1, zp_X_b1, zp_X_f1, y_b2, Xcap_b_1, Xcap_f_1, w */
/* double *****P_3_lis; * zp_X_b1, zp_c_2, Xcap_b_1, Xcap_f_1, w */
double *****P_2_lis; /* Ncap_f_2, Xcap_f_1, zp_c_2, y_f_2, w */
double ****P_1_lis; /* Zcap_c_2, Xcap_f_1, z_c_2, y_f_2 */
double ****RWTLC_lis; /* Ncap_f_2, Xcap_f_1, z_c_2, y_f_2 */

/* Fontos! A mu erteke 1-re van "normalva", vagyis tulajdonkeppen a
   ro = lambda/mu erteket hasznaljuk lambdakent! */


/*
double theo_multimost_complex(int f): ebben a szamolas
figyelembe veszi a linkek osszefuggeset is es fixpontot is keres es
lehetoseg van tobboptikas linkek kezelesere is (1-nel tobb szal)
A szalak szama a link kapacitasabol szamolodik ki (fel van mar 
jegyezve az adott linkhez).
f: hullamhosszak szama
*** egyelore csak a Poisson van meg! ***
Feljegyzi az iteraciok szamat is.
*/


double theo_multimost_complex(int f, double *poiblocking_prob, int *iternum)
{
	int i, j, jsuc, mf, s, d, k;// tobbnyire indexelesre hasznalt valtozok
	/*double dn[MAX_LINK];		// tomb a linkenkenti Markov lancok meretenek reszszamitasaihoz*/
	double **alfajm;			// erkezesi intenzitas a j linken, felteve m szabad csatornat
	double **alr;				// erkezesi intenzitas
	double **qjm;				// m szabad csatorna allapotvaloszinusege
	int **neighbours;			// hany utban szomszedos az adott linkpar
	double rblock, prevblock, actblock = 0.0;	// blokkolasertekek (iteracio miatt tobb van)
	double signum;				// signum fuggveny
								/* double ntag, dtag, bintagprod;
	int b; */
	double sum;					// szamolaskor hasznalt segedvaltozok
	int Mj, Mjsuc, Mk;			// szalak szama a j, jsuc, ill k linken
	Path *pa;					// ut
	LinkList *p;				// link
	Boolean k_is_in;
	
	double blocking_prob = 0.0; // a kimenet
	double firstblock;
	*(poiblocking_prob) = 0.0;
	
	/* Allocate memory for the arrays */
	alfajm = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_multimost_complex");
	alr = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_multimost_complex");
	qjm = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_multimost_complex");
	betaji = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_multimost_complex");
	etaji = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_multimost_complex");
	lambdajk = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_multimost_complex");
	pcl1 = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_multimost_complex");
	pcl2 = (double ****)MMmalloc(sizeof(double***)*no_of_nnlinks, "theo_multimost_complex");
	pn = (double ***)MMmalloc(sizeof(double***)*no_of_nnlinks, "theo_multimost_complex");
	neighbours = (int **)MMmalloc(sizeof(int*)*no_of_nnlinks, "theo_multimost_complex");
	
	gM0 = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_multimost_complex");
	gM1 = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_multimost_complex");
	jim = (double ***)MMmalloc(sizeof(double**)*no_of_nnlinks, "theo_multimost_complex");
	
	for(j=0; j<no_of_nnlinks; j++)
	{
		Mj = link_vect[nnlink_vect[j]]->fiber_num;
		alfajm[j] = (double *)MMmalloc(sizeof(double)*(Mj * f + 1), "theo_multimost_complex");
		alr[j] = (double *)MMmalloc(sizeof(double)*(Mj * f + 1), "theo_multimost_complex");
		qjm[j] = (double *)MMmalloc(sizeof(double)*(Mj * f + 1), "theo_multimost_complex");
		betaji[j] = (double *)MMmalloc(sizeof(double)*(Mj * f + 1), "theo_multimost_complex");
		etaji[j] = (double *)MMmalloc(sizeof(double)*(Mj * f + 1), "theo_multimost_complex");
		lambdajk[j] = (double *)MMmalloc(sizeof(double)*(no_of_nnlinks),
			"theo_multimost_complex");
		pcl1[j] = (double *)MMmalloc(sizeof(double)*(Mj + 1), "theo_multimost_complex");
		pcl2[j] = (double ***)MMmalloc(sizeof(double**)*(no_of_nnlinks),
			"theo_multimost_complex");
		pn[j] = (double **)MMmalloc(sizeof(double*)*(no_of_nnlinks),
			"theo_multimost_complex");
		neighbours[j] = (int *)MMmalloc(sizeof(int)*no_of_nnlinks, "theo_multimost_complex");
		gM0[j] = (double *)MMmalloc(sizeof(double)*no_of_nnlinks, "theo_multimost_complex");
		gM1[j] = (double *)MMmalloc(sizeof(double)*no_of_nnlinks, "theo_multimost_complex");
		jim[j] = (double **)MMmalloc(sizeof(double*)*(f + 1), "theo_multimost_complex");
		
		for(jsuc = 0; jsuc<no_of_nnlinks; jsuc++)
		{
			pcl2[j][jsuc] = (double **)MMmalloc(sizeof(double*)*(Mj + 1),
				"theo_multimost_complex");
			pn[j][jsuc] = (double *)MMmalloc(sizeof(double)*(Mj + 1),
				"theo_multimost_complex");
			
			for(i = 0; i < Mj + 1; i++)
				pcl2[j][jsuc][i] = (double *)MMmalloc(sizeof(double)*(Mj + 1),
				"theo_multimost_complex");
			neighbours[j][jsuc] = 0;
		}
		for(i = 0; i < f + 1; i++)
		{
			jim[j][i] = (double *)MMmalloc(sizeof(double)*(Mj * f + 1),
				"theo_multimost_complex");
			for(mf = 0; mf < Mj * f + 1; mf++)
				jim[j][i][mf] = getjim(Mj, i, mf, f);
		}
		
	}
	
	/* set neighbourhood information */
	for(s = 0; s < no_of_cgens; s++)
	{
		for(d = 0; d < no_of_cgens; d++)
		{
			if(callgen_vect[s]->type == CG_TYPE10 ||
				callgen_vect[s]->type == CG_TYPE11 ||
				callgen_vect[s]->type == CG_TYPE12) continue;
			if(NULL == (pa = path_mat[(&callgen_vect[s]->choosen_routing)->
				path_order_type][s][d])) continue;
			for(p = pa->path->next; p->next->link_elem->is_node_node; p = p->next)
			{
				neighbours[p->link_elem->nnid]
					[p->next->link_elem->nnid]++;
				neighbours[p->next->link_elem->nnid]
					[p->link_elem->nnid]++;
			}
		}
	}
	
	betaji[0][0] = -1.0;
	*(iternum) = 0;
	
	/* Iteracio kerete */
	do
	{
		/* inicializalas */
		for(j = 0; j < no_of_nnlinks; j++)
		{
			for(mf = 0; mf < link_vect[nnlink_vect[j]]->fiber_num * f + 1; mf++)
			{
				alr[j][mf] = 0.0;
			}
		}
		/* terhelesek es Markov lanc meretek osszegzese 
		(megfeleloen az egymasrahatasbol adodo csokkenesnek) */
		for(s = 0; s < no_of_cgens; s++)
		{
			for(d = 0; d < no_of_cgens; d++)
			{
				if(callgen_vect[s]->type == CG_TYPE10 ||
					callgen_vect[s]->type == CG_TYPE11 ||
					callgen_vect[s]->type == CG_TYPE12) continue;
				if(NULL == (pa = path_mat[(&callgen_vect[s]->choosen_routing)->
					path_order_type][s][d])) continue;
				for(p = pa->path; p != NULL; p = p->next)
				{
					if(p->link_elem->is_node_node)
					{
						for(mf = 0; mf < p->link_elem->fiber_num * f + 1; mf++)
						{
						/* the intensity will be summed, but we had to use the 
							reduced value (reduction because of intereffect of links) */
							alr[p->link_elem->nnid][mf] += load * load_conv_fact * 
								traff_pattern[callgen_vect[s]->ref_user->id]
								[callgen_vect[d]->ref_user->id] *  
								cor_prxr_multi(pa, p, f, mf);
/*							switch(callgen_vect[s]->type)
							{
							case CG_TYPE6:
							case CG_TYPE9:
							case CG_TYPE7:
								break;
								default: fprintf(stderr, 
										 "\nThe theoretical analysis works only for ASON sources!\n");
									exit(1);
							}*/
						}
					}
					
				}
			}
		}
		
		/* minden linkre */
		/* alfak es n-ek meghatarozasa */
		for(j = 0; j < no_of_nnlinks; j++) 
		{
			Mj = link_vect[nnlink_vect[j]]->fiber_num;
			for(mf = 0; mf < Mj * f + 1; mf++)
			{
				// Poisson forgalomnal:
				alfajm[j][mf] = alr[j][mf];
			}
			
			/*  minden terhelesre q allapotvaloszinusegek ertekek meghatarozasa */
			qjm[j][0] = 1;
			for(mf = 1; mf < Mj * f + 1; mf++) //szabad csatornak szama
			{
				qjm[j][mf] = qjm[j][mf-1] * (Mj* f + 1 - mf) / alfajm[j][mf];
				qjm[j][0] += qjm[j][mf];
			}
			/* Normalization and assign of mean arrival rates */
			lambdaj[j] = 0.0;
			for(mf = 1; mf < Mj * f + 1; mf++)
			{
				qjm[j][mf] /= qjm[j][0];
				lambdaj[j] += alfajm[j][mf] * qjm[j][mf];
			}
			qjm[j][0] = 1 / qjm[j][0];
			
			for(k = 0; k < no_of_nnlinks; k++)
			{
				lambdajk[j][k] = 0.0;
			}
		}
		
		for(k = 0; k < no_of_nnlinks; k++)
		{
			Mk = link_vect[nnlink_vect[k]]->fiber_num;
			for(s = 0; s < no_of_cgens; s++)
			{
				for(d = 0; d < no_of_cgens; d++)
				{
					if(callgen_vect[s]->type == CG_TYPE10 ||
						callgen_vect[s]->type == CG_TYPE11 ||
						callgen_vect[s]->type == CG_TYPE12) continue;
					if(NULL == (pa = path_mat[(&callgen_vect[s]->choosen_routing)->
						path_order_type][s][d])) continue;
					
						/* take care only about the routes which don't contain
					the link k */
					k_is_in = FALSE;
					for(p = pa->path; p != NULL; p = p->next)
					{
						if(p->link_elem->is_node_node &&
							p->link_elem->nnid == k)
							k_is_in = TRUE;
					}
					
					if( !k_is_in )
					{
						for(p = pa->path; p != NULL; p = p->next)
						{
							/* update the value of non correlated traffic on the links */
							if(p->link_elem->is_node_node)
							{
								Mjsuc = link_vect[nnlink_vect
									[p->link_elem->nnid]]->fiber_num;
								for(mf = 1; mf < f * Mjsuc + 1; mf++)
								{
									lambdajk[p->link_elem->nnid][k] +=
										load * load_conv_fact * traff_pattern
										[callgen_vect[s]->ref_user->id]
										[callgen_vect[d]->ref_user->id] * 
										cor_prxr_multi(pa, p, f, mf) * 
										qjm[p->link_elem->nnid][mf];
								}
							}
						} /* p */
					}
				} /* d */
			} /* s */
		} /* k */
		
		/* uj betak es etak meghatarozasa minden darabszamra */
		for(j = 0; j < no_of_nnlinks; j++)
		{
			Mj = link_vect[nnlink_vect[j]]->fiber_num;
			betaji[j][0] = 1.0; /* prxr call not for the first time */
			for(i = 1; i < f+1; i++)
			{
				betaji[j][i] = 0.0;
				for(mf = i; mf < Mj * f + 1; mf++)
				{
					betaji[j][i] += qjm[j][mf] * jim[j][i][mf];
				}
				etaji[j][i] = betaji[j][i]/betaji[j][i-1];
			}
			
			/* set the probabilities of link states (dependent on other links) */
			for(k = 0; k < Mj + 1; k++)
			{
				pcl1[j][k] = 0.0;
				for( mf = Mj - k; mf < Mj * f - k + 1; mf++)
					pcl1[j][k] += qjm[j][mf] * fbinom(Mj * (f-1), mf - Mj + k) /
					fbinom(Mj * f, mf);
				pcl1[j][k] *= fbinom(Mj, k);
			}
		} /* for j */
		
		/* dependent probablities of link states */
		for(j = 0; j < no_of_nnlinks; j++)
		{
			Mj = link_vect[nnlink_vect[j]]->fiber_num;
			for(jsuc = 0; jsuc < no_of_nnlinks; jsuc++)
			{
				Mjsuc = link_vect[nnlink_vect[jsuc]]->fiber_num;
				/* the probabilities will be used only in case 
				of following links */
				if(neighbours[j][jsuc])
				{
					for(i = 0; i < Mj + 1; i++) /* continuing */
					{
						sum = 0.0;
						for(k = 0; k < Mj - i + 1; k++) /* non continuing */
							pcl2[j][jsuc][i][k] = pcl1[j][i+k] *
							powering(1.0 - lambdajk[j][jsuc]/lambdaj[j],i) *
							powering(lambdajk[j][jsuc]/lambdaj[j],k) *
							fbinom(i+k,i);
						
						for(k = 0; k < Mjsuc - i + 1; k++) /* non continuing */
							/* used in pn as denominator */
							sum += pcl1[jsuc][k+i] *
							powering(1.0 - lambdajk[jsuc][j]/lambdaj[jsuc],i) *
							powering(lambdajk[jsuc][j]/lambdaj[jsuc],k) *
							fbinom(i+k,i);
						
						if(i < Mjsuc + 1 && sum)
							pn[j][jsuc][i] = pcl1[jsuc][Mjsuc] *
							powering(1.0 - lambdajk[jsuc][j]/lambdaj[jsuc],i) *
							powering(lambdajk[jsuc][j]/lambdaj[jsuc],Mjsuc - i) *
							fbinom(Mjsuc,i) / sum;
						else
							pn[j][jsuc][i] = 0.0;
					}
				}
				else
				{
					for(i = 0; i < Mj + 1; i++)
					{
						for(k = 0; k < Mj + 1; k++)
							pcl2[j][jsuc][i][k] = -1.0;
						pn[j][jsuc][i] = -1.0;
					}
				}
				gM0[j][jsuc] = -1.0;
				gM1[j][jsuc] = -1.0;
			} /* for jsuc */
		} /* for j */
		
		
		prevblock = actblock;
		actblock = 0.0;
		
		/* recalculate blocking */
		for(s = 0; s < no_of_cgens; s++)
		{
			for(d = 0; d < no_of_cgens; d++)
			{
				if(callgen_vect[s]->type == CG_TYPE10 ||
					callgen_vect[s]->type == CG_TYPE11 ||
					callgen_vect[s]->type == CG_TYPE12) continue;
				if(NULL == (pa = path_mat[(&callgen_vect[s ]->choosen_routing)->
					path_order_type][s][d])) continue;
				signum = 1.0;
				rblock = 1.0;
				/* calculate and sum for each i wavelength set */
				for(i = 1; i < f+1; i++)
				{
					rblock -= signum * fbinom(f, i) * cor_gri_multi(pa, i);
					signum *= -1.0;
				}
				actblock += rblock  * traff_pattern
					[callgen_vect[s]->ref_user->id][callgen_vect[d]->ref_user->id];
			}
		}
		
		if(*(iternum) == 0)
			firstblock = actblock;
		
		*(iternum) = *(iternum)+1;
	}
	while(fabs(prevblock-actblock) > 10E-6 && *iternum<100);

	for(j=0; j<no_of_nnlinks; j++)
	{
		Mj = link_vect[nnlink_vect[j]]->fiber_num;
		free(alfajm[j]);
		free(alr[j]);
		free(qjm[j]);
		free(betaji[j]);
		free(etaji[j]);
		free(lambdajk[j]);
		free(neighbours[j]);
		free(pcl1[j]);
		for(k=0; k<no_of_nnlinks; k++)
		{
			for(i = 0; i < Mj + 1; i++)
				free(pcl2[j][k][i]);
			free(pcl2[j][k]);
			free(pn[j][k]);
		}
		
		for(i=0; i<f +1; i++)
			free(jim[j][i]);
		
		free(pcl2[j]);
		free(pn[j]);
		free(gM0[j]);
		free(gM1[j]);
		free(jim[j]);
	}
	
	free(alfajm);
	free(alr);
	free(qjm);
	free(betaji);
	free(etaji);
	free(lambdajk);
	free(neighbours);
	free(pcl1);
	free(pcl2);
	free(pn);
	free(jim);
	free(gM0);
	free(gM1);
	
	*(poiblocking_prob) = actblock;
	return firstblock; /*prevblock;*/
	
}


/*-----------------------------------------------------------------------------------*/

/*
double theo_multiindep_complex(int f): ebben a szamolas
nem veszi figyelembe a linkek osszefuggeset, de  fixpontot is keres es
lehetoseg van tobboptikas linkek kezelesere is (1-nel tobb szal)
A szalak szama a link kapacitasabol szamolodik ki (fel van mar 
jegyezve az adott linkhez).
f: hullamhosszak szama
*** egyelore csak a Poisson van meg! ***
Feljegyzi az iteraciok szamat is.
*/


double theo_multiindep_complex(int f, double *poiblocking_prob, int *iternum)
{
	int i, j, mf, s, d;			// tobbnyire indexelesre hasznalt valtozok
	double **alfajm;			// erkezesi intenzitas a j linken, felteve m szabad csatornat
	double **alr;				// erkezesi intenzitas
	double **qjm;				// m szabad csatorna allapotvaloszinusege
	double rblock, prevblock, actblock = 0.0;	// blokkolasertekek (iteracio miatt tobb van)
	double signum;				// signum fuggveny
								/* double ntag, dtag, bintagprod, sum;
	int Mjsuc, b, jsuc;*/
	int Mj; 				// szalak szama a j linken
	Path *pa;					// ut
	LinkList *p;				// link
	double blocking_prob = 0.0; // a kimenet
	double firstblock;
	*(poiblocking_prob) = 0.0;
	
	/* Allocate memory for the arrays */
	alfajm = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_multiindep_complex");
	alr = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_multiindep_complex");
	qjm = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_multiindep_complex");
	betaji = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_multiindep_complex");
	jim = (double ***)MMmalloc(sizeof(double**)*no_of_nnlinks, "theo_multiindep_complex");
	
	for(j=0; j<no_of_nnlinks; j++)
	{
		Mj = link_vect[nnlink_vect[j]]->fiber_num;
		alfajm[j] = (double *)MMmalloc(sizeof(double)*(Mj * f + 1), "theo_multiindep_complex");
		alr[j] = (double *)MMmalloc(sizeof(double)*(Mj * f + 1), "theo_multiindep_complex");
		qjm[j] = (double *)MMmalloc(sizeof(double)*(Mj * f + 1), "theo_multiindep_complex");
		betaji[j] = (double *)MMmalloc(sizeof(double)*(Mj * f + 1), "theo_multiindep_complex");
		jim[j] = (double **)MMmalloc(sizeof(double*)*(f + 1), "theo_multiindep_complex");
		
		for(i = 0; i < f + 1; i++)
		{
			jim[j][i] = (double *)MMmalloc(sizeof(double)*(Mj * f + 1),
				"theo_multiindep_complex");
			for(mf = 0; mf < Mj * f + 1; mf++)
				jim[j][i][mf] = getjim(Mj, i, mf, f);
		}
	}
	
	
	betaji[0][0] = -1.0;
	*(iternum) = 0;
	
	/* Iteracio kerete */
	do
	{
		/* inicializalas */
		for(j = 0; j < no_of_nnlinks; j++)
		{
			for(mf = 0; mf < link_vect[nnlink_vect[j]]->fiber_num * f + 1; mf++)
			{
				alr[j][mf] = 0.0;
			}
		}
		/* terhelesek es Markov lanc meretek osszegzese 
		(megfeleloen az egymasrahatasbol adodo csokkenesnek) */
		for(s = 0; s < no_of_cgens; s++)
		{
			for(d = 0; d < no_of_cgens; d++)
			{
				if(callgen_vect[s]->type == CG_TYPE10 ||
					callgen_vect[s]->type == CG_TYPE11 ||
					callgen_vect[s]->type == CG_TYPE12) continue;
				if(NULL == (pa = path_mat[(&callgen_vect[s]->choosen_routing)->
					path_order_type][s][d])) continue;
				for(p = pa->path; p != NULL; p = p->next)
				{
					if(p->link_elem->is_node_node)
					{
						for(mf = 0; mf < p->link_elem->fiber_num * f + 1; mf++)
						{
						/* the intensity will be summed, but we had to use the 
							reduced value (reduction because of intereffect of links) */
							alr[p->link_elem->nnid][mf] += load * load_conv_fact * 
								traff_pattern[callgen_vect[s]->ref_user->id]
								[callgen_vect[d]->ref_user->id] *  
								prxr(pa, p, f, mf, p->link_elem->fiber_num);
/*							switch(callgen_vect[s]->type)
							{
							case CG_TYPE6:
							case CG_TYPE9:
								break;
							case CG_TYPE7:
								break;
								default: fprintf(stderr, 
										 "\nThe theoretical analysis works only for ASON sources!\n");
									exit(1);
							}*/
						}
					}
					
				}
			}
		}
		
		/* minden linkre */
		// alfak es n-ek meghatarozasa
		for(j = 0; j < no_of_nnlinks; j++) 
		{
			Mj = link_vect[nnlink_vect[j]]->fiber_num;
			for(mf = 0; mf < Mj * f + 1; mf++)
			{
				// Poisson forgalomnal:
				alfajm[j][mf] = alr[j][mf];
			}
			
			//  minden terhelesre q allapotvaloszinusegek ertekek meghatarozasa
			qjm[j][0] = 1;
			for(mf = 1; mf < Mj * f + 1; mf++) //szabad csatornak szama
			{
				qjm[j][mf] = qjm[j][mf-1] * (Mj* f + 1 - mf) / alfajm[j][mf];
				qjm[j][0] += qjm[j][mf];
			}
			/* Normalization and assign of mean arrival rates */
			for(mf = 1; mf < Mj * f + 1; mf++)
			{
				qjm[j][mf] /= qjm[j][0];
			}
			qjm[j][0] = 1 / qjm[j][0];
			
			/* uj betak es etak meghatarozasa minden darabszamra */
			
			Mj = link_vect[nnlink_vect[j]]->fiber_num;
			betaji[j][0] = 1.0; /* prxr call not for the first time */
			for(i = 1; i < f+1; i++)
			{
				betaji[j][i] = 0.0;
				for(mf = i; mf < Mj * f + 1; mf++)
				{
					betaji[j][i] += qjm[j][mf] * jim[j][i][mf];
				}
			}
			
		} /* for j */
		
		prevblock = actblock;
		actblock = 0.0;
		
		/* recalculate blocking */
		for(s = 0; s < no_of_cgens; s++)
		{
			for(d = 0; d < no_of_cgens; d++)
			{
				if(callgen_vect[s]->type == CG_TYPE10 ||
					callgen_vect[s]->type == CG_TYPE11 ||
					callgen_vect[s]->type == CG_TYPE12) continue;
				if(NULL == (pa = path_mat[(&callgen_vect[s ]->choosen_routing)->
					path_order_type][s][d])) continue;
				signum = 1.0;
				rblock = 1.0;
				/* calculate and sum for each i wavelength set */
				for(i = 1; i < f+1; i++)
				{
					rblock -= signum * fbinom(f, i) * gri(pa, i);
					signum *= -1.0;
				}
				actblock += rblock  * traff_pattern
					[callgen_vect[s]->ref_user->id][callgen_vect[d]->ref_user->id];
			}
		}
		
		if(*(iternum) == 0)
			firstblock = actblock;
		
		*(iternum) = *(iternum)+1;
	}
	while(fabs(prevblock-actblock) > 10E-6 && *iternum<100);
	
	for(j=0; j<no_of_nnlinks; j++)
	{
		free(alfajm[j]);
		free(alr[j]);
		free(qjm[j]);
		free(betaji[j]);
		for(i=0; i<f +1; i++)
			free(jim[j][i]);
		free(jim[j]);
	}
	
	free(alfajm);
	free(alr);
	free(qjm);
	free(betaji);
	free(jim);
	
	*(poiblocking_prob) = actblock;
	return firstblock; /*prevblock;*/
}


/*-----------------------------------------------------------------------------------*/

/*
double theo_bpp_complex(int f):
Kepes kezelni nem Poisson tipusu forgalmakat is, BPP modellt hasznalva.
Egyebkent majdnem ugyanazt csinalja mint a multimost.
*/
double theo_bpp_complex(int f, double *poiblocking_prob, int *iternum)
{
	int i, j, jsuc, mf, s, d, k;// tobbnyire indexelesre hasznalt valtozok
	/*double dn[MAX_LINK];		// tomb a linkenkenti Markov lancok meretenek reszszamitasaihoz*/
	double **alfajm;			// erkezesi intenzitas a j linken, felteve m szabad csatornat
	double **alr;				// erkezesi intenzitas
	double *sigma2;				// atlagos erkezes szorasnegyzete
	double **qjm;				// m szabad csatorna allapotvaloszinusege
	int **neighbours;			// hany utban szomszedos az adott linkpar
	double rblock, prevblock, actblock = 0.0;	// blokkolasertekek (iteracio miatt tobb van)
	double signum;				// signum fuggveny
	double sum;					// szamolaskor hasznalt segedvaltozok
	double Zmean;				// mean peakedness on the given link
	int Mj, Mjsuc, Mk;			// szalak szama a j, jsuc, ill k linken
	Path *pa;					// ut
	LinkList *p;				// link
	Boolean k_is_in;
    CGType7 *ci7;
	CGType9 *ci9;

	double pnoloss, nlcgload, puj, now, rocap, dirprob;
	double ***cgl;				// load of sd pair on the link
	double cgload, n, ro;
	int N;
	double *offeredtrf;
	double **qarr;				// state probabilities seen by arrivals

	double blocking_prob = 0.0; // a kimenet
	double firstblock;
	*(poiblocking_prob) = 0.0;
	
	/* Allocate memory for the arrays */
	alfajm = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_bpp_complex");
	alr = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_bpp_complex");
	sigma2 = (double *)MMmalloc(sizeof(double)*no_of_nnlinks, "theo_bpp_complex");
	qjm = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_bpp_complex");
	qarr = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_bpp_complex");
	betaji = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_bpp_complex");
	etaji = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_bpp_complex");
	lambdajk = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_bpp_complex");
	pcl1 = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_bpp_complex");
	pcl2 = (double ****)MMmalloc(sizeof(double***)*no_of_nnlinks, "theo_bpp_complex");
	pn = (double ***)MMmalloc(sizeof(double***)*no_of_nnlinks, "theo_bpp_complex");
	neighbours = (int **)MMmalloc(sizeof(int*)*no_of_nnlinks, "theo_bpp_complex");
	
	gM0 = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_bpp_complex");
	gM1 = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_bpp_complex");
	jim = (double ***)MMmalloc(sizeof(double**)*no_of_nnlinks, "theo_bpp_complex");
	
	offeredtrf = (double*)MMmalloc(sizeof(double)*no_of_nnlinks, "theo_bpp_complex");

	for(j=0; j<no_of_nnlinks; j++)
	{
		Mj = link_vect[nnlink_vect[j]]->fiber_num;
		alfajm[j] = (double *)MMmalloc(sizeof(double)*(Mj * f + 1), "theo_bpp_complex");
		alr[j] = (double *)MMmalloc(sizeof(double)*(Mj * f + 2), "theo_bpp_complex");
		qjm[j] = (double *)MMmalloc(sizeof(double)*(Mj * f + 1), "theo_bpp_complex");
		qarr[j] = (double *)MMmalloc(sizeof(double)*(Mj * f + 1), "theo_bpp_complex");
		betaji[j] = (double *)MMmalloc(sizeof(double)*(Mj * f + 1), "theo_bpp_complex");
		etaji[j] = (double *)MMmalloc(sizeof(double)*(Mj * f + 1), "theo_bpp_complex");
		lambdajk[j] = (double *)MMmalloc(sizeof(double)*(no_of_nnlinks),
			"theo_bpp_complex");
		pcl1[j] = (double *)MMmalloc(sizeof(double)*(Mj + 1), "theo_bpp_complex");
		pcl2[j] = (double ***)MMmalloc(sizeof(double**)*(no_of_nnlinks),
			"theo_bpp_complex");
		pn[j] = (double **)MMmalloc(sizeof(double*)*(no_of_nnlinks),
			"theo_bpp_complex");
		neighbours[j] = (int *)MMmalloc(sizeof(int)*no_of_nnlinks, "theo_bpp_complex");
		gM0[j] = (double *)MMmalloc(sizeof(double)*no_of_nnlinks, "theo_bpp_complex");
		gM1[j] = (double *)MMmalloc(sizeof(double)*no_of_nnlinks, "theo_bpp_complex");
		jim[j] = (double **)MMmalloc(sizeof(double*)*(f + 1), "theo_bpp_complex");
		
		for(jsuc = 0; jsuc<no_of_nnlinks; jsuc++)
		{
			pcl2[j][jsuc] = (double **)MMmalloc(sizeof(double*)*(Mj + 1),
				"theo_bpp_complex");
			pn[j][jsuc] = (double *)MMmalloc(sizeof(double)*(Mj + 1),
				"theo_bpp_complex");
			
			for(i = 0; i < Mj + 1; i++)
				pcl2[j][jsuc][i] = (double *)MMmalloc(sizeof(double)*(Mj + 1),
				"theo_bpp_complex");
			neighbours[j][jsuc] = 0;
		}
		for(i = 0; i < f + 1; i++)
		{
			jim[j][i] = (double *)MMmalloc(sizeof(double)*(Mj * f + 1),
				"theo_bpp_complex");
			for(mf = 0; mf < Mj * f + 1; mf++)
				jim[j][i][mf] = getjim(Mj, i, mf, f);
		}
		for(mf = 0; mf < Mj * f; mf++)
		{
			qjm[j][mf] = 0.0;
			qarr[j][mf] = 0.0;
		}
		qjm[j][Mj * f] = 1.0;
		qarr[j][Mj * f] = 1.0;
		
	}

	cgl = (double ***)MMmalloc(sizeof(double**)*no_of_cgens, "theo_bpp_complex");
	
	/* set neighbourhood information */
	for(s = 0; s < no_of_cgens; s++)
	{
		cgl[s] = (double **)MMmalloc(sizeof(double*)*no_of_cgens, "theo_bpp_complex");
		for(d = 0; d < no_of_cgens; d++)
		{
			cgl[s][d]=(double *)MMmalloc(sizeof(double)*no_of_nnlinks, "theo_bpp_complex");
			if(callgen_vect[s]->type == CG_TYPE10 ||
				callgen_vect[s]->type == CG_TYPE11 ||
				callgen_vect[s]->type == CG_TYPE12) continue;
			if(NULL == (pa = path_mat[(&callgen_vect[s]->choosen_routing)->
				path_order_type][s][d])) continue;
			for(p = pa->path->next; p->next->link_elem->is_node_node; p = p->next)
			{
				neighbours[p->link_elem->nnid]
					[p->next->link_elem->nnid]++;
				neighbours[p->next->link_elem->nnid]
					[p->link_elem->nnid]++;
			}
		}
	}
	
	betaji[0][0] = -1.0;
	*(iternum) = 0;
	

	
	/* Iteracio kerete */
	do
	{
		for(j = 0; j < no_of_nnlinks; j++)
		{
			for(mf = 0; mf < link_vect[nnlink_vect[j]]->
				fiber_num * f + 2; mf++)
					alr[j][mf] = 0.0;
			sigma2[j] = 0.0;
		}
		/* terhelesek es Markov lanc meretek osszegzese 
		(megfeleloen az egymasrahatasbol adodo csokkenesnek) */
		for(s = 0; s < no_of_cgens; s++)
		{
			for(d = 0; d < no_of_cgens; d++)
			{
				if(callgen_vect[s]->type == CG_TYPE10 ||
					callgen_vect[s]->type == CG_TYPE11 ||
					callgen_vect[s]->type == CG_TYPE12) continue;
				if(NULL == (pa = path_mat[(&callgen_vect[s]->choosen_routing)->
					path_order_type][s][d])) continue;
				cgload = load * 
					load_conv_fact * traff_pattern[callgen_vect[s]->
					ref_user->id][callgen_vect[d]->ref_user->id];
				if(callgen_vect[s]->type == CG_TYPE9 )
					ci9 = (CGType9 *)callgen_vect[s]->info;
				if(callgen_vect[s]->type == CG_TYPE7 )
					ci7 = (CGType7 *)callgen_vect[s]->info;
				
				for(p = pa->path; p != NULL; p = p->next)
				{
					if(p->link_elem->is_node_node)
					{
						Mj = p->link_elem->fiber_num;
						pnoloss = 0.0;
						for(mf = 0; mf < Mj * f + 1; mf++)
						{
							puj = cor_pujrxr_multi(pa, p, f, mf);
							/* calculate the probability of nonloss */
							pnoloss += puj * qarr[p->link_elem->nnid][mf];
							alr[p->link_elem->nnid][mf] += puj * cgload;
						}
						/* mean arrival intensity is calculated for the not full states */
						switch(callgen_vect[s]->type)
						{
						case CG_TYPE6:
							{
								nlcgload = cgload * pnoloss;
								alr[p->link_elem->nnid][Mj * f + 1] += nlcgload;
								sigma2[p->link_elem->nnid] += nlcgload;
							}
							break;
						case CG_TYPE7:
							{
								now = ci7->no_of_waves * 
									destination_prob[callgen_vect[s]->ref_user->id]
									[callgen_vect[d]->ref_user->id];
								/* determine the arrival rate of an equivalent Poisson traffic */							
								nlcgload = cgload * pnoloss * now / 
									(now - (1.0 - pnoloss)*cgload);
								alr[p->link_elem->nnid][Mj * f + 1] += nlcgload;
								sigma2[p->link_elem->nnid] += nlcgload * 
									(1.0 - nlcgload/now);
							}
							break;
						case CG_TYPE9:
							{
								/* determine the arrival rate of an equivalent Poisson traffic */
								dirprob = traff_pattern[callgen_vect[s]->ref_user->id]
									[callgen_vect[d]->ref_user->id] / 
									source_prob[callgen_vect[s]->ref_user->id];
								rocap = dirprob * pnoloss / (1.0 + 1.0/(ci9->peakedness-1.0) -
									pnoloss * (1.0 - dirprob));
								nlcgload = rocap * cgload / (dirprob *
									(ci9->peakedness - 1.0) * (1.0 - rocap));
								alr[p->link_elem->nnid][Mj * f + 1] += nlcgload;
								sigma2[p->link_elem->nnid] += nlcgload / (1.0 - rocap) ;
							}
							break;
							default: /*fprintf(stderr, 
									 "\nThe theoretical analysis works only for ASON sources!\n");
								exit(1)*/;
						}
						/* traffic of this sd pair on the link p */
						cgl[s][d][p->link_elem->nnid] = nlcgload;
					}
					
				}
			}
		}
		
		/* for each link */
		for(j = 0; j < no_of_nnlinks; j++) 
		{
			Mj = link_vect[nnlink_vect[j]]->fiber_num;
			Zmean = sigma2[j] / alr[j][Mj * f + 1];
/* n can be a non-integer ? */
			n = alr[j][Mj * f + 1]/(1.0 - Zmean);
			ro = (1.0 - Zmean) / Zmean;
			
			if(Zmean < 0.99) 
			{
				/* binomial case */
				for(mf = 0; mf < Mj * f + 1; mf++)
				{
					/* calculation of arrival intensity */
					if(Mj* f - mf < n)
						alfajm[j][mf] = ro * (n - (Mj*f - mf)) /* 
						(alr[j][mf]/alr[j][Mj * f + 1])*/;
					else
						alfajm[j][mf] = 0.0;
				}
			}
			else
			{
				if(Zmean < 1.01)
				{
					/* Poisson case */
					for(mf = 0; mf < Mj * f + 1; mf++)
					{
						/* using the load as intensity in each state */
						alfajm[j][mf] = alr[j][Mj * f + 1]/* alr[j][mf]*/;
					}
				}
				else
				{
					/* Pascal case */
					for(mf = 0; mf < Mj * f + 1; mf++)
					{
						alfajm[j][mf] = ro * (n - (Mj*f - mf)) /* 
							(alr[j][mf]/alr[j][Mj * f + 1])*/;
					}
				}
			}
			

			if(Mj*f < n || n < 0)
				N = Mj*f;
			else
				N = (int)n;

			/*   Determine the q state probabilities */
			for(mf = 0; mf < Mj*f - N; mf++)
				/* zero intensity to these states */
				qjm[j][mf] = 0.0;
			
			qjm[j][Mj*f - N] = 1.0;
			for(mf = Mj*f - N + 1; mf < Mj*f + 1; mf++) /* free channels */
			{
				qjm[j][mf] = qjm[j][mf-1] * (Mj*f + 1 - mf) / alfajm[j][mf];
				qjm[j][Mj*f - N] += qjm[j][mf];
			}

			/* offered traffic weighted by state probs */
			offeredtrf[j] = 0.0;
			/* Normalization */
			for(mf = Mj* f - N + 1; mf < Mj * f + 1; mf++)
			{
				qjm[j][mf] /= qjm[j][Mj*f - N];
				offeredtrf[j] += alfajm[j][mf] * qjm[j][mf];
			}
			qjm[j][Mj*f - N] = 1 / qjm[j][Mj*f - N];
			offeredtrf[j] += alfajm[j][Mj*f - N] * qjm[j][Mj*f - N];

			/* Instead of time stationary state probability qjm 
				we should use probabilities seen by arrivals : qarr */

			for(mf = 0; mf < Mj * f + 1; mf++)
			{
				qarr[j][mf] = qjm[j][mf] * alfajm[j][mf] / offeredtrf[j];
			}

			/* Assign mean arrival rates */
			lambdaj[j] = alr[j][Mj * f + 1];
			
			for(k = 0; k < no_of_nnlinks; k++)
			{
				lambdajk[j][k] = 0.0;
			}
		}

		for(k = 0; k < no_of_nnlinks; k++)
		{
			Mk = link_vect[nnlink_vect[k]]->fiber_num;
			for(s = 0; s < no_of_cgens; s++)
			{
				for(d = 0; d < no_of_cgens; d++)
				{
					if(callgen_vect[s]->type == CG_TYPE10 ||
						callgen_vect[s]->type == CG_TYPE11 ||
						callgen_vect[s]->type == CG_TYPE12) continue;
					if(NULL == (pa = path_mat[(&callgen_vect[s]->choosen_routing)->
						path_order_type][s][d])) continue;
					
						/* take care only about the routes which don't contain
					the link k */
					k_is_in = FALSE;
					for(p = pa->path; p != NULL; p = p->next)
					{
						if(p->link_elem->is_node_node &&
							p->link_elem->nnid == k)
							k_is_in = TRUE;
					}
					
					if( !k_is_in )
					{
						for(p = pa->path; p != NULL; p = p->next)
						{
							/* update the value of non correlated traffic on the links */
							if(p->link_elem->is_node_node)
								lambdajk[p->link_elem->nnid][k] +=
									cgl[s][d][p->link_elem->nnid];
						} /* p */
					}
				} /* d */
			} /* s */
		} /* k */
		
		/* uj betak es etak meghatarozasa minden darabszamra */
		for(j = 0; j < no_of_nnlinks; j++)
		{
			Mj = link_vect[nnlink_vect[j]]->fiber_num;
			betaji[j][0] = 1.0; /* prxr call not for the first time */
			for(i = 1; i < f+1; i++)
			{
				betaji[j][i] = 0.0;
				for(mf = i; mf < Mj * f + 1; mf++)
				{
					betaji[j][i] += qarr[j][mf] * jim[j][i][mf];
				}
				etaji[j][i] = betaji[j][i]/betaji[j][i-1];
			}
			
			/* set the probabilities of link states (dependent on other links) */
			for(k = 0; k < Mj + 1; k++)
			{
				pcl1[j][k] = 0.0;
				for( mf = Mj - k; mf < Mj * f - k + 1; mf++)
					pcl1[j][k] += qarr[j][mf] * fbinom(Mj * (f-1), mf - Mj + k) /
					fbinom(Mj * f, mf);
				pcl1[j][k] *= fbinom(Mj, k);
			}
		} /* for j */
		
		/* dependent probablities of link states */
		for(j = 0; j < no_of_nnlinks; j++)
		{
			Mj = link_vect[nnlink_vect[j]]->fiber_num;
			for(jsuc = 0; jsuc < no_of_nnlinks; jsuc++)
			{
				Mjsuc = link_vect[nnlink_vect[jsuc]]->fiber_num;
				/* the probabilities will be used only in case 
				of following links */
				if(neighbours[j][jsuc])
				{
					for(i = 0; i < Mj + 1; i++) /* continuing */
					{
						sum = 0.0;
						for(k = 0; k < Mj - i + 1; k++) /* non continuing */
							pcl2[j][jsuc][i][k] = pcl1[j][i+k] *
							powering(1.0 - lambdajk[j][jsuc]/lambdaj[j],i) *
							powering(lambdajk[j][jsuc]/lambdaj[j],k) *
							fbinom(i+k,i);
						
						for(k = 0; k < Mjsuc - i + 1; k++) /* non continuing */
							/* used in pn as denominator */
							sum += pcl1[jsuc][k+i] *
							powering(1.0 - lambdajk[jsuc][j]/lambdaj[jsuc],i) *
							powering(lambdajk[jsuc][j]/lambdaj[jsuc],k) *
							fbinom(i+k,i);
						
						if(i < Mjsuc + 1 && sum)
							pn[j][jsuc][i] = pcl1[jsuc][Mjsuc] *
							powering(1.0 - lambdajk[jsuc][j]/lambdaj[jsuc],i) *
							powering(lambdajk[jsuc][j]/lambdaj[jsuc],Mjsuc - i) *
							fbinom(Mjsuc,i) / sum;
						else
							pn[j][jsuc][i] = 0.0;
					}
				}
				else
				{
					for(i = 0; i < Mj + 1; i++)
					{
						for(k = 0; k < Mj + 1; k++)
							pcl2[j][jsuc][i][k] = -1.0;
						pn[j][jsuc][i] = -1.0;
					}
				}
				gM0[j][jsuc] = -1.0;
				gM1[j][jsuc] = -1.0;
			} /* for jsuc */
		} /* for j */
		
		
		prevblock = actblock;
		actblock = 0.0;
		
		/* recalculate blocking */
		for(s = 0; s < no_of_cgens; s++)
		{
			for(d = 0; d < no_of_cgens; d++)
			{
				if(callgen_vect[s]->type == CG_TYPE10 ||
					callgen_vect[s]->type == CG_TYPE11 ||
					callgen_vect[s]->type == CG_TYPE12) continue;
				if(NULL == (pa = path_mat[(&callgen_vect[s ]->choosen_routing)->
					path_order_type][s][d])) continue;
				signum = 1.0;
				rblock = 1.0;
				/* calculate and sum for each i wavelength set */
				for(i = 1; i < f+1; i++)
				{
					rblock -= signum * fbinom(f, i) * cor_gri_multi(pa, i);
					signum *= -1.0;
				}
				actblock += rblock  * traff_pattern
					[callgen_vect[s]->ref_user->id][callgen_vect[d]->ref_user->id];
			}
		}
		
		if(*(iternum) == 0)
			firstblock = actblock;
		
		*(iternum) = *(iternum)+1;
/*		printf("iteration: %d, block: %g\n",*(iternum),actblock);*/
	}
	while(fabs(prevblock-actblock) > 10E-6 && *iternum<100);

	for(j=0; j<no_of_nnlinks; j++)
	{
		Mj = link_vect[nnlink_vect[j]]->fiber_num;
		free(alfajm[j]);
		free(alr[j]);
		free(qjm[j]);
		free(qarr[j]);
		free(betaji[j]);
		free(etaji[j]);
		free(lambdajk[j]);
		free(neighbours[j]);
		free(pcl1[j]);
		for(k=0; k<no_of_nnlinks; k++)
		{
			for(i = 0; i < Mj + 1; i++)
				free(pcl2[j][k][i]);
			free(pcl2[j][k]);
			free(pn[j][k]);
		}
		
		for(i=0; i<f +1; i++)
			free(jim[j][i]);
		
		free(pcl2[j]);
		free(pn[j]);
		free(gM0[j]);
		free(gM1[j]);
		free(jim[j]);
	}

	for(j=0; j<no_of_cgens; j++)
	{
		for(k=0; k<no_of_cgens; k++)
			free(cgl[j][k]);
		free(cgl[j]);
	}
	free(cgl);
	
	free(alfajm);
	free(alr);
	free(sigma2);
	free(qjm);
	free(qarr);
	free(betaji);
	free(etaji);
	free(lambdajk);
	free(neighbours);
	free(pcl1);
	free(pcl2);
	free(pn);
	free(jim);
	free(gM0);
	free(gM1);
	free(offeredtrf);

	*(poiblocking_prob) = actblock;
	return firstblock; /*prevblock;*/
	
}

/*-----------------------------------------------------------------------------------*/


/*
double gri(Path* pa, int i, int M): a fuggveny meghatarozza, hogy a pa ut
menten osszefuggo linkeken mekkora valoszinuseggel szabad egy
adott hullamhossz-halmaz (i db), de nem veszi figyelembe a
hullamhossz szintu osszefuggeseit a hivasoknak. M a szalak szama.
*/

double gri(Path* pa, int i)
{
	LinkList* p;
	double g = 1.0;
	
	if(!pa)
		return 0.0;
	
	for(p = pa->path; p != NULL; p = p->next)
	{
		if(p->link_elem->is_node_node)
			g *= betaji[p->link_elem->nnid][i];
	}
	return g;
}

/*-----------------------------------------------------------------------------------*/

/*
double prxr(Path* pa, LinkList* link, int f, int m, int M): a fuggveny meghatarozza,
hogy a pa ut menten osszefuggo linkeken mekkora valoszinuseggel letezik szabad
hullamhossz-halmaz, felteve, hogy a p linken m darab foglalt hh. van
Ha ez az elso hivas, akkor 1-t ad vissza. M a szalak szama.
*/

double prxr(Path* pa, LinkList* jlink, int f, int m, int M)
{
	double g, pr = 0.0, signum = 1.0;
	int i, maxi;
	LinkList* p;
	
	if(betaji[0][0] < 0.0)
		return 1.0;
	
	if(m < f)
		maxi = m;
	else
		maxi = f;
	
	for(i = 1; i < maxi + 1; i++)
	{
		g = jim[jlink->link_elem->nnid][i][m] / 
			betaji[jlink->link_elem->nnid][i];
		for(p = pa->path; p != NULL; p = p->next)
		{
			if(p->link_elem->is_node_node)
				g *= betaji[p->link_elem->nnid][i];
		}
		pr += signum * fbinom(f, i) * g;
		signum *= -1.0;
	}
	return pr;
}

/*-----------------------------------------------------------------------------------*/

/*
double cor_gri(Path* pa, int i): a fuggveny meghatarozza, hogy a pa ut
menten osszefuggo linkeken mekkora valoszinuseggel szabad egy
adott hullamhossz-halmaz (i db), figyelembe veszi a
hullamhossz szintu osszefuggeseit a hivasoknak (korrelacios modell) !
*/

double cor_gri(Path* pa, int i)
{
	LinkList* p;
	double g = 1.0;
	int j, k;
	
	if(!pa)
		return 0.0;
	
	
	for(p = pa->path; p != NULL; p = p->next)
	{
		if(p->link_elem->is_node_node)
		{
			j = p->link_elem->nnid;
			if(	p->next->link_elem->is_node_node)
			{
				for(k = 1; k < i+1; k++)
					g *= etaji[j][k] /
					( etaji[j][k] +
					(lambdajk[j]
					[p->next->link_elem->nnid]/
					lambdaj[j]) *
					(1 - etaji[j][k]));
			}
			else
				g *= betaji[j][i];
		}
	}
	return g;
}

/*-----------------------------------------------------------------------------------*/

/*
double cor_prxr(Path* pa, LinkList* link, int f, int m): a fuggveny meghatarozza,
hogy a pa ut menten osszefuggo linkeken mekkora valoszinuseggel letezik szabad
hullamhossz-halmaz, felteve, hogy a p linken m darab foglalt hh. van.
Figyelembe veszi a hullamhosszak miatti korrelaciot is!
Ha ez az elso hivas, akkor 1-t ad vissza.
*/

double cor_prxr(Path* pa, LinkList* jlink, int f, int m)
{
	double g, pr = 0.0, signum = 1.0;
	int i, j, k;
	LinkList* p;
	
	if(betaji[0][0] < 0.0)
		return 1.0;
	
	for(i = 1; i < m+1; i++)
	{
		g = 1.0;
		for(p = pa->path; p != NULL; p = p->next)
		{
			if(p->link_elem->is_node_node)
			{
				j = p->link_elem->nnid;
				if(p->next->link_elem->is_node_node &&
					jlink != p)
				{
					for(k = 1; k < i+1; k++)
						g *= etaji[j][k] /
						( etaji[j][k] +
						(lambdajk[j]
						[p->next->link_elem->nnid]/
						lambdaj[j]) *
						(1 - etaji[j][k]));
				}
				else
				{
					if(jlink != p)
						g *= betaji[j][i];
				}
			}
		}
		pr += signum * fbinom(m, i) * g;
		signum *= -1.0;
	}
	return pr;
}


/*-----------------------------------------------------------------------------------*/

/*
double cor_gri_multi(Path* pa, int i): a fuggveny meghatarozza, hogy a pa ut
menten osszefuggo linkeken mekkora valoszinuseggel szabad egy
adott hullamhossz-halmaz (i db), figyelembe veszi a
hullamhossz szintu osszefuggeseit a hivasoknak (korrelacios modell) !
tobbszalas modellben is alkalmazhato
*/
double cor_gri_multi(Path* pa, int i)
{
	LinkList* p;
	double g = 1.0;
	double gammaM0; // gamma^0_{j,j-1} multifiberes esetre
	double gammaM1; // gamma^1_{j,j-1} multifiberes esetre
	int j, jsuc;	// az aktualis es a rakovetkezo link indexe
	int	k;		// hullamhosszindex
	
	if(!pa)
		return 0.0;
	
	
	for(p = pa->path; p != NULL; p = p->next)
	{
		if(p->link_elem->is_node_node)
		{
			j = p->link_elem->nnid;
			if(	p->next->link_elem->is_node_node)
			{
				jsuc = p->next->link_elem->nnid;
				gammaM0 = gamM0(jsuc, j);
				gammaM1 = gamM1(jsuc, j);
				for(k = 1; k < i+1; k++)
				{
					if(betaji[j][1]<1.0)
						g *= betaji[j][k] * (1.0 - gammaM0/betaji[j][1]) /
							( betaji[j][k-1] - betaji[j][k]*gammaM0/betaji[j][1] - 
							(betaji[j][k-1]-betaji[j][k])*gammaM1/(1.0-betaji[j][1]));
					else
						g *= betaji[j][k] * (1.0 - gammaM0/betaji[j][1]) /
						( betaji[j][k-1] - betaji[j][k]*gammaM0/betaji[j][1] );

				}
			}
			else
				g *= betaji[j][i];
		}
	}
	return g;
}

/*-----------------------------------------------------------------------------------*/

/*
double cor_prxr_multi(Path* pa, LinkList* link, int f, int m): a fuggveny meghatarozza,
hogy a pa ut menten osszefuggo linkeken mekkora valoszinuseggel letezik szabad
hullamhossz-halmaz, felteve, hogy a p linken m darab foglalt hh. van.
Figyelembe veszi a hullamhosszak miatti korrelaciot is!
Ha ez az elso hivas, akkor 1-t ad vissza.
tobbszalas modellben is alkalmazhato
*/
double cor_prxr_multi(Path* pa, LinkList* jlink, int f, int m)
{
	double g, pr = 0.0, signum = 1.0, prob;
	int i, maxi, k;
	LinkList* p;
	double gammaM0; // gamma^0_{j,j-1} multifiberes esetre
	double gammaM1; // gamma^1_{j,j-1} multifiberes esetre
	int j, jsuc;	// az aktualis es a rakovetkezo link indexe
	int Mj;			// szalak szama a j linken
	
	if(betaji[0][0] < 0.0)
		return 1.0;
	
	Mj = link_vect[nnlink_vect[jlink->link_elem->nnid]]->fiber_num;
	
	if(m < f)
		maxi = m;
	else
		maxi = f;
	
	for(i = 1; i < maxi + 1; i++)
	{
		g = jim[jlink->link_elem->nnid][i][m] /
			betaji[jlink->link_elem->nnid][i];
		for(p = pa->path; p != NULL; p = p->next)
		{
			if(p->link_elem->is_node_node)
			{
				j = p->link_elem->nnid;
				/* multiplicate by beta'*/
				if(p->next->link_elem->is_node_node) 
				{
					jsuc = p->next->link_elem->nnid;
					gammaM0 = gamM0(jsuc, j);
					gammaM1 = gamM1(jsuc, j);
					for(k = 1; k < i+1; k++)
					{
						g *= betaji[j][k] * (1.0 - gammaM0/betaji[j][1]) /
							( betaji[j][k-1] - betaji[j][k]*gammaM0/betaji[j][1] - 
							(betaji[j][k-1]-betaji[j][k])*gammaM1/(1.0-betaji[j][1]));
/*						if(prob > 1.0)
						{
							printf("anyad\n");
							exit(1);
						}
						g *= prob;*/
					}
				}
				else /* last link of route */
					g *= betaji[j][i];
			}
		}
		pr += signum * fbinom(f, i) * g;
		signum *= -1.0;
	}
	return pr;
}


/*-----------------------------------------------------------------------------------*/

/*
double cor_pujrxr_multi(Path* pa, LinkList* link, int f, int m): a fuggveny meghatarozza,
hogy a (pa ut - p link) menten osszefuggo linkeken mekkora valoszinuseggel letezik szabad
hullamhossz-halmaz, felteve, hogy a p linken m darab foglalt hh. van.
Figyelembe veszi a hullamhosszak miatti korrelaciot is!
Ha ez az elso hivas, akkor 1-t ad vissza.
tobbszalas modellben is alkalmazhato
*/
double cor_pujrxr_multi(Path* pa, LinkList* jlink, int f, int m)
{
	double pr = 0.0, signum = 1.0, g, gcomp, prob, tag3;
	int i, maxi, k, thelink;
	LinkList* p;
	double gammaM0; // gamma^0_{j,j-1} multifiberes esetre
	double gammaM1; // gamma^1_{j,j-1} multifiberes esetre
	int j, jsuc;	// az aktualis es a rakovetkezo link indexe
	int Mj;			// szalak szama a j linken
	
	if(betaji[0][0] < 0.0)
		return 1.0;
	
	thelink = jlink->link_elem->nnid;

	Mj = link_vect[nnlink_vect[jlink->link_elem->nnid]]->fiber_num;
	
	for(i = 1; i < f + 1; i++)
	{
		g = jim[thelink][i][m] / 
			betaji[thelink][i];
		gcomp = (1.0 - jim[thelink][i][m]) / 
			(1.0 - betaji[thelink][i]);

		for(p = pa->path; p != NULL; p = p->next)
		{
			if(p->link_elem->is_node_node)
			{
				j = p->link_elem->nnid;
				/* multiplicate by beta'*/
				if(p->next->link_elem->is_node_node) 
				{
					jsuc = p->next->link_elem->nnid;
					gammaM0 = gamM0(jsuc, j);
					gammaM1 = gamM1(jsuc, j);
					prob = 1.0;
					for(k = 1; k < i+1; k++)
					{
						if(betaji[j][1] < 1.0)
							tag3 = gammaM1/(1.0-betaji[j][1]);
						else
							tag3 = 0.0;
						prob *= betaji[j][k] * (1.0 - gammaM0/betaji[j][1]) /
							( betaji[j][k-1] - betaji[j][k]*gammaM0/betaji[j][1] - 
							(betaji[j][k-1]-betaji[j][k])*tag3);
					}
					g *= prob;
					
					if(thelink == j)
						gcomp *= (1.0 - prob);
					else
					{
						if(thelink == jsuc)
						{
							gammaM0 = gamM0(j, jsuc);
							gammaM1 = gamM1(j, jsuc);
							prob = 1.0;
							for(k = 1; k < i+1; k++)
							{
								if(betaji[jsuc][1] < 1.0)
									tag3 = gammaM1/(1.0-betaji[jsuc][1]);
								else
									tag3 = 0.0;
								prob *= betaji[jsuc][k] * (1.0 - gammaM0/betaji[jsuc][1]) /
									( betaji[jsuc][k-1] - betaji[jsuc][k]*gammaM0/betaji[jsuc][1] - 
									(betaji[jsuc][k-1]-betaji[jsuc][k])*tag3);
							}
							if(betaji[jsuc][i] < 1.0)
								gcomp *= (betaji[j][i] - prob * betaji[j][i]) / 
									(1.0 - betaji[jsuc][i]);
							else
								gcomp = 0.0;
						}
						else
							gcomp *= prob;
					}

				}
				else /* last link of route */
				{
					g *= betaji[j][i];
					if(thelink != j)
						gcomp *= betaji[j][i];
					else
						gcomp *= (1.0 - betaji[j][i]);
				}
			}
		}
		pr += signum * fbinom(f, i) * (g + gcomp);
		signum *= -1.0;
	}
	if(pr>1.0)
	{
		if(pr - 1.0 > 0.0001)
			printf("prob>1:  %.10g\n",pr - 1.0);
		pr = 1.0;
	}
	if(pr <0.0)
	{
/*		printf("prob<0:  %g.10\n",pr);*/
		pr = 0.0;
	}
	return pr;
}

/*-----------------------------------------------------------------------------------*/

/*
double gamM0(int j, int jsuc): a fuggveny meghatarozza,
hogy mekkora valoszinuseggel lesz szabad egy adott hh. a j liken es 
foglalt a jsuc linken
tobbszalas modellben (is) alkalmazhato
*/
double gamM0(int jsuc, int j)
{
	int k, i, maxi;
	int Mj, Mjsuc;
	double sum = 0.0;
	
	if(gM0[jsuc][j] == -1)
	{
		
		Mj = link_vect[nnlink_vect[j]]->fiber_num;
		Mjsuc = link_vect[nnlink_vect[jsuc]]->fiber_num;
		
		for(k = 0; k < Mj; k++ )
		{
			if(Mjsuc < k)
				maxi = Mjsuc;
			else
				maxi = k;
			
			for(i = 0; i < maxi + 1; i++)
				sum += pn[j][jsuc][i] * pcl2[j][jsuc][i][k-i];
		}
		gM0[jsuc][j] = sum;
	}
	return gM0[jsuc][j];
}


/*-----------------------------------------------------------------------------------*/

/*
double gamM1(int j, int jsuc): a fuggveny meghatarozza,
hogy mekkora valoszinuseggel lesz foglalt egy adott hh. a j linken es 
a jsuc linken is.
tobbszalas modellben (is) alkalmazhato
*/
double gamM1(int jsuc, int j)
{
	int k, minM;
	int Mj, Mjsuc;
	double sum = 0.0;
	
	
	if(gM1[jsuc][j] == -1)
	{
		Mj = link_vect[nnlink_vect[j]]->fiber_num;
		Mjsuc = link_vect[nnlink_vect[jsuc]]->fiber_num;
		
		if(Mj > Mjsuc)
			minM = Mjsuc;
		else
			minM = Mj;
		
		for(k = 0; k < minM + 1; k++)
			sum += pn[j][jsuc][k] * pcl2[j][jsuc][k][Mj-k];
		
		
		gM1[jsuc][j] = sum;
	}
	return gM1[jsuc][j];
}

/* double getjim (int Mj, int i, int mf, int f):
A fuggveny kiszamolja, hogy az Mj*f kapacitasu linken
mekkora valoszinuseggel lesz szabad egy adott i hullamhossz-
halmaz, felteve mf szabad csatornat */
double getjim (int Mj, int i, int mf, int f)
{
	int maxk, k;
	double jimmy = 0.0;
	double signum = 1.0;
	
	if( i > mf)
		return 0.0;
	
	if( i == 0)
		return 1.0;
	
	if( Mj == 1)
		return fbinom(mf, i) / fbinom(f, i);
	else
	{
	/* If there are more than 1 fibers on the link
	the number of the not available wavelengths in the
		set i can not be bigger then: */
		maxk = (Mj * f - mf)/Mj;
		
		/* and of course it is less then i too */
		if(i < maxk )
			maxk = i;
		
		for(k = 0; k < maxk + 1; k++)
		{
			jimmy += signum * fbinom(i,k) * 
				fbinom(Mj * f - Mj * k, Mj * f - Mj * k - mf)/
				fbinom(Mj * f, Mj * f - mf);
			signum *= -1.0;
		}
	}
	return jimmy;
}

/* Hatvanyozas */
double powering(double a, int i)
{
	double ret = 1.0;
	int k;
	
	if (i < 0)
	{
		for(k=0; k<-i; k++)
			ret /= a;
	}
	else
	{
		for(k=0; k<i; k++)
			ret *= a;
	}
	return ret;
}


/*-----------------------------------------------------------------------------------*/

/*
double theo_lisomani_complex(int W): 
A Li-Somani cikk szerint szamol, linkkapacitasnak a linkek
atlagkapcitasat (egesz szamu szalra kerekitve) veszi
W: hullamhosszak szama egy szalon
Visszaadja a szamolt blokkolast.
*/


double theo_lisomani_complex(int inW)
{
	int i, j, jprev, s, d, w;	// tobbnyire indexelesre hasznalt valtozok
	Path *pa;					// ut
	LinkList *p;				// link
	int z;					// atlagos szalszam
	double blocking;
	
	int maxNcap_f_l, Ncap_f_l, y_f_l;
	int x_f_l1, z_c_l, max_zcl, Ncap_f_l1, maxNcapfl1;
	double U, S;
	double** P_6;
	double** P_6prev;
	double *pnorm;			/* normalizing factor of 1 dim MC */
	
	/* Allocate memory for the arrays */
	/*  lambdaej = (double *)MMmalloc(sizeof(double)*no_of_nnlinks, "theo_bpp_complex");
	lambdalj = (double *)MMmalloc(sizeof(double)*no_of_nnlinks, "theo_bpp_complex");*/
	lambdacjk = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_lisomani_complex");
	pidenom = (double **)MMmalloc(sizeof(double*)*no_of_nnlinks, "theo_lisomani_complex");
	pnorm = (double *)MMmalloc(sizeof(double)*no_of_nnlinks, "theo_lisomani_complex");
	
	
	for(j=0; j<no_of_nnlinks; j++)
	{
		lambdacjk[j] = (double *)MMmalloc(sizeof(double)*(no_of_nnlinks),
			"theo_lisomani_complex");
		pidenom[j] = (double *)MMmalloc(sizeof(double)*(no_of_nnlinks),
			"theo_lisomani_complex");
		/* calculate the average number of fibers */
		F += link_vect[nnlink_vect[j]]->fiber_num;
		
		lambdaj[j] = 0.0;
		for(jprev=0; jprev<no_of_nnlinks; jprev++)
			lambdacjk[j][jprev] = 0.0;
	}
	
	F /= no_of_nnlinks;
	W = inW;
	
	P_6 = (double **)MMmalloc(sizeof(double*)*(W+1), "theo_lisomani_complex");
	P_6prev = (double **)MMmalloc(sizeof(double*)*(W+1), "theo_lisomani_complex");
	for(z=0; z<W+1; z++)
	{
		P_6[z] = (double *)MMmalloc(sizeof(double)*(F*W+1), "theo_lisomani_complex");
		P_6prev[z] = (double *)MMmalloc(sizeof(double)*(F*W+1), "theo_lisomani_complex");
	}
	
	/* allocate and set the values of function f */
	f_lisomani = (double **)MMmalloc(sizeof(double*)*(F*W+1), "theo_lisomani_complex");
	for(z=0; z<F*W+1; z++)
	{
		f_lisomani[z] = (double *)MMmalloc(sizeof(double)*(W+1), "theo_lisomani_complex");
		
		j = z/F;
		for(w = 0; w < W + 1; w++)
		{
			if(z > w*(F-1) )
				f_lisomani[z][w] = 0;
			else
			{
				f_lisomani[z][w] = fbinom(w*F, z);
				for(i = 1; i < j+1; i++)
					f_lisomani[z][w] -= fbinom(w, i) * f_lisomani[z-i*F][w-i];
			}
		}
		
	}
	
	/* allocate and set the values of function g */
	g_lisomani = (double ***)MMmalloc(sizeof(double**)*(W+1), "theo_lisomani_complex");
	for(s = 0; s < W+1; s++)  /* Ncap_F_2 */
	{
		g_lisomani[s] = (double **)MMmalloc(sizeof(double*)*(F*s+1), "theo_lisomani_complex");
		
		for(z = 0; z < F*s + 1; z++)
		{
			g_lisomani[s][z] = (double *)MMmalloc(sizeof(double)*(F*s-z+1),
				"theo_lisomani_complex");
			for(d = 0; d < F*s - z + 1; d++) /* y_N_f_2 */
			{
				g_lisomani[s][z][d] = fbinom(s*F, z) * fbinom(s*F - z, d);
				
				for(i = 1; i < (z+d)/F+1; i++)
				{
					if(i*F < z)
						jprev = i*F;
					else
						jprev = z;
					
					if(i*F - d > 0)
						w = i*F - d;
					else
						w = 0;
					
					for(j = w; j < jprev+1; j++)
						g_lisomani[s][z][d] -= fbinom(s, i) * fbinom(i*F, j) *
						
						g_lisomani[s-i][z-j][d-(i*F-j)];
				}
			}
		}
		
	}
	
	pi_lis = (double *****)MMmalloc(sizeof(double****)*(F*W+1), "theo_lisomani_complex");
	for(s = 0; s < F*W+1; s++)
	{
		pi_lis[s] = 
			(double ****)MMmalloc(sizeof(double***)*(F*W+1), "theo_lisomani_complex");
		for(d = 0; d < F*W+1; d++)
		{
			pi_lis[s][d] = 
				(double ***)MMmalloc(sizeof(double**)*(F*W+1), "theo_lisomani_complex");
			for(z = 0; z < F*W+1; z++)
			{
				pi_lis[s][d][z] = 
					(double **)MMmalloc(sizeof(double*)*(no_of_nnlinks),
					"theo_bpp_complex");
				for(j = 0; j < no_of_nnlinks; j++)
				{
					pi_lis[s][d][z][j] = 
						(double *)MMmalloc(sizeof(double)*(no_of_nnlinks),
						"theo_bpp_complex");
					for(jprev = 0; jprev < no_of_nnlinks; jprev++)
						pi_lis[s][d][z][j][jprev] = -1.0;
					
				}
			}
		}
	}
	
	P_5_lis = (double ****)MMmalloc(sizeof(double***)*(W+1), "theo_lisomani_complex");
	/*P_4_lis*/
	/*P_3_lis*/
	P_2_lis = (double *****)MMmalloc(sizeof(double****)*(W+1), "theo_lisomani_complex");
	P_1_lis = (double ****)MMmalloc(sizeof(double***)*(W+1), "theo_lisomani_complex");
	RWTLC_lis = (double ****)MMmalloc(sizeof(double***)*(W+1), "theo_lisomani_complex");
	
	for(s = 0; s < W+1; s++)
	{
		P_5_lis[s] = (double ***)MMmalloc(sizeof(double**)*(W+1),
			"theo_lisomani_complex");
		P_2_lis[s] = (double ****)MMmalloc(sizeof(double***)*(W+1), 
			"theo_lisomani_complex");
		P_1_lis[s] = (double ***)MMmalloc(sizeof(double**)*(W+1), 
			"theo_lisomani_complex");
		RWTLC_lis[s] = (double ***)MMmalloc(sizeof(double**)*(W+1), 
			"theo_lisomani_complex");
		for(d = 0; d < W+1; d++)
		{
			P_5_lis[s][d] = (double **)MMmalloc(sizeof(double*)*(F*W+1), 
				"theo_lisomani_complex");
			P_2_lis[s][d] = (double ***)MMmalloc(sizeof(double**)*(F*W+1), 
				"theo_lisomani_complex");
			P_1_lis[s][d] = (double **)MMmalloc(sizeof(double*)*(F*W+1), 
				"theo_lisomani_complex");
			RWTLC_lis[s][d] = (double **)MMmalloc(sizeof(double*)*(F*W+1), 
				"theo_lisomani_complex");
			for(z = 0; z < F*W+1; z++)
			{
				P_5_lis[s][d][z] = (double *)MMmalloc(sizeof(double)*(F*W+1),
					"theo_lisomani_complex");
				P_2_lis[s][d][z] = (double **)MMmalloc(sizeof(double*)*(F*W+1),
					"theo_lisomani_complex");
				P_1_lis[s][d][z] = (double *)MMmalloc(sizeof(double)*(F*W+1),
					"theo_lisomani_complex");
				RWTLC_lis[s][d][z] = (double *)MMmalloc(sizeof(double)*(F*W+1),
					"theo_lisomani_complex");
				for(i = 0; i < F*W+1; i++)
				{
					P_5_lis[s][d][z][i] = -1.0;
					P_2_lis[s][d][z][i] = (double *)MMmalloc(sizeof(double)*(W+1),
						"theo_lisomani_complex");
					P_1_lis[s][d][z][i] = -1.0;
					RWTLC_lis[s][d][z][i] = -1.0;
					for(j = 0; j < W+1; j++)
						P_2_lis[s][d][z][i][j] = -1.0;
					
				}
			}
			
		}
		
	}
	
	
	/* terhelesek es Markov lanc meretek osszegzese */
	for(s = 0; s < no_of_cgens; s++)
	{
		for(d = 0; d < no_of_cgens; d++)
		{
			if(callgen_vect[s]->type == CG_TYPE10 ||
				callgen_vect[s]->type == CG_TYPE11 ||
				callgen_vect[s]->type == CG_TYPE12) continue;
			if(NULL == (pa = path_mat[(&callgen_vect[s]->choosen_routing)->
				path_order_type][s][d])) continue;
			for(p = pa->path; p != NULL; p = p->next)
			{
				if(p->link_elem->is_node_node)
				{
					lambdaj[p->link_elem->nnid] += load * load_conv_fact * 
						traff_pattern[callgen_vect[s]->ref_user->id]
						[callgen_vect[d]->ref_user->id];
					if(p->next->link_elem->is_node_node)
					{
						lambdacjk[p->link_elem->nnid][p->next->link_elem->nnid]
							+= load * load_conv_fact * 
							traff_pattern[callgen_vect[s]->ref_user->id]
							[callgen_vect[d]->ref_user->id];
						lambdacjk[p->next->link_elem->nnid][p->link_elem->nnid]
							+= load * load_conv_fact * 
							traff_pattern[callgen_vect[s]->ref_user->id]
							[callgen_vect[d]->ref_user->id];
					}
					
					
					/* Fontos!!!
					lambdaejk[jprev][j] = lambdaj[j]-lambdacjk[jprev][j] es
					lambdaljk[j][jsuc] = lambdaj[j]-lambdacjk[j][jsuc] */	   
				}
			}
		}
	}
	
	/* Calculate the normalizing factor in the 3D Markov chain 
	and calculate the normalization factor of the 1D Markov Chain */
	
	for(jprev = 0; jprev < no_of_nnlinks; jprev++)
	{
		
		pnorm[jprev] = 1.0;
		U = 1.0;
		for(i = 1; i < F*W+1; i++)
		{
			U *= lambdaj[jprev]/i;
			pnorm[jprev] += U;
		}
		
		for(j = 0; j < no_of_nnlinks; j++)
		{
			pidenom[jprev][j] = 0.0;
			for(i = 0; i < F*W +1; i++)
			{
				for(s = 0; s < F*W + 1 - i; s++)
				{
					for(d = 0; d < F*W + 1 - i; d++)
						pidenom[jprev][j] +=
						(powering(lambdacjk[jprev][j], i) / factorial(i)) *
						(powering(lambdaj[jprev] - lambdacjk[jprev][j], s) 
						/ factorial(s)) *
						(powering(lambdaj[j] - lambdacjk[jprev][j], d) 
						/ factorial(d));
				}
			}
		}
	}
	
	
	
	/* blokkolas meghatarozasa */  
	blocking = 0.0;
	for(s = 0; s < no_of_cgens; s++)
	{
		for(d = 0; d < no_of_cgens; d++)
		{
			if(callgen_vect[s]->type == CG_TYPE10 ||
				callgen_vect[s]->type == CG_TYPE11 ||
				callgen_vect[s]->type == CG_TYPE12) continue;
			if(NULL == (pa = path_mat[(&callgen_vect[s]->choosen_routing)->
				path_order_type][s][d])) continue;
			
			maxNcap_f_l = W;
			p = pa->path->next;
			jprev = p->link_elem->nnid;
			for(Ncap_f_l=0; Ncap_f_l < maxNcap_f_l + 1; Ncap_f_l++)
			{
				for(y_f_l=0; y_f_l < Ncap_f_l; y_f_l++)
					P_6prev[Ncap_f_l][y_f_l] = 0.0;
				
				S = powering(lambdaj[jprev], W*F - Ncap_f_l)/factorial(W*F - Ncap_f_l);
				for(y_f_l=Ncap_f_l; y_f_l < Ncap_f_l*F + 1; y_f_l++)
				{
					P_6prev[Ncap_f_l][y_f_l] = 
						/* szerintem ez rossz!	P_1_lisomani(Ncap_f_l, 0, 0, F*W - y_f_l); */
						(fbinom(W, Ncap_f_l)*
						f_lisomani[Ncap_f_l*F-y_f_l][Ncap_f_l]/
						fbinom(W*F, y_f_l)) *  S / pnorm[jprev];
					
					S *= (W*F - y_f_l)/lambdaj[jprev];
				}
				
				for(y_f_l=Ncap_f_l*F + 1; y_f_l < W*F + 1; y_f_l++)
					P_6prev[Ncap_f_l][y_f_l] = 0.0;
				
				for(y_f_l=0; y_f_l < W*F + 1; y_f_l++)
					P_6[Ncap_f_l][y_f_l] = P_6prev[Ncap_f_l][y_f_l];
				
			}
			
			for(p = pa->path->next->next; p->link_elem->is_node_node; p = p->next)
			{
				
				j = p->link_elem->nnid;
				
				/* for last link only the 0 (blocking) is needed in P_6 */
				if(!p->next->link_elem->is_node_node)
					maxNcap_f_l = 0;
				
				for(Ncap_f_l=0; Ncap_f_l < maxNcap_f_l + 1; Ncap_f_l++)
				{
					for(y_f_l=0; y_f_l < W*F + 1; y_f_l++)
					{
						P_6[Ncap_f_l][y_f_l] = 0;
						for(x_f_l1 = 0; x_f_l1 < W*F + 1; x_f_l1++)
						{
							S = S_lisomani(y_f_l, x_f_l1, j, jprev);
							if(W * F - x_f_l1 > W * F - y_f_l)
								max_zcl = W * F - y_f_l;
							else
								max_zcl = W * F - x_f_l1;
							for(z_c_l = 0; z_c_l < max_zcl + 1; z_c_l++)
							{
								U = U_lisomani(z_c_l, y_f_l, x_f_l1, j, jprev);
								if(x_f_l1 < W)
									maxNcapfl1 = x_f_l1;
								else
									maxNcapfl1 = W;
								for(Ncap_f_l1 = Ncap_f_l; Ncap_f_l1 < 
									maxNcapfl1 + 1; Ncap_f_l1++)
									
									P_6[Ncap_f_l][y_f_l] +=
									P_6prev[Ncap_f_l1][x_f_l1] * S * U *
									RWTLC(Ncap_f_l, Ncap_f_l1, z_c_l, y_f_l);
							}
						}
					}
				}
				for(Ncap_f_l=0; Ncap_f_l < maxNcap_f_l + 1; Ncap_f_l++)
				{
					for(y_f_l=0; y_f_l < W*F + 1; y_f_l++)
					{
						P_6prev[Ncap_f_l][y_f_l] = P_6[Ncap_f_l][y_f_l];
					}
				}
				jprev = j;
			}
			for(y_f_l=0; y_f_l < W*F + 1; y_f_l++)
				blocking += P_6[0][y_f_l] * 
				traff_pattern[callgen_vect[s]->ref_user->id]
				[callgen_vect[d]->ref_user->id];
		} /* for d */
	} /* for s */
	
	
	
	for(j=0; j<no_of_nnlinks; j++)
	{
		free(lambdacjk[j]);
		free(pidenom[j]);
	}
	
	for(z=0; z<W+1; z++)
	{
		free(P_6[z]);
		free(P_6prev[z]);
	}
	
	for(z=0; z<W*F+1; z++)
		free(f_lisomani[z]);
	
	for(s = 0; s < W+1; s++)
	{
		for(z = 0; z < F*s + 1; z++)
			free(g_lisomani[s][z]);
		free(g_lisomani[s]);
	}
	
	free(f_lisomani);
	free(g_lisomani);
	free(P_6);
	free(P_6prev);
	free(lambdacjk);
	free(pnorm);
	
	for(s = 0; s < W+1; s++)
	{
		for(d = 0; d < W+1; d++)
		{
			for(z = 0; z < F*W+1; z++)
			{
				for(i = 0; i < F*W+1; i++)
					free(P_2_lis[s][d][z][i]);
				
				free(P_5_lis[s][d][z]);
				free(P_2_lis[s][d][z]);
				free(P_1_lis[s][d][z]);
				free(RWTLC_lis[s][d][z]);
			}
			free(P_5_lis[s][d]);
			free(P_2_lis[s][d]);
			free(P_1_lis[s][d]);
			free(RWTLC_lis[s][d]);
			
		}
		free(P_5_lis[s]);
		free(P_2_lis[s]);
		free(P_1_lis[s]);
		free(RWTLC_lis[s]);
	}
	free(P_5_lis);
	free(P_2_lis);
	free(P_1_lis);
	free(RWTLC_lis);
	
	for(s = 0; s < F*W+1; s++)
	{
		for(d = 0; d < F*W+1; d++)
		{
			for(z = 0; z < F*W+1; z++)
			{
				for(j = 0; j < no_of_nnlinks; j++)
					free(pi_lis[s][d][z][j]);
				free(pi_lis[s][d][z]);
			}
			free(pi_lis[s][d]);
		}
		free(pi_lis[s]);
	}
	free(pi_lis);
	
	return blocking;
}

/* Az alabbi fuggvenyek megfelelenek a Li- Somani cikk(ek)ben szereplo
kepleteknek */

double U_lisomani(int z_c_2, int y_f_2, int x_f_1, int j, int jprev)
{
	int x_c, maxxc;
	double denom = 0.0;
	
	if(F*W - x_f_1 < F*W - y_f_2)
		maxxc = F*W - x_f_1;
	else
		maxxc = F*W - y_f_2;
	
	for(x_c = 0; x_c < maxxc + 1; x_c++)
		denom += pi_lisomani(W*F - x_f_1 - x_c, x_c,
		W*F - y_f_2 - x_c, j, jprev);
	
	if(denom == 0.0)
		return denom;
	
	return pi_lisomani(W*F - x_f_1 - z_c_2, z_c_2,
		W*F - y_f_2 - z_c_2, j, jprev) / denom;
}

double S_lisomani(int y_f_2, int x_f_1, int j, int jprev)
{
	int x_c, x_e;
	double denom = 0.0;
	double nom = 0.0;
	
	for(x_c = 0; x_c < F*W - x_f_1 + 1; x_c++)
	{
		if(x_c < F*W - y_f_2 + 1)
			nom += pi_lisomani(W*F - x_f_1 - x_c, x_c,
			W*F - y_f_2 - x_c, j, jprev);
		
		for(x_e = 0; x_e < F*W - x_c+1; x_e ++)
			denom += pi_lisomani(W*F - x_f_1 - x_c, x_c, x_e, j, jprev);
	}
	
	return nom / denom;
}

double RWTLC(int Ncap_f_2, int Xcap_f_1, int z_c_2, int y_f_2)
{
	int Zcap_c_2, minzcapc2;
	double ret=0.0;
	
	if(RWTLC_lis[Ncap_f_2][Xcap_f_1][z_c_2][y_f_2] == -1.0)
	{
		if(z_c_2 - W*(F-1) > 0)
			minzcapc2 = z_c_2 - W*(F-1);
		else
			minzcapc2 = 0;
		
		for(Zcap_c_2 = minzcapc2;Zcap_c_2 < z_c_2/F + 1;Zcap_c_2++)
			ret+= P_1_lisomani(Zcap_c_2, Xcap_f_1, z_c_2, y_f_2) *
			P_2_lisomani(Ncap_f_2, Xcap_f_1, z_c_2 - Zcap_c_2 * F, 
			y_f_2, W - Zcap_c_2);
		
		RWTLC_lis[Ncap_f_2][Xcap_f_1][z_c_2][y_f_2] = ret;
	}
	return RWTLC_lis[Ncap_f_2][Xcap_f_1][z_c_2][y_f_2];
}

double P_1_lisomani(int Zcap_c_2, int Xcap_f_1, int z_c_2, int y_f_2)
{
	if(P_1_lis[Zcap_c_2][Xcap_f_1][z_c_2][y_f_2] == -1.0)
	{
		if(Zcap_c_2 + Xcap_f_1 > W)
			P_1_lis[Zcap_c_2][Xcap_f_1][z_c_2][y_f_2] = 0.0;
		if(y_f_2 + z_c_2 > W * F)
			P_1_lis[Zcap_c_2][Xcap_f_1][z_c_2][y_f_2] = 0.0;
			/*	fbinom(W, Zcap_c_2) * f_lisomani[z_c_2 - Zcap_c_2 * F][W] /
		fbinom(W * F, z_c_2); Szerintem hibas!!! */
		P_1_lis[Zcap_c_2][Xcap_f_1][z_c_2][y_f_2] =
			fbinom(W, Zcap_c_2) * f_lisomani[z_c_2 - Zcap_c_2 * F][W-Zcap_c_2] /
			fbinom(W * F, z_c_2);
	}
	return P_1_lis[Zcap_c_2][Xcap_f_1][z_c_2][y_f_2];
}

double P_2_lisomani(int Ncap_f_2, int Xcap_f_1, int zp_c_2, int y_f_2, int w)
{
	int zp_X_b1, y_X_b1, minyxb1, minzpxb1, maxyxb1;
	double ret = 0.0;
	
	if(P_2_lis[Ncap_f_2][Xcap_f_1][zp_c_2][y_f_2][w] == -1.0)
	{
		/* szukseg eseten nem nullatol indulunk! */
		if(zp_c_2 - Xcap_f_1 * (F - 1) > 0)
			minzpxb1 = zp_c_2 - Xcap_f_1 * (F - 1);
		else 
			minzpxb1 = 0;
		
		for(zp_X_b1 = minzpxb1; zp_X_b1 < zp_c_2+1; zp_X_b1++)
		{
			/* szukseg eseten nem nullatol indulunk! */
			if((w-Xcap_f_1)*F - y_f_2 - zp_X_b1 > 0)
				minyxb1 = (w-Xcap_f_1)*F - y_f_2 - zp_X_b1;
			else
				minyxb1 = 0;
			
			/*		if(w*F - zp_c_2- y_f_2 < F*(w - Xcap_f_1) - zp_X_b1)*/
			if(y_f_2 + zp_c_2 > F * Xcap_f_1 + zp_X_b1)
				maxyxb1 = w*F - zp_c_2 - y_f_2;
			else
				maxyxb1 = F*(w - Xcap_f_1) - zp_X_b1;
			
			for(y_X_b1 = minyxb1; y_X_b1 < maxyxb1 + 1; y_X_b1++)
				ret += P_3_lisomani(zp_X_b1, zp_c_2, w - Xcap_f_1, Xcap_f_1, w) *
				P_4_lisomani(y_X_b1, zp_X_b1, zp_c_2 - zp_X_b1, 
				W*F - y_f_2 - zp_c_2 - F*(W-w), w - Xcap_f_1, Xcap_f_1, w) *
				P_5_lisomani(Ncap_f_2, Xcap_f_1, zp_c_2 - zp_X_b1, 
				W*F - y_f_2 - zp_c_2 - F*(W-w) - y_X_b1);
		}
		P_2_lis[Ncap_f_2][Xcap_f_1][zp_c_2][y_f_2][w] = ret;
	}
	return P_2_lis[Ncap_f_2][Xcap_f_1][zp_c_2][y_f_2][w];
}

double P_3_lisomani(int zp_X_b1, int zp_c_2, int Xcap_b_1, int Xcap_f_1, int w)
{
	return f_lisomani[zp_X_b1][Xcap_b_1] * 
		f_lisomani[zp_c_2 - zp_X_b1][Xcap_f_1] / f_lisomani[zp_c_2][w];
}

double P_4_lisomani(int y_X_b1, int zp_X_b1, int zp_X_f1, int y_b2, 
					int Xcap_b_1, int Xcap_f_1, int w)
{
	return fbinom(Xcap_b_1*F - zp_X_b1, y_X_b1) * 
		fbinom(Xcap_f_1*F - zp_X_f1, y_b2 - y_X_b1) /
		fbinom(w*F - zp_X_b1 - zp_X_f1, y_b2);
}

double P_5_lisomani(int Ncap_f_2, int Xcap_f_1, int zp_X_f1, int y_X_f1)
{
	int zp_N_b2, minzpnb2;
	double ret = 0.0;
	
	if(P_5_lis[Ncap_f_2][Xcap_f_1][zp_X_f1][y_X_f1] == -1.0)
	{
		/* szukseg eseten nem nullatol indulunk! */
		if(zp_X_f1 - Ncap_f_2 * (F - 1) > 0)
			minzpnb2 = zp_X_f1 - Ncap_f_2 * (F - 1);
		else 
			minzpnb2 = 0;
		
		if(minzpnb2 < F*(Xcap_f_1 - Ncap_f_2) - y_X_f1)
			minzpnb2 = F*(Xcap_f_1 - Ncap_f_2) - y_X_f1;
		
		for(zp_N_b2 = minzpnb2; zp_N_b2 < zp_X_f1+1; zp_N_b2++)
			ret += fbinom(Xcap_f_1, Ncap_f_2) * f_lisomani[zp_N_b2]
			[Xcap_f_1 - Ncap_f_2] * g_lisomani[Ncap_f_2][zp_X_f1-zp_N_b2]
			[y_X_f1-(F*(Xcap_f_1 - Ncap_f_2) - zp_N_b2)] /
			(f_lisomani[zp_X_f1][Xcap_f_1] * fbinom(Xcap_f_1*F - zp_X_f1, y_X_f1));
		P_5_lis[Ncap_f_2][Xcap_f_1][zp_X_f1][y_X_f1] = ret;
	}
	return P_5_lis[Ncap_f_2][Xcap_f_1][zp_X_f1][y_X_f1];
}

double pi_lisomani(int x_l, int x_c, int x_e, int j, int jprev)
{
	if(pi_lis[x_l][x_c][x_e][j][jprev] == -1.0)
		pi_lis[x_l][x_c][x_e][j][jprev] =
		(powering(lambdacjk[jprev][j], x_c) / factorial(x_c)) *
		(powering(lambdaj[jprev] - lambdacjk[jprev][j], x_l) / factorial(x_l)) *
		(powering(lambdaj[j] - lambdacjk[jprev][j], x_e) / factorial(x_e)) /
		pidenom[jprev][j];
	
	return pi_lis[x_l][x_c][x_e][j][jprev];
}



/*-----------------------------------------------------------------------------------*/
/*
double fbinom(int n, int k): n alatt a k binominalis egyutthatot adja vissza
*/

double fbinom(int n, int k)
{
	if(n<0 || k<0 || n<k)
		exit(1);
	
    return bino[n][k];
}

/*-----------------------------------------------------------------------------------*/
/*
void createbinom(int maxn): n alatt a k binominalis egyutthatok kiszamitasa.
Egyszerubb volt double ertekekkel szamolni es az eredmenyt ugyis elosztjuk majd
egy masik szammal (lasd r fuggvenyt)
*/

void createbinom(int maxn)
{
    int i,k,n;
    double p;
	bino = (double **)MMmalloc(sizeof(double*)*maxn, "createbinom");
	for(n=0; n<maxn+1; n++)
	{
		bino[n] = (double  *)MMmalloc(sizeof(double)*(n+1), "createbinom");
		for(k=0; k<(n+2)/2; k++)
		{
			p = 1.0;
			for(i=1; i<k+1; i++) {p *= ((n-k+i) * 1.0) / (i * 1.0);}
			bino[n][k] = p;
			bino[n][n-k] = p;
		}
	}
}
/*-----------------------------------------------------------------------------------*/
/*
double factorial(int n): n faktorialist adja vissza
*/

double factorial(int n)
{
    return facto[n];
}

/*-----------------------------------------------------------------------------------*/
/*
void createfacto(int maxn): n faktorialis ertekei
*/

void createfacto(int maxn)
{
    int n, i;
	facto = (double *)MMmalloc(sizeof(double)*(maxn+1), "createfacto");
	for(n=0; n<maxn+1; n++)
	{
		facto[n] = 1.0;
		for(i=2; i<n+1; i++) facto[n] *= i;
	}
}
